var metaBoxOrderShipments = new Vue({
    el: '#wcdpi-order-shipment-meta-box-content',
    data: {
        loading: true,
        singleShipmentUrl: null,
        orderId: null,
        active: {
            tab: -1,
            shipments: []
        },
        tabs: {
            overview: -1,
            prepared: 0,
            readyToShip: 1,
            shipped: 2,
            delivered: 3,
            refunded: 4,
            returned: 5
        },
        shipments: {
            all: [],
            totals: {
                0: null,
                1: null,
                2: null,
                3: null,
                4: null,
                5: null
            }
        },
        display: {
            currencySymbol: '&euro;',
            weightUnit: 'g',
            decimalSeparator: ',',
            thousandSeparator: '.',
            decimals: 2,
            priceFormat: null
        }
    },
    methods: {
        init: function() {
            this.active.tab = this.tabs.overview;
            let orderId = this.orderId;

            jQuery.ajax({
                url: ajaxurl,
                type: 'POST',
                dataType: 'JSON',
                data: {
                    action: 'wcdpi_prepare_shipment_meta_box',
                    orderId: orderId
                },
                success: function(data) {
                    metaBoxOrderShipments.shipments.all = data.shipments;
                    metaBoxOrderShipments.currencySymbol = data.currencySymbol;
                    metaBoxOrderShipments.decimalSeparator = data.decimalSeparator;
                    metaBoxOrderShipments.thousandSeparator = data.thousandSeparator;
                    metaBoxOrderShipments.decimals = data.decimals;
                    metaBoxOrderShipments.priceFormat = data.priceFormat;
                },
                error: function(response) {
                    console.log(response);
                },
                complete: function() {
                    metaBoxOrderShipments.loading = false;
                }
            });

        },
        openLabelWizard: function() {
            let wizard = document.querySelector('#wcdpi-label-wizard');

            wizard.classList.add('open');
        },
        openShipmentModal: function(event, shipmentId) {
            event.preventDefault();

            let shipmentModal = document.querySelector('#wcdpi-shipment-modal');

            console.log('Shipment ID: ' + shipmentId);
            console.log('Set some variables');
            console.log('Open Shipment Modal');

            if (shipmentModal) {
                console.log(shipmentModal);
            }
        },
        changeTab: function(newTab) {
            metaBoxOrderShipments.active.tab = newTab;

            if (newTab === metaBoxOrderShipments.tabs.overview) {
                return;
            }

            metaBoxOrderShipments.active.shipments = metaBoxOrderShipments.shipments.all.filter(function (shipment) {
                return parseInt(shipment.status) === metaBoxOrderShipments.active.tab;
            });

            for (let i = 0; i < metaBoxOrderShipments.active.shipments; i++) {
                let shipment = metaBoxOrderShipments.active.shipments[i];

                shipment.checked = false;
            }
        },
        formatPrice: function(value, currencySymbol) {
            let amount       = parseFloat(value);
            let thousands    = this.display.thousandSeparator;
            let decimal      = this.display.decimalSeparator;
            let decimalCount = Math.abs(this.display.decimals);
            currencySymbol   = currencySymbol ? currencySymbol : this.display.currencySymbol;
            decimalCount     = isNaN(decimalCount) ? 2 : decimalCount;

            const negativeSign = amount < 0 ? '-' : '';

            let i = parseInt(amount = Math.abs(Number(amount) || 0).toFixed(decimalCount)).toString();
            let j = (i.length > 3) ? i.length % 3 : 0;

            return negativeSign + (j ? i.substr(0, j) + thousands : '') + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + thousands) + (decimalCount ? decimal + Math.abs(amount - i).toFixed(decimalCount).slice(2) : "") + ' ' + currencySymbol;
        },
        formatWeight: function(value) {
            if (this.display.weightUnit !== 'g') {
                value = this.convertWeight(value, this.display.weightUnit);
            }

            return this.formatPrice(value, this.display.weightUnit);
        },
        convertWeight: function(weight, unit) {
            // Prepare factor for calculation
            let factor = 1;

            switch (unit) {
                case 'kg':
                    factor = 1000; // 1 kg  = 1000 g
                    break;
                case 'lbs':
                    factor = 453.5924; // 1 lbs = 453.5924 g
                    break;
                case 'oz':
                    factor = 28.34952; // 1 oz  = 28.34952 g
                    break;
                case 'g':
                    factor = 1; // 1 g = 1 g
                    break;
                default:
                    factor = 1;
                    break;
            }

            // Calculation
            return parseFloat(weight) / factor;
        },
        getTrackingLink: function(trackingNumber, date) {
            date = new Date(date);

            if (!date) {
                return trackingNumber;
            }

            let D = date.getDay();
            let M = date.getMonth();
            let Y = date.getFullYear();

            let url = 'https://www.deutschepost.de/sendung/simpleQueryResult.html?form.sendungsnummer='
                + trackingNumber;
                //+ '&form.einlieferungsdatum_tag='
                //+ D
                //+ '&form.einlieferungsdatum_monat='
                //+ M
                //+ '&form.einlieferungsdatum_jahr='
                //+ Y;

            return '<a target="_blank" rel="noopener" href="' + url + '">' + trackingNumber + '</a>';
        }
    },
    mounted: function() {
        var orderId = parseInt(document.querySelector('#wcdpi-order-shipment-meta-box-content-order-id').value);
        this.orderId = orderId;
        this.init();
    }
});