var app = new Vue({
    el: '.wcdpi-modal',
    data: {
        orderId: null,
        loading: false,
        steps: {
            current: 1,
            max: 3
        },
        form: {
            sender: {
                firstName: null,
                lastName: null,
                company: null,
                email: null,
                phone: null,
                street: null,
                houseNumber: null,
                additional: null,
                city: null,
                zip: null,
                country: null
            },
            recipient: {
                firstName: null,
                lastName: null,
                company: null,
                email: null,
                phone: null,
                street: null,
                houseNumber: null,
                additional: null,
                city: null,
                zip: null,
                country: null,
                state: null,
            }
        },
        list: {
            menu: [],
            product: null,
            options: null,
            lineItems: []
        },
        active: {
            prodwsId: null,
            harmonizedLabel: false,
            previewImageUrl: null,
            shippingCategory: null,
            shippingProduct: null,
            shippingOption: null,
            shipmentAction: null,
            confirmationDialog: null,
        },
        eu: {
            countries: [
                'AT', 'BE', 'BG', 'HR', 'CY', 'CZ', 'DK', 'EE', 'FI', 'FR', 'DE', 'GR', 'HU', 'IE',
                'IT', 'LV', 'LT', 'LU', 'MT', 'NL', 'PL', 'PT', 'RO', 'SK', 'SI', 'ES', 'SE',
            ],
            shipment: false
        },
        total: {
            weight: 0,
            value: 0
        },
        display: {
            weightUnit: '',
            price: '0,00 €',
            currencySymbol: null,
            decimalSeparator: null,
            thousandSeparator: null,
            decimals: null,
            priceFormat: null,
            dimensions: null,
            info: null
        },
        error: {
            labelNotFound: false
        }
    },
    methods: {
        init: async function(orderId) {
            jQuery.ajax({
                url: ajaxurl,
                type: 'POST',
                dataType: 'JSON',
                data: {
                    action: 'wcdpi_prepare_shipment_form',
                    orderId: orderId
                },
                success: function(data) {
                    app.form.sender = data.sender;
                    app.form.recipient = data.recipient;
                    app.list.menu = data.menuList;
                    app.list.product = data.productList;
                    app.list.options = data.optionsList;
                    app.list.lineItems = data.lineItems;
                    app.display.currencySymbol = data.currencySymbol;
                    app.display.decimalSeparator = data.decimalSeparator;
                    app.display.thousandSeparator = data.thousandSeparator;
                    app.display.decimals = data.decimals;
                    app.display.priceFormat = data.priceFormat;
                    app.display.price = app.formatPrice(0, '&euro;');
                    app.display.weightUnit = data.weightUnit;
                },
                error: function(response) {
                    console.error(response);
                },
                complete: function() {
                }
            });
        },
        close: function() {
            this.$el.classList.remove('open');
        },
        nextStep: function() {
            if (this.inputValidation()) {
                ++app.steps.current
            }
        },
        previousStep: function() {
            --app.steps.current;
        },
        getStepClass: function(step) {
            if (step === this.steps.current) {
                return 'active';
            }

            if (step > this.steps.current) {
                return 'pending';
            }

            if (step < this.steps.current) {
                return 'complete';
            }
        },
        inputValidation: function() {
            return true;
        },
        isEuropeanUnionShipment: function() {
            return this.eu.countries.indexOf(this.form.recipient.country) !== -1;
        },
        shouldShippingGroupBeVisible: function(slug) {
            if (this.form.recipient.country === 'DE') {
                return slug === 'national' || slug === 'national-special';
            }

            if (this.form.recipient.country !== 'DE') {
                return slug === 'international' || slug === 'international-business';
            }

            return false;
        },
        shouldShippingCategoryBeVisible: function(slug) {
            if (slug === 'warenposteu' && !this.isEuropeanUnionShipment()) {
                return false;
            }

            if (slug === 'warenpostnoneu' && this.isEuropeanUnionShipment()) {
                return false;
            }

            return true;
        },
        changeShippingCategory: function(slug) {
            this.active.shippingCategory = slug;
        },
        changeShippingProduct: function(prodwsId) {
            app.error.labelNotFound = false;

            this.loadProductInfo(prodwsId)
                .then(function(data) {
                    app.active.shippingProduct = prodwsId;
                    app.active.harmonizedLabel = false;
                })
                .catch(function(error) {
                    app.error.labelNotFound = true;
                });
        },
        changeShippingOption: function(prodwsId) {
            this.loadProductInfo(prodwsId)
                .then(function(data) {
                })
                .catch(function(error) {
                    console.error(error);
                });
        },
        loadProductInfo: function(prodwsId) {
            this.loading = true;

            return new Promise(function(resolve, reject) {
                jQuery.ajax({
                    url: ajaxurl,
                    type: 'POST',
                    dataType: 'JSON',
                    data: {
                        action: 'wcdpi_preview_label',
                        amount: 1,
                        prodwsId: prodwsId,
                        harmonizedLabel: app.active.harmonizedLabel
                    },
                    success: function(data) {
                        app.active.previewImageUrl = data.url;
                        app.active.prodwsId        = prodwsId;
                        app.display.price          = data.total;
                        app.display.dimensions     = data.dimensions;
                        app.display.info           = data.info;
                        resolve(data);
                    },
                    error: function(error) {
                        reject(error);
                    },
                    complete: function() {
                        app.loading = false;
                    }
                });
            });
        },
        isHarmonizedLabelPossible: function() {
            let allowedShippingGroups = ['warenposteu', 'warenpostnoneu', 'warenpostkilotarif'];

            return allowedShippingGroups.indexOf(this.active.shippingCategory) !== -1;
        },
        toggleHarmonizedLabel: function() {
            this.loadProductInfo(this.active.prodwsId);
        },
        getShipmentWeight: function() {
            let weight = 0;

            for (let i = 0; i < this.list.lineItems.length; i++) {
                weight += (this.list.lineItems[i].qtySelected * this.list.lineItems[i].weight);
            }

            return weight;
        },
        getShipmentValue: function() {
            let value = 0;

            for (let i = 0; i < this.list.lineItems.length; i++) {
                value += (this.list.lineItems[i].qtySelected * this.list.lineItems[i].price);
            }

            return value;
        },
        showConfirmationDialog: function() {
            this.active.confirmationDialog = true;
        },
        formatPrice: function(value, currencySymbol) {
            let amount       = parseFloat(value);
            let thousands    = this.display.thousandSeparator;
            let decimal      = this.display.decimalSeparator;
            let decimalCount = Math.abs(this.display.decimals);
            currencySymbol   = currencySymbol ? currencySymbol : this.display.currencySymbol;
            decimalCount     = isNaN(decimalCount) ? 2 : decimalCount;

            const negativeSign = amount < 0 ? '-' : '';

            let i = parseInt(amount = Math.abs(Number(amount) || 0).toFixed(decimalCount)).toString();
            let j = (i.length > 3) ? i.length % 3 : 0;

            return negativeSign + (j ? i.substr(0, j) + thousands : '') + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + thousands) + (decimalCount ? decimal + Math.abs(amount - i).toFixed(decimalCount).slice(2) : "") + ' ' + currencySymbol;
        },
        submit: function(shipmentAction) {
            this.active.shipmentAction = shipmentAction;

            Vue.nextTick(function() {
                app.$refs.labelWizardForm.submit();
            });
        }
    },
    mounted: function() {
        let orderIdElement = document.querySelector('#post_ID');
        let orderId = null;

        if (orderIdElement) {
            orderId = orderIdElement.value;
        }

        this.init(orderId);
    }
});