jQuery(function($) {
  'use strict';

  const userDropdownSearchConfig = {
    ajax: {
      cache: true,
      delay: 500,
      url: ajaxurl,
      method: 'POST',
      dataType: 'json',
      data: function(params) {
        return {
          action: 'wcdpi_dropdown_user_search',
          security: security.userSearch,
          term: params.term,
          page: params.page,
          type: 'user'
        };
      },
      processResults: function(data, params) {
        params.page = params.page || 1;

        return {
          results: data.results,
          pagination: {
            more: data.pagination.more
          }
        };
      }
    },
    width: '100%',
    selectionCssClass: 'wcdpi-select2-selection',
    dropdownCssClass: 'wcdpi-select2-dropdown',
    placeholder: i18n.placeholder.userSearch,
    minimumInputLength: 1,
    allowClear: true
  };
  const productDropdownSearchConfig = {
    ajax: {
      cache: true,
      delay: 500,
      url: ajaxurl,
      method: 'GET',
      dataType: 'json',
      data: function(params) {
        return {
          action: 'woocommerce_json_search_products_and_variations',
          security: security.productSearch,
          term: params.term
        };
      },
      processResults: function(data) {
        let terms = [];

        if (data) {
          $.each(data, function(id, text) {
            terms.push({ id: id, text: text });
          });
        }

        return {
          results: terms
        };
      }
    },
    width: '100%',
    selectionCssClass: 'wcdpi-select2-selection',
    dropdownCssClass: 'wcdpi-select2-dropdown',
    placeholder: i18n.placeholder.productSearch,
    minimumInputLength: 1,
    allowClear: true
  };

  const internetmarkeLabelShop = new Vue({
    el: '#wcdpi-label-shop',
    data: {
      loading: false,
      steps: {
        current: 1,
        max: 3
      },
      form: {
        sender: {
          firstName: null,
          lastName: null,
          company: null,
          email: null,
          phone: null,
          street: null,
          houseNumber: null,
          additional: null,
          city: null,
          zip: null,
          country: 'DE'
        },
        recipient: {
          firstName: null,
          lastName: null,
          company: null,
          email: null,
          phone: null,
          street: null,
          houseNumber: null,
          additional: null,
          city: null,
          zip: null,
          country: 'DE',
          state: null,
        }
      },
      list: {
        menu: [],
        product: null,
        options: null,
        lineItems: []
      },
      active: {
        prodwsId: null,
        harmonizedLabel: false,
        previewImageUrl: null,
        shippingCategory: null,
        shippingProduct: null,
        shippingOption: null,
        shipmentAction: null,
        confirmationDialog: null,
      },
      eu: {
        countries: [],
        shipment: false
      },
      total: {
        weight: 0,
        value: 0
      },
      userId: {
        sender: null,
        recipient: null
      },
      display: {
        weightUnit: '',
        price: '0,00 €',
        currencySymbol: null,
        decimalSeparator: null,
        thousandSeparator: null,
        decimals: null,
        priceFormat: null,
        dimensions: null,
        info: null
      },
      error: {
        labelNotFound: false
      }
    },
    methods: {
      init: async function() {
        $.ajax({
          url: ajaxurl,
          type: 'POST',
          dataType: 'JSON',
          data: {
            action: 'wcdpi_label_shop_prepare_form'
          },
          success: function(data) {
            internetmarkeLabelShop.eu.countries = data.euCountries;
            internetmarkeLabelShop.list.menu = data.menuList;
            internetmarkeLabelShop.list.product = data.productList;
            internetmarkeLabelShop.list.options = data.optionsList;
            internetmarkeLabelShop.list.lineItems = data.lineItems;
            internetmarkeLabelShop.display.currencySymbol = data.currencySymbol;
            internetmarkeLabelShop.display.decimalSeparator = data.decimalSeparator;
            internetmarkeLabelShop.display.thousandSeparator = data.thousandSeparator;
            internetmarkeLabelShop.display.decimals = data.decimals;
            internetmarkeLabelShop.display.priceFormat = data.priceFormat;
            internetmarkeLabelShop.display.price = internetmarkeLabelShop.formatPrice(0, '&euro;');
            internetmarkeLabelShop.display.weightUnit = data.weightUnit;
          },
          error: function(response) {
            console.error(response);
          }
        });
      },
      nextStep: function() {
        if (this.inputValidation()) {
          ++this.steps.current
        }

        if (this.steps.current === 3) {
          const searchProduct = $('.wcdpi-search-product');
          const fnSearchProduct = function(e) {
            let productId = null;

            if (e.target.selectedIndex !== -1) {
              productId = parseInt(e.target[e.target.selectedIndex].value);
            }

            internetmarkeLabelShop.addLineItem(productId);
          };

          if (searchProduct) {
            searchProduct.select2(productDropdownSearchConfig);
            searchProduct.off('change');
            searchProduct.on('change', fnSearchProduct);
          }
        }
      },
      previousStep: function() {
        --this.steps.current;
      },
      getStepClass: function(step) {
        return {
          active: step === this.steps.current,
          pending: step > this.steps.current,
          complete: step < this.steps.current
        }
      },
      inputValidation: function() {
        return true;
      },
      isEuropeanUnionShipment: function() {
        return this.eu.countries.indexOf(this.form.recipient.country) !== -1;
      },
      shouldShippingGroupBeVisible: function(slug) {
        if (this.form.recipient.country === 'DE') {
          return slug === 'national' || slug === 'national-special';
        }

        if (this.form.recipient.country !== 'DE') {
          return slug === 'international' || slug === 'international-business';
        }

        return false;
      },
      shouldShippingCategoryBeVisible: function(slug) {
        if (slug === 'warenposteu' && !this.isEuropeanUnionShipment()) {
          return false;
        }

        return !(slug === 'warenpostnoneu' && this.isEuropeanUnionShipment());
      },
      changeShippingCategory: function(slug) {
        this.active.shippingCategory = slug;
      },
      changeShippingProduct: function(prodwsId) {
        this.error.labelNotFound = false;
        const that = this;

        this.loadProductInfo(prodwsId)
          .then(function() {
            that.active.shippingProduct = prodwsId;
            that.active.harmonizedLabel = false;
          })
          .catch(function() {
            that.error.labelNotFound = true;
          });
      },
      changeShippingOption: function(prodwsId) {
        this.loadProductInfo(prodwsId)
          .then(function(data) {
          })
          .catch(function(error) {
            console.error(error);
          });
      },
      loadProductInfo: function(prodwsId) {
        this.loading = true;
        const that = this;

        return new Promise(function(resolve, reject) {
          $.ajax({
            url: ajaxurl,
            type: 'POST',
            dataType: 'JSON',
            data: {
              action: 'wcdpi_preview_label',
              amount: 1,
              prodwsId: prodwsId,
              harmonizedLabel: that.active.harmonizedLabel
            },
            success: function(data) {
              that.active.previewImageUrl = data.url;
              that.active.prodwsId        = prodwsId;
              that.display.price          = data.total;
              that.display.dimensions     = data.dimensions;
              that.display.info           = data.info;
              resolve(data);
            },
            error: function(error) {
              reject(error);
            },
            complete: function() {
              that.loading = false;
            }
          });
        });
      },
      isHarmonizedLabelPossible: function() {
        let allowedShippingGroups = ['warenposteu', 'warenpostnoneu', 'warenpostkilotarif'];

        return allowedShippingGroups.indexOf(this.active.shippingCategory) !== -1;
      },
      toggleHarmonizedLabel: function() {
        this.loadProductInfo(this.active.prodwsId).then();
      },
      getShipmentWeight: function() {
        let weight = 0;

        for (let i = 0; i < this.list.lineItems.length; i++) {
          weight += (this.list.lineItems[i].qtySelected * this.list.lineItems[i].weight);
        }

        return weight;
      },
      getShipmentValue: function() {
        let value = 0;

        for (let i = 0; i < this.list.lineItems.length; i++) {
          value += (this.list.lineItems[i].qtySelected * this.list.lineItems[i].price);
        }

        return value;
      },
      showConfirmationDialog: function() {
        this.active.confirmationDialog = true;
      },
      formatPrice: function(value, currencySymbol) {
        let amount       = parseFloat(value);
        let thousands    = this.display.thousandSeparator;
        let decimal      = this.display.decimalSeparator;
        let decimalCount = Math.abs(this.display.decimals);
        currencySymbol   = currencySymbol ? currencySymbol : this.display.currencySymbol;
        decimalCount     = isNaN(decimalCount) ? 2 : decimalCount;

        const negativeSign = amount < 0 ? '-' : '';

        let i = parseInt(amount = Math.abs(Number(amount) || 0).toFixed(decimalCount)).toString();
        let j = (i.length > 3) ? i.length % 3 : 0;

        return negativeSign + (j ? i.substr(0, j) + thousands : '') + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + thousands) + (decimalCount ? decimal + Math.abs(amount - i).toFixed(decimalCount).slice(2) : "") + ' ' + currencySymbol;
      },
      clearAddress: function(target) {
        this.form[target].firstName = null;
        this.form[target].lastName = null;
        this.form[target].company = null;
        this.form[target].email = null;
        this.form[target].phone = null;
        this.form[target].street = null;
        this.form[target].houseNumber = null;
        this.form[target].additional = null;
        this.form[target].city = null;
        this.form[target].zip = null;
        this.form[target].state = null;

        if (target === 'sender') {
          this.form[target].country = 'DE';
        } else {
          this.form[target].country = null;
        }
      },
      loadDefaultSender: function(target) {
        this.loading = true;
        const that = this;

        $.ajax({
          url: ajaxurl,
          type: 'POST',
          dataType: 'JSON',
          data: {
            action: 'wcdpi_label_shop_load_default_sender'
          },
          success: function(data) {
            that.clearAddress(target);
            that.form[target].firstName = data.firstName;
            that.form[target].lastName = data.lastName;
            that.form[target].company = data.company;
            that.form[target].email = data.email;
            that.form[target].phone = data.phone;
            that.form[target].street = data.street;
            that.form[target].houseNumber = data.houseNumber;
            that.form[target].additional = data.additional;
            that.form[target].city = data.city;
            that.form[target].zip = data.zip;
            that.form[target].state = data.state;
            that.form[target].country = data.country;
          },
          error: function(response) {
            console.error(response);
          },
          complete: function() {
            that.loading = false;
          }
        })
      },
      loadCustomerAddress: function(target, addressType) {
        this.loading = true;
        const that = this;

        $.ajax({
          url: ajaxurl,
          type: 'POST',
          dataType: 'JSON',
          data: {
            action: 'wcdpi_label_shop_load_customer_address',
            addressType: addressType,
            userId: that.userId[target]
          },
          success: function(data) {
            that.clearAddress(target);
            that.form[target].firstName = data.firstName;
            that.form[target].lastName = data.lastName;
            that.form[target].company = data.company;
            that.form[target].email = data.email;
            that.form[target].phone = data.phone;
            that.form[target].street = data.street;
            that.form[target].houseNumber = data.houseNumber;
            that.form[target].additional = data.additional;
            that.form[target].city = data.city;
            that.form[target].zip = data.zip;
            that.form[target].state = data.state;

            if (target === 'sender') {
              that.form[target].country = 'DE';
            } else {
              that.form[target].country = data.country;
            }
          },
          error: function(response) {
            console.error(response);
          },
          complete: function() {
            that.loading = false;
          }
        })
      },
      addLineItem: function(productId) {
        if (productId === null) {
          return;
        }

        const lineItem = this.list.lineItems.filter(function(lineItem) {
          return lineItem.productId === productId;
        });

        if (lineItem.length > 0) {
          return;
        }

        this.loading = true;

        this.getProductDetails(productId)
          .then(function(lineItem) {
            internetmarkeLabelShop.list.lineItems.push(lineItem);
            internetmarkeLabelShop.loading = false;
          })
          .catch(function(error) {
            console.error(error);
            internetmarkeLabelShop.loading = false;
          });
      },
      removeLineItem: function(productId) {
        for (let i = 0; i < this.list.lineItems.length; i++) {
          const lineItem = this.list.lineItems[i];

          if (lineItem.productId === productId) {
            this.list.lineItems.splice(i, 1);
            break;
          }
        }
      },
      canSubmitShipment: function() {
        let result = true;

        if (this.active.harmonizedLabel && (this.list.lineItems.length < 1 || this.list.lineItems.length > 5)) {
          result = false;
        }

        return result;
      },
      getProductDetails: function(productId) {
        return new Promise(function (resolve, reject) {
          $.ajax({
            url: ajaxurl,
            type: 'POST',
            dataType: 'JSON',
            data: {
              action: 'wcdpi_label_shop_add_line_item',
              security: security.addLineItem,
              productId: productId
            },
            success: function(data) {
              resolve(data);
            },
            error: function(error) {
              reject(error);
            }
          });
        });
      }
    },
    mounted: function() {
      this.init();
      const searchCustomer = $('.wcdpi-search-customer');

      if (searchCustomer) {
        searchCustomer.select2(userDropdownSearchConfig);
        searchCustomer.on('change', function(e) {
          if (e.target.selectedIndex === -1) {
            internetmarkeLabelShop.userId[this.dataset.target] = null;
          } else {
            internetmarkeLabelShop.userId[this.dataset.target] = e.target[e.target.selectedIndex].value;
          }
        });
      }
    }
  });
});

