var pageShipmentsBulkCreate = new Vue({
  el: '#wcdpi-shipments-bulk-create',
  data: {
    loading: true,
    mode: 'single',
    confirmationDialog: false,
    progressDialog: false,
    progressMax: 4,
    progressValue: 10,
    shippingLabel: null,
    harmonizedLabel: false,
    label: {
      previewUrl: '',
      info: null,
      dimensions: null,
      total: null,
    },
    orders: [],
    progress: [],
    currentOrder: false
  },
  methods: {
    onMounted: function(orders) {
      jQuery.ajax({
        url: ajaxurl,
        type: 'POST',
        dataType: 'JSON',
        data: {
          action: 'wcdpi_init_single_mode',
          orders: orders
        },
        success: function(data) {
          pageShipmentsBulkCreate.orders = data;
        },
        error: function(response) {
          console.log(response);
        }
      });
    },
    onChangeLabel: function() {
      pageShipmentsBulkCreate.loading = true;

      jQuery.ajax({
        url: ajaxurl,
        type: 'POST',
        dataType: 'JSON',
        data: {
          action: 'wcdpi_preview_label',
          amount: pageShipmentsBulkCreate.orders.length,
          prodwsId: pageShipmentsBulkCreate.shippingLabel,
          harmonizedLabel: pageShipmentsBulkCreate.harmonizedLabel
        },
        success: function(data) {
          pageShipmentsBulkCreate.label.dimensions = data.dimensions;
          pageShipmentsBulkCreate.label.info = data.info;
          pageShipmentsBulkCreate.label.total = data.total;
          pageShipmentsBulkCreate.label.previewUrl = data.url;
        },
        error: function(response) {
          console.log(response);
        },
        complete: function() {
          pageShipmentsBulkCreate.loading = false;
        }
      });
    },
    onSubmit: async function(action) {
      pageShipmentsBulkCreate.confirmationDialog = false;
      pageShipmentsBulkCreate.progressDialog = true;
      pageShipmentsBulkCreate.progressValue = 0;
      pageShipmentsBulkCreate.progress = [];

      for (var i = 0; i < pageShipmentsBulkCreate.orders.length; i++) {
        pageShipmentsBulkCreate.currentorder = false;
        var order = pageShipmentsBulkCreate.orders[i];
        pageShipmentsBulkCreate.currentorder = order;

        order.status = 'progress';

        try {
          var shipment = await pageShipmentsBulkCreate.shipmentSubmit(
            action,
            order,
            pageShipmentsBulkCreate.shippingLabel,
            pageShipmentsBulkCreate.harmonizedLabel
          );

          order.status = 'success';
          order.shippingLabelUrl = shipment.shippingLabelUrl;
        } catch (error) {
          order.details = error.responseText;
          order.status = 'error';
        }

        pageShipmentsBulkCreate.progressValue += 1;
      }
    },
    
    // Shipments
    shipmentSubmit: async function(action, order, prodwsId, harmonizedLabel) {
      if (action !== 'prepare' && action !== 'create') {
        console.error('Internetmarke - Invalid form action.');
        return;
      }

      return new Promise(function (resolve, reject) {
        jQuery.ajax({
          url: ajaxurl,
          type: 'POST',
          dataType: 'JSON',
          data: {
            action: 'wcdpi_shipment_submit',
            type: action,
            orderId: order.id,
            orderNumber: order.orderNumber,
            prodwsId: prodwsId,
            harmonizedLabel: harmonizedLabel
          },
          success: function(result) {
            console.log(result);
            resolve(result);
          },
          error: function(error) {
            console.error(error);
            reject(error);
          },
          complete: function() {
            pageShipmentsBulkCreate.loading = false;
          }
        });

      });
    }
  },
  mounted: function() {
    var orders = jQuery('input#wcdpi-orders');

    this.onMounted(JSON.parse(orders.val()));

    this.loading = false;

    var selectShippingCarrier = jQuery('select#wcdpi-shipping-carrier');
    var selectShippingLabel   = jQuery('select#wcdpi-shipping-label');

    if (selectShippingCarrier) {
      selectShippingCarrier.select2();
    }

    if (selectShippingLabel) {
      selectShippingLabel.select2({
        placeholder: '- ' + i18n.placeholder + ' -',
        selectionCssClass: 'wcdpi-select2-selection',
        dropdownCssClass: 'wcdpi-select2-dropdown',
        width: '100%'
      });
      selectShippingLabel.on('change', function(e) {
        pageShipmentsBulkCreate.shippingLabel = e.target[e.target.selectedIndex].value;
        pageShipmentsBulkCreate.onChangeLabel();
      })
    }
  }
});