<?php

use DeutschePost\Internetmarke\Enums\ShipmentStatus;
use DeutschePost\Internetmarke\Enums\ShippingRegion as ShippingRegionEnum;
use DeutschePost\Internetmarke\Models\Resources\ProductSales as ProductSalesResourceModel;
use DeutschePost\Internetmarke\Products\SalesProductList;
use DeutschePost\Internetmarke\Repositories\Resources\ProductSales as ProductSalesResourceRepository;

/**
 * @param string $iso2
 *
 * @return bool|string
 */
function wcdpi_convert_country_iso2_to_iso3($iso2) {
    $countries = array(
        'AF' => 'AFG', //Afghanistan
        'AX' => 'ALA', //&#197;land Islands
        'AL' => 'ALB', //Albania
        'DZ' => 'DZA', //Algeria
        'AS' => 'ASM', //American Samoa
        'AD' => 'AND', //Andorra
        'AO' => 'AGO', //Angola
        'AI' => 'AIA', //Anguilla
        'AQ' => 'ATA', //Antarctica
        'AG' => 'ATG', //Antigua and Barbuda
        'AR' => 'ARG', //Argentina
        'AM' => 'ARM', //Armenia
        'AW' => 'ABW', //Aruba
        'AU' => 'AUS', //Australia
        'AT' => 'AUT', //Austria
        'AZ' => 'AZE', //Azerbaijan
        'BS' => 'BHS', //Bahamas
        'BH' => 'BHR', //Bahrain
        'BD' => 'BGD', //Bangladesh
        'BB' => 'BRB', //Barbados
        'BY' => 'BLR', //Belarus
        'BE' => 'BEL', //Belgium
        'BZ' => 'BLZ', //Belize
        'BJ' => 'BEN', //Benin
        'BM' => 'BMU', //Bermuda
        'BT' => 'BTN', //Bhutan
        'BO' => 'BOL', //Bolivia
        'BQ' => 'BES', //Bonaire, Saint Estatius and Saba
        'BA' => 'BIH', //Bosnia and Herzegovina
        'BW' => 'BWA', //Botswana
        'BV' => 'BVT', //Bouvet Islands
        'BR' => 'BRA', //Brazil
        'IO' => 'IOT', //British Indian Ocean Territory
        'BN' => 'BRN', //Brunei
        'BG' => 'BGR', //Bulgaria
        'BF' => 'BFA', //Burkina Faso
        'BI' => 'BDI', //Burundi
        'KH' => 'KHM', //Cambodia
        'CM' => 'CMR', //Cameroon
        'CA' => 'CAN', //Canada
        'CV' => 'CPV', //Cape Verde
        'KY' => 'CYM', //Cayman Islands
        'CF' => 'CAF', //Central African Republic
        'TD' => 'TCD', //Chad
        'CL' => 'CHL', //Chile
        'CN' => 'CHN', //China
        'CX' => 'CXR', //Christmas Island
        'CC' => 'CCK', //Cocos (Keeling) Islands
        'CO' => 'COL', //Colombia
        'KM' => 'COM', //Comoros
        'CG' => 'COG', //Congo
        'CD' => 'COD', //Congo, Democratic Republic of the
        'CK' => 'COK', //Cook Islands
        'CR' => 'CRI', //Costa Rica
        'CI' => 'CIV', //Côte d\'Ivoire
        'HR' => 'HRV', //Croatia
        'CU' => 'CUB', //Cuba
        'CW' => 'CUW', //Curaçao
        'CY' => 'CYP', //Cyprus
        'CZ' => 'CZE', //Czech Republic
        'DK' => 'DNK', //Denmark
        'DJ' => 'DJI', //Djibouti
        'DM' => 'DMA', //Dominica
        'DO' => 'DOM', //Dominican Republic
        'EC' => 'ECU', //Ecuador
        'EG' => 'EGY', //Egypt
        'SV' => 'SLV', //El Salvador
        'GQ' => 'GNQ', //Equatorial Guinea
        'ER' => 'ERI', //Eritrea
        'EE' => 'EST', //Estonia
        'ET' => 'ETH', //Ethiopia
        'FK' => 'FLK', //Falkland Islands
        'FO' => 'FRO', //Faroe Islands
        'FJ' => 'FIJ', //Fiji
        'FI' => 'FIN', //Finland
        'FR' => 'FRA', //France
        'GF' => 'GUF', //French Guiana
        'PF' => 'PYF', //French Polynesia
        'TF' => 'ATF', //French Southern Territories
        'GA' => 'GAB', //Gabon
        'GM' => 'GMB', //Gambia
        'GE' => 'GEO', //Georgia
        'DE' => 'DEU', //Germany
        'GH' => 'GHA', //Ghana
        'GI' => 'GIB', //Gibraltar
        'GR' => 'GRC', //Greece
        'GL' => 'GRL', //Greenland
        'GD' => 'GRD', //Grenada
        'GP' => 'GLP', //Guadeloupe
        'GU' => 'GUM', //Guam
        'GT' => 'GTM', //Guatemala
        'GG' => 'GGY', //Guernsey
        'GN' => 'GIN', //Guinea
        'GW' => 'GNB', //Guinea-Bissau
        'GY' => 'GUY', //Guyana
        'HT' => 'HTI', //Haiti
        'HM' => 'HMD', //Heard Island and McDonald Islands
        'VA' => 'VAT', //Holy See (Vatican City State)
        'HN' => 'HND', //Honduras
        'HK' => 'HKG', //Hong Kong
        'HU' => 'HUN', //Hungary
        'IS' => 'ISL', //Iceland
        'IN' => 'IND', //India
        'ID' => 'IDN', //Indonesia
        'IR' => 'IRN', //Iran
        'IQ' => 'IRQ', //Iraq
        'IE' => 'IRL', //Republic of Ireland
        'IM' => 'IMN', //Isle of Man
        'IL' => 'ISR', //Israel
        'IT' => 'ITA', //Italy
        'JM' => 'JAM', //Jamaica
        'JP' => 'JPN', //Japan
        'JE' => 'JEY', //Jersey
        'JO' => 'JOR', //Jordan
        'KZ' => 'KAZ', //Kazakhstan
        'KE' => 'KEN', //Kenya
        'KI' => 'KIR', //Kiribati
        'KP' => 'PRK', //Korea, Democratic People\'s Republic of
        'KR' => 'KOR', //Korea, Republic of (South)
        'KW' => 'KWT', //Kuwait
        'KG' => 'KGZ', //Kyrgyzstan
        'LA' => 'LAO', //Laos
        'LV' => 'LVA', //Latvia
        'LB' => 'LBN', //Lebanon
        'LS' => 'LSO', //Lesotho
        'LR' => 'LBR', //Liberia
        'LY' => 'LBY', //Libya
        'LI' => 'LIE', //Liechtenstein
        'LT' => 'LTU', //Lithuania
        'LU' => 'LUX', //Luxembourg
        'MO' => 'MAC', //Macao S.A.R., China
        'MK' => 'MKD', //Macedonia
        'MG' => 'MDG', //Madagascar
        'MW' => 'MWI', //Malawi
        'MY' => 'MYS', //Malaysia
        'MV' => 'MDV', //Maldives
        'ML' => 'MLI', //Mali
        'MT' => 'MLT', //Malta
        'MH' => 'MHL', //Marshall Islands
        'MQ' => 'MTQ', //Martinique
        'MR' => 'MRT', //Mauritania
        'MU' => 'MUS', //Mauritius
        'YT' => 'MYT', //Mayotte
        'MX' => 'MEX', //Mexico
        'FM' => 'FSM', //Micronesia
        'MD' => 'MDA', //Moldova
        'MC' => 'MCO', //Monaco
        'MN' => 'MNG', //Mongolia
        'ME' => 'MNE', //Montenegro
        'MS' => 'MSR', //Montserrat
        'MA' => 'MAR', //Morocco
        'MZ' => 'MOZ', //Mozambique
        'MM' => 'MMR', //Myanmar
        'NA' => 'NAM', //Namibia
        'NR' => 'NRU', //Nauru
        'NP' => 'NPL', //Nepal
        'NL' => 'NLD', //Netherlands
        'AN' => 'ANT', //Netherlands Antilles
        'NC' => 'NCL', //New Caledonia
        'NZ' => 'NZL', //New Zealand
        'NI' => 'NIC', //Nicaragua
        'NE' => 'NER', //Niger
        'NG' => 'NGA', //Nigeria
        'NU' => 'NIU', //Niue
        'NF' => 'NFK', //Norfolk Island
        'MP' => 'MNP', //Northern Mariana Islands
        'NO' => 'NOR', //Norway
        'OM' => 'OMN', //Oman
        'PK' => 'PAK', //Pakistan
        'PW' => 'PLW', //Palau
        'PS' => 'PSE', //Palestinian Territory
        'PA' => 'PAN', //Panama
        'PG' => 'PNG', //Papua New Guinea
        'PY' => 'PRY', //Paraguay
        'PE' => 'PER', //Peru
        'PH' => 'PHL', //Philippines
        'PN' => 'PCN', //Pitcairn
        'PL' => 'POL', //Poland
        'PT' => 'PRT', //Portugal
        'PR' => 'PRI', //Puerto Rico
        'QA' => 'QAT', //Qatar
        'RE' => 'REU', //Reunion
        'RO' => 'ROU', //Romania
        'RU' => 'RUS', //Russia
        'RW' => 'RWA', //Rwanda
        'BL' => 'BLM', //Saint Barth&eacute;lemy
        'SH' => 'SHN', //Saint Helena
        'KN' => 'KNA', //Saint Kitts and Nevis
        'LC' => 'LCA', //Saint Lucia
        'MF' => 'MAF', //Saint Martin (French part)
        'SX' => 'SXM', //Sint Maarten / Saint Matin (Dutch part)
        'PM' => 'SPM', //Saint Pierre and Miquelon
        'VC' => 'VCT', //Saint Vincent and the Grenadines
        'WS' => 'WSM', //Samoa
        'SM' => 'SMR', //San Marino
        'ST' => 'STP', //S&atilde;o Tom&eacute; and Pr&iacute;ncipe
        'SA' => 'SAU', //Saudi Arabia
        'SN' => 'SEN', //Senegal
        'RS' => 'SRB', //Serbia
        'SC' => 'SYC', //Seychelles
        'SL' => 'SLE', //Sierra Leone
        'SG' => 'SGP', //Singapore
        'SK' => 'SVK', //Slovakia
        'SI' => 'SVN', //Slovenia
        'SB' => 'SLB', //Solomon Islands
        'SO' => 'SOM', //Somalia
        'ZA' => 'ZAF', //South Africa
        'GS' => 'SGS', //South Georgia/Sandwich Islands
        'SS' => 'SSD', //South Sudan
        'ES' => 'ESP', //Spain
        'LK' => 'LKA', //Sri Lanka
        'SD' => 'SDN', //Sudan
        'SR' => 'SUR', //Suriname
        'SJ' => 'SJM', //Svalbard and Jan Mayen
        'SZ' => 'SWZ', //Swaziland
        'SE' => 'SWE', //Sweden
        'CH' => 'CHE', //Switzerland
        'SY' => 'SYR', //Syria
        'TW' => 'TWN', //Taiwan
        'TJ' => 'TJK', //Tajikistan
        'TZ' => 'TZA', //Tanzania
        'TH' => 'THA', //Thailand
        'TL' => 'TLS', //Timor-Leste
        'TG' => 'TGO', //Togo
        'TK' => 'TKL', //Tokelau
        'TO' => 'TON', //Tonga
        'TT' => 'TTO', //Trinidad and Tobago
        'TN' => 'TUN', //Tunisia
        'TR' => 'TUR', //Turkey
        'TM' => 'TKM', //Turkmenistan
        'TC' => 'TCA', //Turks and Caicos Islands
        'TV' => 'TUV', //Tuvalu
        'UG' => 'UGA', //Uganda
        'UA' => 'UKR', //Ukraine
        'AE' => 'ARE', //United Arab Emirates
        'GB' => 'GBR', //United Kingdom
        'US' => 'USA', //United States
        'UM' => 'UMI', //United States Minor Outlying Islands
        'UY' => 'URY', //Uruguay
        'UZ' => 'UZB', //Uzbekistan
        'VU' => 'VUT', //Vanuatu
        'VE' => 'VEN', //Venezuela
        'VN' => 'VNM', //Vietnam
        'VG' => 'VGB', //Virgin Islands, British
        'VI' => 'VIR', //Virgin Island, U.S.
        'WF' => 'WLF', //Wallis and Futuna
        'EH' => 'ESH', //Western Sahara
        'YE' => 'YEM', //Yemen
        'ZM' => 'ZMB', //Zambia
        'ZW' => 'ZWE', //Zimbabwe
    );

    if (!isset($countries[$iso2])) {
        return false;
    }

    return $countries[$iso2];
}

/**
 * @param float  $amount
 * @param string $currencySymbol
 *
 * @return string
 */
function wcdpi_format_price($amount, $currencySymbol = '€') {
    if (function_exists('wc_price')) {
        $price = wc_price($amount, array('currency' => 'EUR'));
    } else {
        $price = number_format((float)$amount, 2, ',', '') . ' ' . $currencySymbol;
    }

    return $price;
}

/**
 * @param float $amount
 *
 * @return string
 */
function wcdpi_format_weight($amount) {
    if (function_exists('wc_format_weight')) {
        $weight = wc_format_weight($amount);
    } else {
        $weight = sprintf('%s%s', $amount, 'g');
    }

    return $weight;
}

/**
 * @param float $amount
 *
 * @return float
 */
function wcdpi_euros_to_cents($amount) {
    return round($amount * 100);
}

/**
 * @param string $from
 * @param string $to
 * @param float  $weight
 * @param int    $quantity
 * @param int    $precision
 *
 * @return float
 */
function wcdpi_convert_weight($from, $to, $weight, $quantity = 1, $precision = 2) {
    switch ($from) {
        case 'kg':
            switch ($to) {
                case 'kg':
                    $factor = 1; // 1 kg = 1 kg
                    break;
                case 'lbs':
                    $factor = 2.204623; // 1 kg = 2.204623 lbs
                    break;
                case 'oz':
                    $factor = 35.27396; // 1 kg = 35.27396 oz
                    break;
                case 'g':
                    $factor = 1000; // 1 kg = 1000 g
                    break;
                default:
                    $factor = 1;
                    break;
            }
            break;
        case 'lbs':
            switch ($to) {
                case 'kg':
                    $factor = 0.4535924; // 1 lbs = 0.4535924 kg
                    break;
                case 'lbs':
                    $factor = 1; // 1 lbs = 1 lbs
                    break;
                case 'oz':
                    $factor = 16; // 1 lbs = 16 oz
                    break;
                case 'g':
                    $factor = 453.5924; // 1 lbs = 453.5924 g
                    break;
                default:
                    $factor = 1;
                    break;
            }
            break;
        case 'oz':
            switch ($to) {
                case 'kg':
                    $factor = 0.02834952; // 0.02834952 oz = 1 kg
                    break;
                case 'lbs':
                    $factor = 0.0625; // 1 oz = 0.0625 lbs
                    break;
                case 'oz':
                    $factor = 1; // 1 oz = 1 oz
                    break;
                case 'g':
                    $factor = 28.34952; // 1 oz = 28.34952 g
                    break;
                default:
                    $factor = 1;
                    break;
            }
            break;
        case 'g':
            switch ($to) {
                case 'kg':
                    $factor = 0.001; // 0.001 g = 1 kg
                    break;
                case 'lbs':
                    $factor = 0.002204623; // 1 g = 0.002204623 lbs
                    break;
                case 'oz':
                    $factor = 0.03527396; // 1 g = 0.03527396 oz
                    break;
                case 'g':
                    $factor = 1; // 1 g = 1 g
                    break;
                default:
                    $factor = 1;
                    break;
            }
            break;
        default:
            $factor = 1;
            break;
    }

    // Calculation
    $weight = (floatval($weight) * $factor) * intval($quantity);

    return round($weight, $precision);
}

/**
 * @param string $voucherId
 *
 * @return int
 */
function wcdpi_convert_to_voucher_no($voucherId) {
    return hexdec(substr($voucherId, 10, 9));
}

/**
 * @param string $trackingNumber
 * @param string $dateTime
 *
 * @return string
 */
function wcdpi_tracking_link($trackingNumber, $dateTime) {
    try {
        $date = new DateTime($dateTime);
    } catch (Exception $e) {
        return '';
    }

    return sprintf(
        WCDPI_TRACKING_URL,
        $trackingNumber,
        $date->format('d'),
        $date->format('m'),
        $date->format('Y'),
        $trackingNumber
    );
}

/**
 * @return string
 */
function wcdpi_get_current_tab() {
    $tab = 'general';

    if (!isset($_GET['tab']) || $_GET['tab'] == 'general') {
        $tab = 'general';
    } elseif ($_GET['tab'] == 'services') {
        $tab = 'services';
    } elseif ($_GET['tab'] == 'tools') {
        $tab = 'tools';
    } elseif ($_GET['tab'] == 'system') {
        $tab = 'system';
    }

    return $tab;
}

/**
 * @param string $tab
 *
 * @return string
 */
function wcdpi_get_current_section($tab) {
    $section = '';

    // System Tab Sections
    if ($tab == 'system') {
        if (!isset($_GET['section']) || $_GET['section'] == 'port_checker') {
            $section = 'port_checker';
        } elseif ($_GET['section'] == 'tools') {
            $section = 'tools';
        } elseif ($_GET['section'] == 'advanced') {
            $section = 'advanced';
        }
    }

    // Portokasse Tab Sections
    elseif ($tab == 'services') {
        if (!isset($_GET['section']) || $_GET['section'] == 'portokasse') {
            $section = 'portokasse';
        } elseif ($_GET['section'] == 'internetmarke_1c4a') {
            $section = 'internetmarke_1c4a';
        } elseif ($_GET['section'] == 'internetmarke_1c4r') {
            $section = 'internetmarke_1c4r';
        } elseif ($_GET['section'] == 'internetmarke_prodws') {
            $section = 'internetmarke_prodws';
        } elseif ($_GET['section'] == 'packet_plus_international') {
            $section = 'packet_plus_international';
        } elseif ($_GET['section'] == 'packet_plus_cross_border') {
            $section = 'packet_plus_cross_border';
        }
    }

    // Tools Tab Sections
    elseif ($tab == 'tools') {
        if (!isset($_GET['section']) || $_GET['section'] == 'port_checker') {
            $section = 'port_checker';
        } elseif ($_GET['section'] == 'synchronization') {
            $section = 'synchronization';
        } elseif ($_GET['section'] == 'scheduled_tasks') {
            $section = 'scheduled_tasks';
        } elseif ($_GET['section'] == 'logger') {
            $section = 'logger';
        }
    }

    return $section;
}

/**
 * @param ProductSalesResourceModel $product
 *
 * @return string
 */
function wcdpi_generate_dimensions_html($product) {
    $html = '';
    $html .= '<small><ul style="margin-left: 1em;">';

    $html .= sprintf(
        '<li style="margin-bottom: 0;">%s: %d-%d %s</li>',
        __('Length', 'woo-dp-internetmarke'),
        $product->getLengthMin(),
        $product->getLengthMax(),
        $product->getLengthUnit()
    );

    $html .= sprintf(
        '<li style="margin-bottom: 0;">%s: %d-%d %s</li> ',
        __('Width', 'woo-dp-internetmarke'),
        $product->getWidthMin(),
        $product->getWidthMax(),
        $product->getWidthUnit()
    );

    if ($product->getHeightMin() == 0) {
        $html .= sprintf(
            '<li style="margin-bottom: 0;">%s: %s %d %s</li>',
            __('Height', 'woo-dp-internetmarke'),
            __('up to', 'woo-dp-internetmarke'),
            $product->getHeightMax(),
            $product->getHeightUnit()
        );
    }

    else {
        $html .= sprintf(
            '<li style="margin-bottom: 0;">%s: %d-%d %s</li>',
            __('Height', 'woo-dp-internetmarke'),
            $product->getHeightMin(),
            $product->getHeightMax(),
            $product->getHeightUnit()
        );
    }

    if ($product->getWeightMin() == null && $product->getWeightMax() == null && $product->getWeightUnit() == null) {
    }

    else {
        if ($product->getWeightMin() == 0) {
            $html .= sprintf(
                '<li style="margin-bottom: 0;">%s: %s %d %s</li>',
                __('Weight', 'woo-dp-internetmarke'),
                __('up to', 'woo-dp-internetmarke'),
                $product->getWeightMax(),
                $product->getWeightUnit()
            );
        }

        else {
            $html .= sprintf(
                '<li style="margin-bottom: 0;">%s: %d-%d %s</li>',
                __('Weight', 'woo-dp-internetmarke'),
                $product->getWeightMin(),
                $product->getWeightMax(),
                $product->getWeightUnit()
            );
        }
    }

    $html .= '</ul></small>';

    return $html;
}

/**
 * @param null|string $destination
 *
 * @return array
 */
function wcdpi_get_product_list_navigation($destination = null) {
    $national = array(
        'slug' => 'national',
        'name' => __('National', 'woo-dp-internetmarke'),
        'sub' =>
            array(
                array(
                    'slug' => 'briefpostkarte-national',
                    'name' => 'Brief / Postkarte',
                ),
                array(
                    'slug' => 'buechersendung',
                    'name' => 'Bücher- und Warensendung',
                ),
                array(
                    'slug' => 'streifbandzeitung',
                    'name' => 'Streifbandzeitung'
                )
            )
    );

    $international = array(
        'slug' => 'international',
        'name' => __('International', 'woo-dp-internetmarke'),
        'sub' =>
            array(
                array(
                    'slug' => 'briefpostkarte-international',
                    'name' => 'Brief / Postkarte'
                ),
                array(
                    'slug' => 'presse',
                    'name' => 'Presse'
                ),
                array(
                    'slug' => 'briefkilotarif',
                    'name' => 'Brief Kilotarif'
                )
            )
    );

    $internationalBusiness = array(
        'slug' => 'international-business',
        'name' => __('International (business)', 'woo-dp-internetmarke'),
        'sub' =>
            array(
                array(
                    'slug' => 'warenpostnoneu',
                    'name' => __('Warenpost (non-EU)', 'woo-dp-internetmarke')
                ),
                array(
                    'slug' => 'warenposteu',
                    'name' => __('Warenpost (EU)', 'woo-dp-internetmarke')
                ),
                array(
                    'slug' => 'warenpostkilotarif',
                    'name' => __('Warenpost (Kilo rate)', 'woo-dp-internetmarke')
                )
            )
    );

    $result = array();

    if (!$destination) {
        $result[] = $national;
        $result[] = $international;
        $result[] = $internationalBusiness;
    }

    elseif ($destination == 'national') {
        $result[] = $national;
    }

    elseif ($destination == 'international') {
        $result[] = $international;
        $result[] = $internationalBusiness;
    }

    return $result;
}

/**
 * @return bool
 */
function wcdpi_is_woocommerce_installed() {
    return in_array(
        'woocommerce/woocommerce.php',
        apply_filters('active_plugins', get_option('active_plugins'))
    );
}

/**
 * @return array
 */
function wcdpi_get_eu_countries() {
    return array(
        'AT', 'BE', 'BG', 'HR', 'CY', 'CZ', 'DK', 'EE', 'FI', 'FR', 'DE', 'GR', 'HU', 'IE',
        'IT', 'LV', 'LT', 'LU', 'MT', 'NL', 'PL', 'PT', 'RO', 'SK', 'SI', 'ES', 'SE',
    );
}

/**
 * @var bool|string $shippingRegion ShippingRegion Enum value
 *
 * @return array
 */
function wcdpi_get_harmonized_labels($shippingRegion = false) {
    $euLabels = array(
        13143, // Warenpost International XS (EU/USt.)
        13144, // Warenpost International S (EU/USt.)
        13145, // Warenpost International M (EU/USt.)
        13146, // Warenpost International L (EU/USt.)
        13147, // Warenpost International XS Tracked (EU/USt.)
        13148, // Warenpost International S Tracked (EU/USt.)
        13149, // Warenpost International M Tracked (EU/USt.)
        13150, // Warenpost International L Tracked (EU/USt.)
        13159, // Warenpost Int. KT (EU/USt.) für Internetmarke
        13160, // Warenpost Int. KT Tracked (EU/USt.) für Internetmarke
        13167, // Warenpost International XS Unterschrift (EU/USt.)
        13168, // Warenpost International S Unterschrift (EU/USt.)
        13169, // Warenpost International M Unterschrift (EU/USt.)
        13170, // Warenpost International L Unterschrift (EU/USt.)
        13175, // Warenpost Int. KT Unterschrift (EU/USt.) für Internetmarke
    );

    $nonEuLabels = array(
        13135, // Warenpost International XS
        13136, // Warenpost International S
        13137, // Warenpost International M
        13138, // Warenpost International L
        13139, // Warenpost International XS Tracked
        13140, // Warenpost International S Tracked
        13141, // Warenpost International M Tracked
        13142, // Warenpost International L Tracked
        13163, // Warenpost International XS Unterschrift
        13164, // Warenpost International S Unterschrift
        13165, // Warenpost International M Unterschrift
        13166, // Warenpost International L Unterschrift
        14110, // Warenpost Int. KT (Non EU) für Internetmarke
        14111, // Warenpost Int. KT Tracked (Non EU) für Internetmarke
        14112, // Warenpost Int. KT Unterschrift (Non EU) für Internetmarke
    );

    if (!$shippingRegion) {
        return array_merge($euLabels, $nonEuLabels);
    }

    switch ($shippingRegion) {
        case ShippingRegionEnum::EUROPEAN_UNION:
            return $euLabels;
        case ShippingRegionEnum::REST_OF_WORLD:
            return $nonEuLabels;
        default:
            return array();
    }
}

/**
 * @param int $shipmentId
 * @return string
 */
function wcdpi_format_shipment_number($shipmentId) {
    return sprintf('#%s', str_pad($shipmentId, 8,'0', STR_PAD_LEFT));
}

/**
 * @param int $shipmentStatus
 *
 * @return array;
 */
function wcdpi_get_shipment_actions($shipmentStatus) {
    switch ($shipmentStatus) {
        case (ShipmentStatus::PREPARED):
            $actions = array(
                'wcdpi_shipment_action_order_labels'    => __('Create shipping label', 'woo-dp-internetmarke'),
                'wcdpi_shipment_action_mark_as_deleted' => __('Delete', 'woo-dp-internetmarke')
            );
            break;
        case (ShipmentStatus::READY_TO_SHIP):
            $actions = array(
                'wcdpi_shipment_action_order_labels'    => __('Create shipping label', 'woo-dp-internetmarke'),
                'wcdpi_shipment_action_mark_as_deleted' => __('Delete', 'woo-dp-internetmarke')
            );
            break;
        case (ShipmentStatus::SHIPPED):
            $actions = array(
                'wcdpi_shipment_action_mark_as_ready_to_ship' => __('Revert to ready to ship', 'woo-dp-internetmarke'),
                'wcdpi_shipment_action_mark_as_delivered'     => __('Mark as delivered', 'woo-dp-internetmarke'),
                'wcdpi_shipment_action_mark_as_returned'      => __('Mark as returned', 'woo-dp-internetmarke'),
            );
            break;
        case (ShipmentStatus::DELIVERED):
            $actions = array(
                'wcdpi_shipment_action_mark_as_shipped'  => __('Revert to shipped', 'woo-dp-internetmarke'),
                'wcdpi_shipment_action_mark_as_returned' => __('Mark as returned', 'woo-dp-internetmarke'),
            );
            break;
        case (ShipmentStatus::REFUNDED):
            $actions = array();
            break;
        case (ShipmentStatus::RETURNED):
            $actions = array(
                'wcdpi_shipment_action_mark_as_shipped'   => __('Revert to shipped', 'woo-dp-internetmarke'),
                'wcdpi_shipment_action_mark_as_delivered' => __('Revert to delivered', 'woo-dp-internetmarke'),
            );
            break;
        case (ShipmentStatus::DELETED):
            $actions = array(
                'wcdpi_shipment_action_delete' => __('Delete permanently', 'woo-dp-internetmarke')
            );
            break;
    }

    return $actions;
}

/**
 * @param string $region
 *
 * @return string|void
 */
function wcdpi_get_shipping_region_name($region) {
    $region = strtoupper($region);

    switch ($region) {
        case 'DE':
            return WC()->countries->countries['DE'];
        case 'EU':
            return __('European union', 'woo-dp-internetmarke');
        case 'ROW':
            return __('Rest of world', 'woo-dp-internetmarke');
    }
}

/**
 * Converts dashes to camel case with first capital letter.
 *
 * @param string $input
 * @param string $separator
 *
 * @return string|string[]
 */
function wcdpi_camelize($input, $separator = '_')
{
    return str_replace($separator, '', ucwords($input, $separator));
}

function wcdpi_get_sales_product_group_label($salesProductGroup) {
    switch ($salesProductGroup) {
        case 'briefpostkarte-national':
            return 'Brief / Postkarte (National)';
        case 'buechersendung':
            return 'Bücher- und Warensendung';
        case 'streifbandzeitung':
            return 'Streifbandzeitung';
        case 'briefpostkarte-international':
            return 'Brief / Postkarte (International)';
        case 'presse':
            return 'Presse';
        case 'briefkilotarif':
            return 'Brief (Kilotarif)';
        case 'warenpostnoneu':
            return 'Warenpost (non-EU)';
        case 'warenposteu':
            return 'Warenpost (EU)';
        case 'warenpostkilotarif':
            return 'Warenpost (Kilotarif)';
        default:
            return $salesProductGroup;
    }
}

function wcdpi_get_shipping_products_array($shouldIncludeFavorites = true, $shippingCountry = null) {
    $result = array();

    if ($shouldIncludeFavorites) {
        $settings = get_option('_wcdpi_settings_internetmarke_1c4a', array());

        if (isset($settings['_wcdpi_favorite_labels_selection'])
            && is_array($settings['_wcdpi_favorite_labels_selection'])
        ) {
            $favorites = $settings['_wcdpi_favorite_labels_selection'];
            $query = array('prodws_id' => $favorites);

            if ($shippingCountry !== null) {
                $query['destination'] = ($shippingCountry === 'DE') ? 'national' : 'international';
            }

            /** @var ProductSalesResourceModel[] $favoriteLabels */
            $favoriteLabels = ProductSalesResourceRepository::instance()->findAllBy($query);

            if ($favoriteLabels) {
                $result[] = array(
                    'slug' => 'favorites',
                    'label' => __('Favorites', 'woo-dp-internetmarke'),
                    'products' => $favoriteLabels
                );
            }
        }
    }

    if ($shippingCountry !== null) {
        /** @var ProductSalesResourceModel[] $productSalesList */
        $productSalesList = ProductSalesResourceRepository::instance()->findAllBy(
            array(
                'destination' => strtoupper($shippingCountry) === 'DE' ? 'national' : 'international'
            ),
            'name',
            'ASC'
        );
    } else {
        /** @var ProductSalesResourceModel[] $allLabels */
        $productSalesList = ProductSalesResourceRepository::instance()->findAll('name', 'ASC');
    }

    $group = array(
        'slug' => 'all_shipping_products',
        'label' => __('All shipping products', 'woo-dp-internetmarke'),
        'products' => $productSalesList
    );

    $result[] = $group;

    return $result;
}
