<?php

use DeutschePost\Internetmarke\Enums\ShipmentStatus as ShipmentStatusEnum;
use DeutschePost\Internetmarke\Models\Resources\ProductSales as ProductSalesResourceModel;
use DeutschePost\Internetmarke\Models\Resources\Shipment as ShipmentResourceModel;
use DeutschePost\Internetmarke\Repositories\Resources\ProductSales as ProductSalesResourceRepository;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH' || exit);

/**
 * @param int $shipmentId
 * @param int $statusId
 * @return bool
 */
function wcdpi_set_shipment_status($shipmentId, $statusId) {
    return Internetmarke()->shipmentService->setStatus($shipmentId, $statusId);
}

/**
 * @param $shipmentId
 * @return bool
 */
function wcdpi_set_shipment_status_ready_to_ship($shipmentId) {
    return wcdpi_set_shipment_status($shipmentId, ShipmentStatusEnum::READY_TO_SHIP);
}

/**
 * @param $shipmentId
 * @return bool
 */
function wcdpi_set_shipment_status_shipped($shipmentId) {
    return wcdpi_set_shipment_status($shipmentId, ShipmentStatusEnum::SHIPPED);
}

/**
 * @param $shipmentId
 * @return bool
 */
function wcdpi_set_shipment_status_delivered($shipmentId) {
    return wcdpi_set_shipment_status($shipmentId, ShipmentStatusEnum::DELIVERED);
}

/**
 * @param $shipmentId
 * @return bool
 */
function wcdpi_set_shipment_status_refunded($shipmentId) {
    return wcdpi_set_shipment_status($shipmentId, ShipmentStatusEnum::REFUNDED);
}

/**
 * @param $shipmentId
 * @return bool
 */
function wcdpi_set_shipment_status_returned($shipmentId) {
    return wcdpi_set_shipment_status($shipmentId, ShipmentStatusEnum::RETURNED);
}

/**
 * @param $shipmentId
 * @return bool
 */
function wcdpi_set_shipment_status_deleted($shipmentId) {
    return wcdpi_set_shipment_status($shipmentId, ShipmentStatusEnum::DELETED);
}

/**
 * @param $shipmentId
 * @return bool
 */
function wcdpi_set_shipment_status_failed($shipmentId) {
    return wcdpi_set_shipment_status($shipmentId, ShipmentStatusEnum::FAILED);
}

/**
 * @param ShipmentResourceModel $shipment
 * @return int
 */
function wcdpi_add_shipment_success_order_note($shipment) {
    if (!$shipment || !$order = wc_get_order($shipment->getOrderId())) {
        return 0;
    }

    /** @var false|ProductSalesResourceModel $productSalesModel */
    $productSalesModel = ProductSalesResourceRepository::instance()->findBy(
        array(
            'prodws_id' => $shipment->getProdwsId()
        )
    );

    $note = 'Internetmarke - ' . __('Shipment created successfully.', 'woo-dp-internetmarke');

    if ($productSalesModel) {
        $note .= ' ' . __('Label', 'woo-dp-internetmarke') . ': ' . $productSalesModel->getName();

        if ($shipment->isHarmonizedLabel()) {
            $note .= ' (' . __('Harmonized label', 'woo-dp-internetmarke') . ')';
        }

        $note .= '.';
    }

    return $order->add_order_note($note, 0, false);
}

/**
 * @param ShipmentResourceModel $shipment
 * @param Exception $e
 * @return int
 */
function wcdpi_add_shipment_failed_order_note($shipment, $e) {
    if (!$shipment || !$order = wc_get_order($shipment->getOrderId())) {
        return 0;
    }

    /** @var false|ProductSalesResourceModel $productSalesModel */
    $productSalesModel = ProductSalesResourceRepository::instance()->findBy(
        array(
            'prodws_id' => $shipment->getProdwsId()
        )
    );

    $note = 'Internetmarke - ' . __('Shipment creation failed.', 'woo-dp-internetmarke');

    if ($productSalesModel) {
        $note .= ' ' . __('Label', 'woo-dp-internetmarke') . ': ' . $productSalesModel->getName();

        if ($shipment->isHarmonizedLabel()) {
            $note .= ' (' . __('Harmonized label', 'woo-dp-internetmarke') . ')';
        }

        $note .= '.';
    }

    $note .= ' ' . __('Error', 'woo-dp-internetmarke') . ': ' . $e->getMessage() . '.';

    return $order->add_order_note($note, 0, false);
}