<?php

namespace DeutschePost\Internetmarke\Abstracts;

defined('ABSPATH') || exit;

abstract class Settings
{
    /**
     * @var array
     */
    protected $settings;

    /**
     * Returns the value from the settings array.
     *
     * @param string $name            The setting array index
     * @param bool   $returnAsBoolean Determines if the output is boolean
     * @param string $defaultValue    The default return value
     *
     * @return string|bool
     */
    protected function getSettingValue($name, $returnAsBoolean = false, $defaultValue = '')
    {
        $value = $defaultValue;

        if (array_key_exists($name, $this->settings)) {
            $value = $this->settings[$name];
        }

        if ($returnAsBoolean) {
            return $value ? true : false;
        }

        return $value;
    }

    /**
     * Creates the description text HTML markup for a settings field.
     *
     * @param string $text
     *
     * @return string
     */
    protected function createDescription($text)
    {
        return sprintf('<p class="description">%s</p>', $text);
    }

    /**
     * Creates a standard input field HTML markup.
     *
     * @param string $group    The setting field group
     * @param string $name     The setting field name
     * @param string $value    The setting field value
     * @param string $type     The setting field type
     * @param bool   $disabled Whether the field is disabled
     *
     * @return string
     */
    protected function createInputField($group, $name, $value, $type = 'text', $disabled = false)
    {
        return sprintf(
            '<input id="%s" class="regular-text" name="%s[%s]" type="%s" value="%s" %s/>',
            $name,
            $group,
            $name,
            $type,
            $value,
            $disabled ? 'disabled="disabled"' : ''
        );
    }
}