<?php

namespace DeutschePost\Internetmarke;

defined('ABSPATH') || exit;

class Address
{
    /**
     * @var string
     */
    protected $firstName;

    /**
     * @var string
     */
    protected $lastName;

    /**
     * @var string
     */
    protected $company;

    /**
     * @var string
     */
    protected $email;

    /**
     * @var string
     */
    protected $phone;

    /**
     * @var string
     */
    protected $contactName;

    /**
     * @var string
     */
    protected $street;

    /**
     * @var string
     */
    protected $houseNumber;

    /**
     * @var string
     */
    protected $additional;

    /**
     * @var string
     */
    protected $zip;

    /**
     * @var string
     */
    protected $city;

    /**
     * @var string
     */
    protected $state;

    /**
     * @var string
     */
    protected $country;

    /**
     * @var string
     */
    protected $countryIso3;

    /**
     * Address constructor.
     *
     * @param null|array $addressData
     */
    function __construct($addressData = null)
    {
        $empty = array(
            'firstName'   => '',
            'lastName'    => '',
            'company'     => '',
            'contactName' => '',
            'email'       => '',
            'phone'       => '',
            'street'      => '',
            'houseNumber' => '',
            'additional'  => '',
            'zip'         => '',
            'city'        => '',
            'state'       => '',
            'country'     => ''
        );

        $address = $empty;

        if (is_array($addressData)) {
            $address = array_merge($empty, $addressData);
        }

        $this->setFirstName($address['firstName']);
        $this->setLastName($address['lastName']);
        $this->setCompany($address['company']);
        $this->setEmail($address['email']);
        $this->setPhone($address['phone']);
        $this->setContactName($address['contactName']);
        $this->setStreet($address['street']);
        $this->setHouseNumber($address['houseNumber']);
        $this->setAdditional($address['additional']);
        $this->setZip($address['zip']);
        $this->setCity($address['city']);
        $this->setState($address['state']);
        $this->setCountry($address['country']);
        $this->setCountryIso3(wcdpi_convert_country_iso2_to_iso3($address['country']));
    }

    /**
     * @return array
     */
    public function toArray()
    {
        return get_object_vars($this);
    }

    /**
     * @return string
     */
    public function getFirstName()
    {
        return $this->firstName;
    }

    /**
     * @param string $firstName
     */
    public function setFirstName($firstName)
    {
        $this->firstName = $firstName;
    }

    /**
     * @return string
     */
    public function getLastName()
    {
        return $this->lastName;
    }

    /**
     * @param string $lastName
     */
    public function setLastName($lastName)
    {
        $this->lastName = $lastName;
    }

    /**
     * @return string
     */
    public function getCompany()
    {
        return $this->company;
    }

    /**
     * @param string $company
     */
    public function setCompany($company)
    {
        $this->company = $company;
    }

    /**
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    /**
     * @return string
     */
    public function getPhone()
    {
        return $this->phone;
    }

    /**
     * @param string $phone
     */
    public function setPhone($phone)
    {
        $this->phone = $phone;
    }

    /**
     * @return string
     */
    public function getContactName()
    {
        return $this->contactName;
    }

    /**
     * @param string $contactName
     */
    public function setContactName($contactName)
    {
        $this->contactName = $contactName;
    }

    /**
     * @return string
     */
    public function getStreet()
    {
        return $this->street;
    }

    /**
     * @param string $street
     */
    public function setStreet($street)
    {
        $this->street = $street;
    }

    /**
     * @return string
     */
    public function getHouseNumber()
    {
        return $this->houseNumber;
    }

    /**
     * @param string $houseNumber
     */
    public function setHouseNumber($houseNumber)
    {
        $this->houseNumber = $houseNumber;
    }

    /**
     * @return string
     */
    public function getAdditional()
    {
        return $this->additional;
    }

    /**
     * @param string $additional
     */
    public function setAdditional($additional)
    {
        $this->additional = $additional;
    }

    /**
     * @return string
     */
    public function getZip()
    {
        return $this->zip;
    }

    /**
     * @param string $zip
     */
    public function setZip($zip)
    {
        $this->zip = $zip;
    }

    /**
     * @return string
     */
    public function getCity()
    {
        return $this->city;
    }

    /**
     * @param string $city
     */
    public function setCity($city)
    {
        $this->city = $city;
    }

    /**
     * @return string
     */
    public function getState()
    {
        return $this->state;
    }

    /**
     * @param string $state
     */
    public function setState($state)
    {
        $this->state = $state;
    }

    /**
     * @return string
     */
    public function getCountry()
    {
        return $this->country;
    }

    /**
     * @param string $country
     */
    public function setCountry($country)
    {
        $this->country = $country;
    }

    /**
     * @return string
     */
    public function getCountryIso3()
    {
        return $this->countryIso3;
    }

    /**
     * @param string $countryIso3
     */
    public function setCountryIso3($countryIso3)
    {
        $this->countryIso3 = $countryIso3;
    }
}