<?php

namespace DeutschePost\Internetmarke\Components\View;

use DateTime;
use Exception;
use setasign\Fpdi\Fpdi;
use setasign\Fpdi\PdfParser\PdfParserException;
use setasign\Fpdi\PdfParser\StreamReader;
use setasign\Fpdi\PdfParser\Type\PdfTypeException;
use setasign\Fpdi\PdfReader\PdfReaderException;
use DeutschePost\Internetmarke\Admin\Notice;
use DeutschePost\Internetmarke\Connectors\OneClickForRefund\Response\RetoureVouchers;
use DeutschePost\Internetmarke\Connectors\PacketPlus\Response\GetItemLabel as GetItemLabelResponse;
use DeutschePost\Internetmarke\Enums\ShipmentAttachment as ShipmentAttachmentEnum;
use DeutschePost\Internetmarke\Enums\ShipmentStatus as ShipmentStatusEnum;
use DeutschePost\Internetmarke\Integrations\Host\PacketPlus as PacketPlusHostIntegration;
use DeutschePost\Internetmarke\Models\Resources\ProductSales as ProductSalesResourceModel;
use DeutschePost\Internetmarke\Models\Resources\Shipment as ShipmentResourceModel;
use DeutschePost\Internetmarke\Models\Resources\ShipmentAttachment as ShipmentAttachmentResourceModel;
use DeutschePost\Internetmarke\Models\Resources\ShipmentItem as ShipmentItemResourceModel;
use DeutschePost\Internetmarke\Repositories\Resources\ProductSales as ProductSalesResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\Shipment as ShipmentResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ShipmentAttachment;
use DeutschePost\Internetmarke\Repositories\Resources\ShipmentAttachment as ShipmentAttachmentResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ShipmentItem as ShipmentItemResourceRepository;
use DeutschePost\Internetmarke\Store;
use WP_List_Table;
use wpdb;
use ZipArchive;
use function DeutschePost\Internetmarke\Internetmarke;
use function WC;

defined('ABSPATH') || exit;

if (!class_exists('WP_List_Table')) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Shipments extends WP_List_Table
{
    /**
     * @var string
     */
    const PAGE = 'internetmarke_shipments';

    /**
     * @var string
     */
    protected $shipment;

    /**
     * @var string
     */
    protected $shipmentItem;

    /**
     * @var wpdb
     */
    protected $wpdb;

    /**
     * @var string
     */
    private $shipmentTableName;

    /**
     * @var string
     */
    private $shipmentItemTableName;

    /**
     * @var string
     */
    private $shipmentAttachmentTableName;

    /**
     * @var Store
     */
    private $store;

    /**
     * @var PacketPlusHostIntegration
     */
    private $packetPlusHostIntegration;

    /**
     * Shipments constructor.
     *
     * @param Store                     $store
     * @param PacketPlusHostIntegration $packetPlusHostIntegration
     */
    public function __construct(
        Store $store,
        PacketPlusHostIntegration $packetPlusHostIntegration
    ) {
        global $wpdb;

        $this->shipmentTableName           = 'wcdpi_shipment';
        $this->shipmentItemTableName       = 'wcdpi_shipment_item';
        $this->shipmentAttachmentTableName = 'wcdpi_shipment_attachment';
        $this->store                       = $store;
        $this->packetPlusHostIntegration   = $packetPlusHostIntegration;
        $this->wpdb                        = $wpdb;
        $this->shipment                    = $wpdb->prefix . $this->shipmentTableName;
        $this->shipmentItem                = $wpdb->prefix . $this->shipmentItemTableName;

        parent::__construct(
            array(
                'singular' => __('Shipment', 'woo-dp-internetmarke'),
                'plural'   => __('Shipments', 'woo-dp-internetmarke'),
                'ajax'     => false
            )
        );
    }

    /**
     * Outputs a message when no shipments exist.
     */
    public function no_items()
    {
        _e('No shipments found.', 'woo-dp-internetmarke');
    }

    /**
     * Sets the shipment table columns.
     *
     * @return array
     */
    public function get_columns()
    {
        $columns = array(
            'cb'            => '<input type="checkbox" />',
            'shipment_info' => __('Shipment info', 'woo-dp-internetmarke'),
            'recipient'     => __('Recipient', 'woo-dp-internetmarke'),
            'package_info'  => __('Package info', 'woo-dp-internetmarke'),
            'last_modified' => __('Last modified', 'woo-dp-internetmarke'),
            'attachments'   => __('Attachments', 'woo-dp-internetmarke')
        );

        return $columns;
    }

    /**
     * Sets the content of the checkbox column.
     *
     * @param array $item
     *
     * @return string
     */
    public function column_cb($item)
    {
        return '<input type="checkbox" name="id[]" value="' . $item['id'] . '" />';
    }

    /**
     * Sets the content of the "shipment_info" column.
     *
     * @param array $item
     *
     * @return string
     */
    public function column_shipment_info($item) {
        /** @var ProductSalesResourceModel $productSales */
        $productSales = ProductSalesResourceRepository::instance()->findBy(
            array(
                'prodws_id' => $item['prodws_id']
            )
        );

        $html   = '';
        $actions = array();

        $html .= '<div>';
        $html .= '<b>' . __('Product', 'woo-dp-internetmarke') . '</b>: ';
        $html .= esc_html($productSales->getName());
        $html .= '</div>';

        if ($item['price']) {
            $html .= '<div>';
            $html .= '<b>' . __('Price', 'woo-dp-internetmarke') . '</b>: ';
            $html .= wcdpi_format_price($item['price'] / 100);
            $html .= '</div>';
        }

        if ($item['shop_order_id']) {
            $html .= '<div>';
            $html .= '<b>' . __('DP Order', 'woo-dp-internetmarke') . '</b>: ';
            $html .= esc_html($item['shop_order_id']);
            $html .= '</div>';
        }

        if ($item['shop_pp_order_id']) {
            $html .= '<div>';
            $html .= '<b>' . __('DP Order (Packet Plus)', 'woo-dp-internetmarke') . '</b>: ';
            $html .= esc_html($item['shop_pp_order_id']);
            $html .= '</div>';
        }

        if ($item['pp_shipment_id']) {
            $html .= '<div>';
            $html .= '<b>' . __('DP Shipment (Packet Plus)', 'woo-dp-internetmarke') . '</b>: ';
            $html .= esc_html($item['pp_shipment_id']);
            $html .= '</div>';
        }

        if ($item['voucher_id']) {
            $html .= '<div>';
            $html .= '<b>' . __('Voucher', 'woo-dp-internetmarke') . '</b>: ';
            $html .= esc_html($item['voucher_id']);
            $html .= '</div>';
        }

        if ($item['tracking']) {
            $trackingLink = wcdpi_tracking_link($item['tracking'], $item['purchase_dt']);

            $html .= sprintf('<div><b>%s</b>: %s</div>', __('Tracking', 'woo-dp-internetmarke'), $trackingLink);
        }

        // ID
        $actions['id'] = 'ID: ' . intval($item['id']);

        // Delete & Order label
        if (intval($item['status']) === ShipmentStatusEnum::PREPARED) {
            $actions['delete'] = sprintf(
                '<a href="%s">%s</a>',
                admin_url(
                    sprintf(
                        'admin.php?page=%s&action=%s&id=%d&_wpnonce=%s',
                        self::PAGE,
                        'delete',
                        intval($item['id']),
                        wp_create_nonce('delete')
                    )
                ),
                __('Delete', 'woo-dp-internetmarke')
            );
            $actions['order'] = sprintf(
                '<a href="%s">%s</a>',
                admin_url(
                    sprintf(
                        'admin.php?page=%s&action=%s&id=%d&_wpnonce=%s',
                        self::PAGE,
                        'order',
                        intval($item['id']),
                        wp_create_nonce('order')
                    )
                ),
                __('Order label', 'woo-dp-internetmarke')
            );
        }

        // Failed labels
        if (intval($item['status']) === ShipmentStatusEnum::FAILED) {
            $actions['delete'] = sprintf(
                '<a href="%s">%s</a>',
                admin_url(
                    sprintf(
                        'admin.php?page=%s&action=%s&id=%d&_wpnonce=%s',
                        self::PAGE,
                        'delete',
                        intval($item['id']),
                        wp_create_nonce('delete')
                    )
                ),
                __('Delete', 'woo-dp-internetmarke')
            );
        }

        // Refund & mark as shipped
        if (intval($item['status']) === ShipmentStatusEnum::READY_TO_SHIP) {
            if (!$item['harmonized_label']) {
                $actions['refund'] = sprintf(
                    '<a href="%s">%s</a>',
                    admin_url(
                        sprintf(
                            'admin.php?page=%s&action=%s&id=%d&_wpnonce=%s',
                            self::PAGE,
                            'refund',
                            intval($item['id']),
                            wp_create_nonce('refund')
                        )
                    ),
                    __('Refund', 'woo-dp-internetmarke')
                );
            }

            $actions['mark_as_shipped'] = sprintf(
                '<a href="%s">%s</a>',
                admin_url(
                    sprintf(
                        'admin.php?page=%s&action=%s&id=%d&_wpnonce=%s',
                        self::PAGE,
                        'mark_as_shipped',
                        intval($item['id']),
                        wp_create_nonce('mark_as_shipped')
                    )
                ),
                __('Mark as shipped', 'woo-dp-internetmarke')
            );
        }

        // Mark as delivered
        if (intval($item['status']) === ShipmentStatusEnum::SHIPPED) {
            $actions['mark_as_delivered'] = sprintf(
                '<a href="%s">%s</a>',
                admin_url(
                    sprintf(
                        'admin.php?page=%s&action=%s&id=%d&_wpnonce=%s',
                        self::PAGE,
                        'mark_as_delivered',
                        intval($item['id']),
                        wp_create_nonce('mark_as_delivered')
                    )
                ),
                __('Mark as delivered', 'woo-dp-internetmarke')
            );
        }

        // Mark as returned
        if (intval($item['status']) === ShipmentStatusEnum::DELIVERED) {
            $actions['mark_as_returned'] = sprintf(
                '<a href="%s">%s</a>',
                admin_url(
                    sprintf(
                        'admin.php?page=%s&action=%s&id=%d&_wpnonce=%s',
                        self::PAGE,
                        'mark_as_returned',
                        intval($item['id']),
                        wp_create_nonce('mark_as_returned')
                    )
                ),
                __('Mark as returned', 'woo-dp-internetmarke')
            );
        }

        return $html . $this->row_actions($actions);
    }

    /**
     * Sets the content of the "recipient" column.
     *
     * @param array $item
     *
     * @return string
     */
    public function column_recipient($item)
    {
        $html = '<address>';

        if ($item['recipient_company_name']) {
            $html .= trim($item['recipient_company_name']) . '<br>';
        }

        if ($item['recipient_first_name'] && $item['recipient_last_name']) {
            $html .= trim($item['recipient_first_name'] . ' ' . $item['recipient_last_name']) . '<br>';
        }

        $html .= implode(' ', array(trim($item['recipient_street']), trim($item['recipient_house_number']))) . '<br>';
        $html .= implode(' ', array(trim($item['recipient_zip']), trim($item['recipient_city']))) . '<br>';

        if ($item['recipient_state']) {
            $html .= trim($item['recipient_state']) . '<br>';
        }

        $html .= WC()->countries->countries[$item['recipient_country_alpha2']];
        $html .= '</address>';

        return $html;
    }

    /**
     * @param array $item
     *
     * @return string
     */
    public function column_package_info($item)
    {
        /** @var ShipmentItemResourceModel[] $shipmentItems */
        $shipmentItems = ShipmentItemResourceRepository::instance()->findAllBy(
            array(
                'shipment_id' => $item['id']
            )
        );

        if (!$shipmentItems) {
            return '';
        }

        $html = '';

        if ($orderId = intval($item['order_id'])) {
            $edit = get_edit_post_link($orderId);
            $html .= '<div><b>' . __('Shop order', 'woo-dp-internetmarke') . '</b>: <a href="' . $edit . '">#' . $orderId . '</a></div>';
        }

        if ($shipmentValue = ShipmentResourceRepository::instance()->value($item['id'])) {
            $html .= sprintf('<div><b>%s</b>: %s</div>', __('Value', 'woo-dp-internetmarke'), wcdpi_format_price($shipmentValue));
        }

        if ($shipmentWeight = ShipmentResourceRepository::instance()->weight($item['id'])) {
            $weight = wcdpi_convert_weight('g', get_option('woocommerce_weight_unit'), $shipmentWeight);
            $html .= sprintf('<div><b>%s</b>: %s</div>', __('Weight', 'woo-dp-internetmarke'), wcdpi_format_weight($weight));
        }

        if (!function_exists('wc_get_product')) {
            return $html;
        }

        $html .= '<div><b>' . __('Items included', 'woo-dp-internetmarke') . ':</b></div>';

        $html .= '<ul class="wcdpi-line-items">';

        /** @var ShipmentItemResourceModel $shipmentItem */
        foreach ($shipmentItems as $shipmentItem) {
            if (!$product = wc_get_product(intval($shipmentItem->getProductId()))) {
                continue;
            }

            $quantity = $shipmentItem->getProductQuantity();
            $edit     = get_edit_post_link($shipmentItem->getProductId());
            $name     = $product->get_name();

            $html .= '<li><small>' . $quantity . ' &times; </small><a href="' . $edit . '"><b>' . $name . '</b></a>';

            if ($product->get_sku()) {
                $html .= ' <small>(SKU: ' . $product->get_sku() . '</small>)';
            }

            $html .= '</li>';
        }

        $html .= '</ul>';

        return $html;
    }

    /**
     * Sets the content of the "package_info" column.
     *
     * @param array $item
     *
     * @return string
     */
    public function column_last_modified($item)
    {
        $html = '';

        if ($item['status_update_dt']) {
            $html .= date_i18n('d.m.Y', strtotime($item['status_update_dt']));
            $html .= '<br>';
            $html .= date_i18n('H:i', strtotime($item['status_update_dt']));

            return $html;
        }

        return $html;
    }

    /**
     * Sets the content of the "attachments" column.
     *
     * @param array $item
     *
     * @return string
     */
    public function column_attachments($item)
    {
        $html = '';

        /** @var ShipmentAttachmentResourceModel[] $attachments */
        $attachments = ShipmentAttachmentResourceRepository::instance()->findAllBy(
            array(
                'shipment_id' => $item['id']
            )
        );

        if ($attachments) {
            /** @var ShipmentAttachmentResourceModel $attachment */
            foreach ($attachments as $attachment) {
                switch ($attachment->getType()) {
                    case ShipmentAttachmentEnum::LABEL:
                        $label = __('Label', 'woo-dp-internetmarke');
                        break;
                    case ShipmentAttachmentEnum::MANIFEST:
                        $label = __('Manifest', 'woo-dp-internetmarke');
                        break;
                    case ShipmentAttachmentEnum::CN22:
                        $label = __('CN22', 'woo-dp-internetmarke');
                        break;
                    default:
                        $label = '';
                        break;
                }

                if ($item['status'] == ShipmentStatusEnum::REFUNDED) {
                    $html .= '<div><b>' . $label . ':</b> ' . __('Refunded', 'woo-dp-internetmarke') . '</div>';
                } else {
                    $url  = $attachment->getUrl();
                    $html .= '<div><b>' . $label . ':</b> <a href="' . $url . '">' . __('Download', 'woo-dp-internetmarke') . '</a></div>';
                }
            }
        }

        return $html;
    }

    /**
     * Returns the default value for an unspecified column.
     *
     * @param array  $item
     * @param string $columnName
     *
     * @return string
     */
    public function column_default($item, $columnName)
    {
        return $item[$columnName];
    }

    /**
     * @return array
     */
    public function get_views()
    {
        $status    = 'prepared';
        $viewLinks = array();

        if (!empty($_REQUEST['status'])) {
            $status = $_REQUEST['status'];
        }

        // Prepared
        $class       = $status === 'prepared' ? ' class="current"' : '';
        $urlPrepared = admin_url(sprintf('admin.php?page=%s&status=%s', self::PAGE, 'prepared'));
        $viewLinks['prepared'] = sprintf(
            '<a href="%s" %s>%s <span class="count">(%d)</span></a>',
            $urlPrepared,
            $class,
            __('Prepared', 'woo-dp-internetmarke'),
            ShipmentResourceRepository::instance()->countBy(array('status' => ShipmentStatusEnum::PREPARED))
        );

        // Ready to ship
        $class          = $status === 'ready_to_ship' ? ' class="current"' : '';
        $urlReadyToShip = admin_url(sprintf('admin.php?page=%s&status=%s', self::PAGE, 'ready_to_ship'));
        $viewLinks['ready_to_ship'] = sprintf(
            '<a href="%s" %s>%s <span class="count">(%d)</span></a>',
            $urlReadyToShip,
            $class,
            __('Ready to ship', 'woo-dp-internetmarke'),
            ShipmentResourceRepository::instance()->countBy(array('status' => ShipmentStatusEnum::READY_TO_SHIP))
        );

        // Shipped
        $class      = $status === 'shipped' ? ' class="current"' : '';
        $urlShipped = admin_url(sprintf('admin.php?page=%s&status=%s', self::PAGE, 'shipped'));
        $viewLinks['shipped'] = sprintf(
            '<a href="%s" %s>%s <span class="count">(%d)</span></a>',
            $urlShipped,
            $class,
            __('Shipped', 'woo-dp-internetmarke'),
            ShipmentResourceRepository::instance()->countBy(array('status' => ShipmentStatusEnum::SHIPPED))
        );

        // Delivered
        $class        = $status === 'delivered' ? ' class="current"' : '';
        $urlDelivered = admin_url(sprintf('admin.php?page=%s&status=%s', self::PAGE, 'delivered'));
        $viewLinks['delivered'] = sprintf(
            '<a href="%s" %s>%s <span class="count">(%d)</span></a>',
            $urlDelivered,
            $class,
            __('Delivered', 'woo-dp-internetmarke'),
            ShipmentResourceRepository::instance()->countBy(array('status' => ShipmentStatusEnum::DELIVERED))
        );

        // Refunded
        $class       = $status === 'refunded' ? ' class="current"' : '';
        $urlRefunded = admin_url(sprintf('admin.php?page=%s&status=%s', self::PAGE, 'refunded'));
        $viewLinks['refunded'] = sprintf(
            '<a href="%s" %s>%s <span class="count">(%d)</span></a>',
            $urlRefunded,
            $class,
            __('Refunded', 'woo-dp-internetmarke'),
            ShipmentResourceRepository::instance()->countBy(array('status' => ShipmentStatusEnum::REFUNDED))
        );

        // Returned
        $class       = $status === 'returned' ? ' class="current"' : '';
        $urlReturned = admin_url(sprintf('admin.php?page=%s&status=%s', self::PAGE, 'returned'));
        $viewLinks['returned'] = sprintf(
            '<a href="%s" %s>%s <span class="count">(%d)</span></a>',
            $urlReturned,
            $class,
            __('Returned', 'woo-dp-internetmarke'),
            ShipmentResourceRepository::instance()->countBy(array('status' => ShipmentStatusEnum::RETURNED))
        );

        // Failed
        $class     = $status === 'failed' ? ' class="current"' : '';
        $urlFailed = admin_url(sprintf('admin.php?page=%s&status=%s', self::PAGE, 'failed'));
        $viewLinks['failed'] = sprintf(
            '<a href="%s" %s>%s <span class="count">(%d)</span></a>',
            $urlFailed,
            $class,
            __('Failed', 'woo-dp-internetmarke'),
            ShipmentResourceRepository::instance()->countBy(array('status' => ShipmentStatusEnum::FAILED))
        );

        return $viewLinks;
    }

    /**
     * Determines which columns can be sorted by.
     *
     * @return array
     */
    public function get_sortable_columns()
    {
        return array(
            'recipient'        => array('recipient',        true),
            'package_info'     => array('order_id',         true),
            'status_update_dt' => array('status_update_dt', true)
        );
    }

    /**
     * @return array
     */
    public function get_bulk_actions()
    {
        $status  = 'prepared';

        if ($this->isViewFilterActive()) {
            $status = $_GET['status'];
        }

        $actions = array(
            'export_csv' => __('Export (.csv)', 'woo-dp-internetmarke')
        );

        switch ($status) {
            case 'prepared':
                $actions['delete'] = __('Delete', 'woo-dp-internetmarke');
                $actions['order']  = __('Create shipping labels', 'woo-dp-internetmarke');
                break;
            case 'ready_to_ship':
                $actions['download_zip']    = __('Download (.zip)', 'woo-dp-internetmarke');
                $actions['download_pdf']    = __('Download (.pdf)', 'woo-dp-internetmarke');
                $actions['refund']          = __('Refund', 'woo-dp-internetmarke');
                $actions['mark_as_shipped'] = __('Mark as shipped', 'woo-dp-internetmarke');
                break;
            case 'shipped':
                $actions['download_zip']      = __('Download (.zip)', 'woo-dp-internetmarke');
                $actions['download_pdf']      = __('Download (.pdf)', 'woo-dp-internetmarke');
                $actions['mark_as_delivered'] = __('Mark as delivered', 'woo-dp-internetmarke');
                break;
            case 'delivered':
                $actions['download_zip']     = __('Download (.zip)', 'woo-dp-internetmarke');
                $actions['download_pdf']     = __('Download (.pdf)', 'woo-dp-internetmarke');
                $actions['mark_as_returned'] = __('Mark as returned', 'woo-dp-internetmarke');
                break;
            case 'returned':
                $actions['download_zip'] = __('Download (.zip)', 'woo-dp-internetmarke');
                $actions['download_pdf'] = __('Download (.pdf)', 'woo-dp-internetmarke');
                break;
            case 'failed':
                $actions['delete'] = __('Delete', 'woo-dp-internetmarke');
                break;
            case 'refunded':
            default:
                break;
        }

        return $actions;
    }

    /**
     * Prepares the shipments for display.
     */
    public function prepare_items()
    {
        $this->_column_headers = array(
            $this->get_columns(),
            array(),
            $this->get_sortable_columns(),
        );

        $this->processCurrentAction();

        $perPage     = $this->get_items_per_page('wcdpi_shipments_per_page', 10);
        $currentPage = $this->get_pagenum();
        $totalItems  = $this->getShipmentsCount();

        $this->set_pagination_args(
            array(
                'total_items' => $totalItems,
                'per_page'    => $perPage
            )
        );

        $this->items = $this->getShipments($perPage, $currentPage);
    }

    /**
     * Checks whether the view filter for the shipment status is set.
     *
     * @return bool
     */
    public function isViewFilterActive()
    {
        return array_key_exists('status', $_GET);
    }

    /**
     * Retrieves the shipments from the database.
     *
     * @param int $perPage
     * @param int $pageNumber
     *
     * @return array
     */
    public function getShipments($perPage = 10, $pageNumber = 1)
    {
        $wpdb  = $this->wpdb;
        $order = 'DESC';
        $sql   = $wpdb->prepare(
            "SELECT * FROM $this->shipment WHERE status NOT LIKE %d",
            ShipmentStatusEnum::DELETED
        );

        if ($this->isViewFilterActive()) {
            $sql .= $wpdb->prepare(
                ' AND status = %d',
                ShipmentStatusEnum::$status[sanitize_text_field($_GET['status'])]
            );
        } else {
            $sql .= $wpdb->prepare(' AND status = %d', ShipmentStatusEnum::PREPARED);
        }

        if (!empty($_REQUEST['order'])) {
            $order = strtoupper($_REQUEST['order']);
        }

        $orderBy = 'id ' . $order;

        if (!empty($_REQUEST['orderby'])) {
            $orderBy = $_REQUEST['orderby'] . ' ' . $order;

            if ($_REQUEST['orderby'] === 'recipient') {
                $orderBy = 'recipient_last_name ' . $order . ', recipient_company_name ' . $order;
            }
        }

        $sql .= ' ORDER BY ' . (empty($_REQUEST['orderby']) ? 'id '. $order : esc_sql($orderBy));
        $sql .= " LIMIT {$perPage}";
        $sql .= ' OFFSET ' . ($pageNumber - 1) * $perPage;

        return $wpdb->get_results($sql, ARRAY_A);
    }

    /**
     * Retrieves the number of shipments for a specific status.
     *
     * @return int
     */
    public function getShipmentsCount()
    {
        if ($this->isViewFilterActive()) {
            return ShipmentResourceRepository::instance()->countBy(
                array(
                    'status' => ShipmentStatusEnum::$status[$_GET['status']]
                )
            );
        }

        return ShipmentResourceRepository::instance()->countBy(
            array(
                'status' => ShipmentStatusEnum::PREPARED
            )
        );
    }

    /**
     * Central method for sorting through and forwarding shipment actions.
     */
    public function processCurrentAction()
    {
        $action = $this->current_action();

        switch ($action) {
            case 'download_zip':
                $this->exportShipmentsZIP('download_zip');
                break;
            case 'download_pdf':
                $this->exportShipmentsPDF('download_pdf');
                break;
            case 'order':
                $this->orderLabels('order');
                break;
            case 'refund':
                $this->refundLabels('refund');
                break;
            case 'delete':
                $this->deleteShipments('delete');
                break;
            case 'mark_as_shipped':
                $this->changeShipmentStatus('mark_as_shipped', ShipmentStatusEnum::SHIPPED);
                break;
            case 'mark_as_delivered':
                $this->changeShipmentStatus('mark_as_delivered', ShipmentStatusEnum::DELIVERED);
                break;
            case 'mark_as_returned':
                $this->changeShipmentStatus('mark_as_returned', ShipmentStatusEnum::RETURNED);
                break;
            case 'export_csv':
                $this->exportCsv();
                break;
            default:
                break;
        }
    }

    /**
     * Verifies that the currently used nonce is valid.
     *
     * @param string $nonce
     */
    private function verifyNonce($nonce)
    {
        if (!wp_verify_nonce($_REQUEST['_wpnonce'], $nonce)
            && !wp_verify_nonce($_REQUEST['_wpnonce'], 'bulk-' . $this->_args['plural'])
        ) {
            Notice::error(__('Invalid nonce', 'woo-dp-internetmarke'));
            wp_redirect(admin_url(sprintf('admin.php?page=%s', self::PAGE)));
            exit();
        }
    }

    /**
     * Verifies if the user has permissions to use the plugin functionality
     */
    private function verifyPermissions()
    {
        if (!current_user_can('manage_internetmarke')) {
            Notice::error(__('You do not have sufficient permission to use the Internetmarke plugin.', 'woo-dp-internetmarke'));
            wp_redirect(admin_url(sprintf('admin.php?page=%s', self::PAGE)));
            exit();
        }
    }

    /**
     * Downloads the selected shipment's shipping labels as a .zip file.
     *
     * @param string $nonce
     */
    private function exportShipmentsZIP($nonce)
    {
        $this->verifyNonce($nonce);
        $this->verifyPermissions();

        $shipmentIds = (array)$_REQUEST['id'];
        $pdfStrings  = array();

        /** @var ShipmentAttachmentResourceModel[] $shipmentAttachments */
        $shipmentAttachments = ShipmentAttachmentResourceRepository::instance()->findAllBy(
            array(
                'shipment_id' => $shipmentIds,
                'type' => ShipmentAttachmentEnum::LABEL
            ),
            'shipment_id',
            'DESC'
        );

        /** @var ShipmentAttachmentResourceModel $shipmentAttachment */
        foreach ($shipmentAttachments as $shipmentAttachment) {
            if (strpos($shipmentAttachment->getUrl(), 'wcdpi_download_packet_plus_label')) {
                /** @var ShipmentResourceModel $shipment */
                $shipment = ShipmentResourceRepository::instance()->find($shipmentAttachment->getShipmentId());

                /** @var GetItemLabelResponse $label */
                $label = $this->packetPlusHostIntegration->getLabel($shipment->getPpShipmentId());

                if ($label) {
                    $pdfStrings[$shipmentAttachment->getId()] = $label->getLabel();
                }
            } else {
                $pdfStrings[$shipmentAttachment->getId()] = file_get_contents($shipmentAttachment->getUrl());
            }
        }

        $file = @tempnam('tmp', 'zip');
        $zip = new ZipArchive();

        if ($zip->open($file, ZipArchive::CREATE) !== true) {
            exit('Cannot open ZIP archive using the PHP ZipArchive class.');
        }

        ob_clean();

        foreach ($shipmentAttachments as $shipmentAttachment) {
            try {
                $date = new DateTime($shipmentAttachment->getMetaInsertDt());
            } catch (Exception $e) {
                $date = 'NO_DATE_AVAILABLE';
            }

            $fileName = sprintf(
                '%s-%s.pdf',
                str_pad($shipmentAttachment->getShipmentId(), 8, '0', STR_PAD_LEFT),
                $date->format('Y_m_d-H_i')
            );

            $zip->addFromString($fileName, $pdfStrings[$shipmentAttachment->getId()]);
        }

        $zip->close();

        header('Content-Type: application/zip');
        header('Content-Length: ' . filesize($file));
        header('Content-Disposition: attachment; filename="internetmarke_export_' . date('Y_m_d-H_i_s') . '.zip"');
        readfile($file);
        unlink($file);
        exit();
    }

    /**
     * Downloads the selected shipment's shipping labels as a .zip file.
     *
     * @param string $nonce
     */
    private function exportShipmentsPDF($nonce)
    {
        $this->verifyNonce($nonce);
        $this->verifyPermissions();

        $shipmentIds = (array)$_REQUEST['id'];
        $pdfStrings  = array();

        /** @var ShipmentAttachmentResourceModel[] $shipmentAttachments */
        $shipmentAttachments = ShipmentAttachmentResourceRepository::instance()->findAllBy(
            array(
                'shipment_id' => $shipmentIds,
                'type' => ShipmentAttachmentEnum::LABEL
            ),
            'shipment_id',
            'DESC'
        );

        /** @var ShipmentAttachmentResourceModel $shipmentAttachment */
        foreach ($shipmentAttachments as $shipmentAttachment) {
            if (strpos($shipmentAttachment->getUrl(), 'wcdpi_download_packet_plus_label')) {
                /** @var ShipmentResourceModel $shipment */
                $shipment = ShipmentResourceRepository::instance()->find($shipmentAttachment->getShipmentId());

                /** @var GetItemLabelResponse $label */
                $label = $this->packetPlusHostIntegration->getLabel($shipment->getPpShipmentId());

                if ($label) {
                    $pdfStrings[$shipmentAttachment->getId()] = $label->getLabel();
                }
            }

            else {
                $pdfStrings[$shipmentAttachment->getId()] = file_get_contents($shipmentAttachment->getUrl());
            }
        }

        $pdf = new Fpdi();

        ob_clean();
        foreach ($shipmentAttachments as $shipmentAttachment) {
            try {
                $pageCount = $pdf->setSourceFile(StreamReader::createByString($pdfStrings[$shipmentAttachment->getId()]));
            } catch (PdfParserException $pdfpe) {
                continue;
            }

            for ($pageNo = 1; $pageNo <= $pageCount; $pageNo++) {
                try {
                    $templateId = $pdf->importPage($pageNo);
                } catch (PdfReaderException $pdfre) {
                    continue;
                } catch (PdfTypeException $pdfte) {
                    continue;
                } catch (PdfParserException $pdfpe) {
                    continue;
                }

                $size = $pdf->getTemplateSize($templateId);

                $pdf->AddPage($size['orientation'], $size);
                $pdf->useTemplate($templateId);
            }
        }

        $pdf->output('D', 'wcdpi_export_' . date('Y_m_d-H_i_s') . '.pdf');
        exit();
    }

    /**
     * Permanently deletes the selected shipments.
     *
     * @param string $nonce
     */
    private function deleteShipments($nonce)
    {
        $this->verifyNonce($nonce);
        $this->verifyPermissions();

        $shipmentIds = (array)$_REQUEST['id'];

        if (count($shipmentIds) <= 0) {
            Notice::warning('No shipments were selected.');
            wp_redirect(admin_url('admin.php?page=internetmarke_shipments'));
            exit();
        }

        $shipmentsDeleted = Internetmarke()->shipmentService->delete($shipmentIds);

        Notice::success(sprintf('%d shipments deleted successfully.', $shipmentsDeleted));
        wp_redirect(admin_url('admin.php?page=internetmarke_shipments'));
        exit();
    }

    /**
     * Orders the shipping labels for the selected shipments.
     *
     * @param string $nonce
     */
    private function orderLabels($nonce)
    {
        // Security
        $this->verifyNonce($nonce);
        $this->verifyPermissions();

        $shipmentIds = (array)$_REQUEST['id'];
        $shipmentsCreated = 0;

        if (count($shipmentIds) <= 0) {
            Notice::warning('No shipments were selected.');
            wp_redirect(admin_url('admin.php?page=internetmarke_shipments'));
            exit();
        }

        foreach ($shipmentIds as $shipmentId) {
            $shipment = ShipmentResourceRepository::instance()->find($shipmentId);

            if (!$shipment) {
                continue;
            }

            try {
                Internetmarke()->shipmentService->order($shipment->getId());
            } catch (Exception $e) {
                wcdpi_set_shipment_status_failed($shipmentId);
                wcdpi_add_shipment_failed_order_note($shipment, $e);
                Internetmarke()->logger()->error($e->getMessage());
                continue;
            }

            $shipmentsCreated++;
        }

        if ($shipmentsCreated === 0) {
            Notice::error('No shipping labels were created.');
            wp_redirect(admin_url('admin.php?page=internetmarke_shipments'));
            exit();
        }

        Notice::success(sprintf('%d shipments created successfully.', $shipmentsCreated));
        wp_redirect(admin_url('admin.php?page=internetmarke_shipments'));
        exit();
    }

    /**
     * Refunds the shipping labels for the selected shipments.
     *
     * @param string $nonce
     */
    private function refundLabels($nonce)
    {
        // Security
        $this->verifyNonce($nonce);
        $this->verifyPermissions();

        $shipmentIds = (array)$_REQUEST['id'];
        $shipmentsRefunded = 0;

        if (count($shipmentIds) <= 0) {
            Notice::warning('No shipments were selected.');
            wp_redirect(admin_url('admin.php?page=internetmarke_shipments'));
            exit();
        }

        foreach ($shipmentIds as $shipmentId) {
            /** @var ShipmentResourceModel $shipment */
            $shipment = ShipmentResourceRepository::instance()->find($shipmentId);

            if ($shipment->isHarmonizedLabel()) {
                continue;
            }

            /** @var RetoureVouchers $refund */
            $refund = apply_filters('wcdpi_integration_oneclickforrefund_refund_label', $shipment);

            if ($refund->getRetoureTransactionId()) {
                $shipmentData = array(
                    'retoure_transaction_id' => $refund->getRetoureTransactionId(),
                    'status' => ShipmentStatusEnum::REFUNDED
                );

                ShipmentResourceRepository::instance()->update($shipmentId, $shipmentData);
                $shipmentsRefunded++;
            }
        }

        Notice::success(sprintf('%d shipments refunded successfully.', $shipmentsRefunded));
        wp_redirect(admin_url('admin.php?page=internetmarke_shipments'));
        exit();
    }

    /**
     * Performs a status change for the selected shipments.
     *
     * @param string $nonce
     * @param int    $status
     */
    private function changeShipmentStatus($nonce, $status)
    {
        $this->verifyNonce($nonce);
        $this->verifyPermissions();

        $shipmentIds = (array)$_REQUEST['id'];

        switch ($status) {
            case ShipmentStatusEnum::DELETED:
                $shipmentStatus = ShipmentStatusEnum::PREPARED;
                break;
            case ShipmentStatusEnum::SHIPPED:
                $shipmentStatus = ShipmentStatusEnum::READY_TO_SHIP;
                break;
            case ShipmentStatusEnum::DELIVERED:
                $shipmentStatus = ShipmentStatusEnum::SHIPPED;
                break;
            case ShipmentStatusEnum::RETURNED:
                $shipmentStatus = ShipmentStatusEnum::DELIVERED;
                break;
            default:
                $shipmentStatus = -1;
                break;
        }

        foreach ($shipmentIds as $shipmentId) {
            /** @var ShipmentResourceModel $shipment */
            $shipment = ShipmentResourceRepository::instance()->find($shipmentId);

            if (!$shipment || intval($shipment->getStatus()) !== $shipmentStatus) {
                continue;
            }

            ShipmentResourceRepository::instance()->update(
                $shipmentId,
                array(
                    'status'           => $status,
                    'status_update_dt' => date('Y-m-d H:i:s')
                )
            );
        }

        $queryArgs = array(
            'page'                         => self::PAGE,
            'wcdpi_shipment_status_update' => 1
        );

        if (!empty($_REQUEST['status'])) {
            $queryArgs['status'] = $_REQUEST['status'];
        }

        // Redirect
        $url = add_query_arg($queryArgs, admin_url() . 'admin.php');
        wp_redirect(htmlspecialchars_decode($url));
        exit();
    }

    private function exportCsv()
    {
        $this->verifyNonce('export_csv');
        do_action('wcdpi_export_shipments_csv', (array)$_REQUEST['id']);
    }
}
