<?php

namespace DeutschePost\Internetmarke\Connectors\Abstracts;

defined ('ABSPATH') || exit;

abstract class Adapter
{
    private $namespace       = '';
    private $connector       = null;
    private $requestFactory  = null;
    private $responseFactory = null;

    private function getNamespace()
    {
        if (empty($this->namespace)) {
            $className = get_class($this);
            $this->namespace = substr(
                $className,
                0,
                strrpos($className, '\\') + 1
            );
        }

        return $this->namespace;
    }

    private function getConnector()
    {
        if (empty($this->connector)) {
            $connectorClassName = $this->getNamespace() . 'Connector';

            if (class_exists($connectorClassName)) {
                $this->connector = new $connectorClassName();
            }
        }

        return $this->connector;
    }

    private function getRequestFactory()
    {
        if (empty($this->requestFactory)) {
            $requestFactoryClassName = $this->getNamespace() . 'RequestFactory';

            if (class_exists($requestFactoryClassName)) {
                $this->requestFactory = new $requestFactoryClassName();
            }
        }

        return $this->requestFactory;
    }

    private function getResponseFactory()
    {
        if (empty($this->responseFactory)) {
            $responseFactoryClassName = $this->getNamespace() . 'ResponseFactory';

            if (class_exists($responseFactoryClassName)) {
                $this->responseFactory = new $responseFactoryClassName();
            }
        }

        return $this->responseFactory;
    }

    final public function __call($name, $arguments) {
        $request = $this->getRequestFactory()->create($name, $arguments);

        if ($request) {
            $result = $this->getConnector()->execute($request);

            if ($result) {
                return $this->getRequestFactory()->create($name, (array) $result);
            }
        }

        return null;
    }
}