<?php

namespace DeutschePost\Internetmarke\Connectors\Abstracts;

use Exception;
use DeutschePost\Internetmarke\Connectors\Abstracts\Request as AbstractRequest;
use DeutschePost\Internetmarke\Logger;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

abstract class Connector
{
    /**
     * @var
     */
    protected $requestFactory;

    /**
     * @var
     */
    protected $responseFactory;

    /**
     * @param string $partnerID
     * @param string $timestamp
     * @param int    $keyPhase
     * @param string $marketplaceKey
     *
     * @return bool|string
     */
    final protected static function calculateSignature($partnerID, $timestamp, $keyPhase, $marketplaceKey)
    {
        return substr(
            md5(
                join(
                    '::',
                    array(
                        $partnerID,
                        $timestamp,
                        $keyPhase,
                        $marketplaceKey
                    )
                )
            ),
            0,
            8
        );
    }

    /**
     * @return false|string
     */
    final protected static function getTimestamp()
    {
        return date_format(
            date_create(
                'now',
                timezone_open('Europe/Berlin')
            ),
            'dmY-His'
        );
    }

    /**
     * @param string $name
     * @param array  $arguments
     *
     * @throws Exception
     * @return mixed
     */
    public function __call($name, $arguments)
    {
        $request  = $this->requestFactory->create($name, $arguments);
        $response = $this->execute($request);
        $logger   = Internetmarke()->logger(Logger::LOG_INTERNETMARKE_API);

        $logger->notice(sprintf('%s, %s', get_class($this), $name));

        if (property_exists($response, 'type') && $response->type === 'error') {
            $error   = $response->message;
            $context = array();

            if (property_exists($response, 'validationErrors')
                && is_array($response->validationErrors)
            ) {
                $context = $response->validationErrors;
            }

            $logger->error($error, $context);
            throw new Exception($error['message']);
        }

        return $this->responseFactory->create($name, $response);
    }

    /**
     * @param AbstractRequest $request
     *
     * @return mixed
     */
    abstract public function execute(AbstractRequest $request);
}