<?php

namespace DeutschePost\Internetmarke\Connectors\Abstracts;

use Exception;

defined ('ABSPATH') || exit;

abstract class RequestFactory
{
    private $namespace;

    private function getNamespace()
    {
        if (empty($this->namespace)) {
            $className = get_class($this);
            $this->namespace = substr(
                $className,
                0,
                strrpos($className, '\\') + 1
            );
        }

        return $this->namespace;
    }

    final public function create($name, $arguments)
    {
        $requestClassName = $this->getNamespace() . 'Request\\' . ucfirst($name);
        $responseClassName = $this->getNamespace() . 'Response\\' . ucfirst($name);

        if (!class_exists($requestClassName)) {
            throw new Exception(sprintf('Request class does not exist for the %s method', $name));
        }

        if (!class_exists($responseClassName)) {
            throw new Exception(sprintf('Response class does not exist for the %s method', $name));
        }

        if (empty($arguments)) {
            return new $requestClassName($arguments);
        }

        return new $requestClassName($arguments[0]);
    }
}