<?php

namespace DeutschePost\Internetmarke\Connectors\Abstracts\SOAP;

use SoapClient;
use SoapFault;
use SoapHeader;
use DeutschePost\Internetmarke\Connectors\Abstracts\Connector as AbstractConnector;
use DeutschePost\Internetmarke\Connectors\Abstracts\Request as AbstractRequest;
use WsdlToPhp\WsSecurity\WsSecurity;

defined ('ABSPATH') || exit;

abstract class Connector extends AbstractConnector
{
    private $soapClient = null;

    private static function getSOAPHeader(
        $namespace,
        $partnerID,
        $keyPhase,
        $marketplaceKey
    ) {
        $timestamp = self::getTimestamp();

        return array(
            new SoapHeader($namespace, 'PARTNER_ID', $partnerID),
            new SoapHeader($namespace, 'REQUEST_TIMESTAMP', $timestamp),
            new SoapHeader($namespace, 'KEY_PHASE', $keyPhase),
            new SoapHeader($namespace, 'PARTNER_SIGNATURE', self::calculateSignature(
                $partnerID,
                $timestamp,
                $keyPhase,
                $marketplaceKey
            ))
        );
    }

    /**
     * @param $wsdl
     * @param $namespace
     * @param $partnerID
     * @param $keyPhase
     * @param $marketplaceKey
     * @throws SoapFault
     */
    final protected function init($wsdl, $namespace, $partnerID, $keyPhase, $marketplaceKey)
    {
        $context = stream_context_create(
            array(
                'ssl' => array(
                    'crypto_method' => STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT,
                    'verify_peer' => false,
                    'verify_peer_name' => false,
                    'allow_self_signed' => true
                )
            )
        );

        $this->soapClient = new SoapClient($wsdl, array('stream_context' => $context));

        $this->soapClient->__setSoapHeaders(
            self::getSOAPHeader(
                $namespace,
                $partnerID,
                $keyPhase,
                $marketplaceKey
            )
        );
    }

    /**
     * @param $wsdl
     * @param $username
     * @param $password
     * @throws SoapFault
     */
    final protected function initWSS($wsdl, $username, $password)
    {
        $this->soapClient = new SoapClient($wsdl);

        $this->soapClient->__setSoapHeaders(
            WsSecurity::createWsSecuritySoapHeader($username, $password)
        );
    }

    final public function execute(AbstractRequest $request)
    {
        return $this->soapClient->__soapCall(
            $request->getMethodName(),
            $request->getArguments()
        );
    }
}