<?php

namespace DeutschePost\Internetmarke\Connectors\Abstracts\cURL;

use DeutschePost\Internetmarke\Connectors\Abstracts\Connector as AbstractConnector;
use DeutschePost\Internetmarke\Connectors\Abstracts\Request as AbstractRequest;

defined ('ABSPATH') || exit;

abstract class Connector extends AbstractConnector
{
    /**
     * @var array
     */
    protected $lastRequest;

    /**
     * @var string
     */
    protected $lastError;

    /**
     * @var string
     */
    private $baseUrl;

    /**
     * @var string
     */
    private $auth;

    /**
     * @var array
     */
    private $header;

    /**
     * @param string $partnerId
     * @param int    $keyPhase
     * @param string $marketplaceKey
     *
     * @return array
     */
    private static function getHTTPHeader($partnerId, $keyPhase, $marketplaceKey)
    {
        $timestamp = self::getTimestamp();

        if (defined('PACKETPLUS_QA')) {
            return array(
                'PARTNER_ID: DP_LT',
                'REQUEST_TIMESTAMP: 16082018-122210',
                'KEY_PHASE: 1',
                'PARTNER_SIGNATURE: 9d7c35be'
            );
        }

        return array(
            'PARTNER_ID: '        . $partnerId,
            'REQUEST_TIMESTAMP: ' . $timestamp,
            'KEY_PHASE: '         . $keyPhase,
            'PARTNER_SIGNATURE: ' . self::calculateSignature($partnerId, $timestamp, $keyPhase, $marketplaceKey)
        );
    }

    final protected function init($baseURL, $partnerId, $keyPhase, $marketplaceKey)
    {
        $this->baseUrl = $baseURL;

        $this->header = self::getHTTPHeader($partnerId, $keyPhase, $marketplaceKey);
    }

    final protected function initForkLane($baseURL, $serviceKey, $serviceID)
    {
        $this->baseUrl = $baseURL;

        $this->auth = '?Key=' . $serviceKey . '&Id=' . $serviceID;
    }

    final public function execute(AbstractRequest $request)
    {
        $curl = curl_init();

        curl_setopt($curl, CURLOPT_URL, $this->baseUrl . $request->getRequestURL() . $this->auth);
        curl_setopt_array($curl, $request->getOptions());
        curl_setopt($curl, CURLOPT_HTTPHEADER, array_merge($this->header, $request->getHeader()));

        $result            = curl_exec($curl);
        $this->lastRequest = curl_getinfo($curl);
        $this->lastError   = curl_error($curl);

        curl_close($curl);

        return $result;
    }

    final protected function getLastRequest()
    {
        return $this->lastRequest;
    }

    final protected function getLastError()
    {
        return $this->lastError;
    }
}