<?php

namespace DeutschePost\Internetmarke\Connectors\Abstracts\cURL;

use DeutschePost\Internetmarke\Connectors\Abstracts\Request as AbstractRequest;

defined ('ABSPATH') || exit;

abstract class Request extends AbstractRequest
{
    private $header = array(
        'cache-control: no-cache'
    );

    private $options = array(
        CURLOPT_HEADER         => false,
        CURLOPT_VERBOSE        => true,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_TIMEOUT        => 10,
        CURLOPT_SSL_VERIFYPEER => false,
        CURLOPT_SSL_VERIFYHOST => false,
        CURLINFO_HEADER_OUT    => true
    );

    private $requestType = RequestType::GET;

    private $requestURL = '';

    final protected function addHeader($entry)
    {
        if (!$entry) {
            return false;
        }

        array_push($this->header, $entry);

        return true;
    }

    final protected function removeHeader($entry)
    {
        if (!$entry) {
            return false;
        }

        $index = array_search($entry, $this->header, true);

        if ($index !== false) {
            unset($this->header[$index]);
            return true;
        }

        return false;
    }

    final public function getHeader()
    {
        return $this->header;
    }

    final protected function addOption($key, $value)
    {
        if (is_null($key) || is_null($value)) {
            return false;
        }

        $this->options[$key] = $value;

        return true;
    }

    final protected function removeOption($key)
    {
        if (is_null($key)) {
            return false;
        }

        if (array_key_exists($key, $this->options)) {
            unset($this->options[$key]);
            return true;
        }

        return false;
    }

    final public function getOptions()
    {
        if($this->requestType === RequestType::POST){
            $this->addOption(CURLOPT_POST, true);
        }
        elseif ($this->requestType === RequestType::PUT) {
            $this->addOption(CURLOPT_PUT, true);
        }
        elseif ($this->requestType === RequestType::DELETE) {
            $this->addOption(CURLOPT_CUSTOMREQUEST, 'DELETE');
        }

        return $this->options;
    }

    final protected function addData($data, $dataType)
    {
        if(is_null($data) || is_null($dataType)){
            return false;
        }

        if ($dataType === DataType::JSON) {
            $this->addOption(CURLOPT_POSTFIELDS, json_encode($data));
            $this->addHeader("Content-Type: application/json");
        }
        elseif ($dataType === DataType::URL_ENCODED) {
            $this->addOption(
                CURLOPT_POSTFIELDS,
                http_build_query(
                    $data,
                    null,
                    null,
                    PHP_QUERY_RFC1738
                )
            );
            $this->addHeader("Content-Type: application/x-www-form-urlencoded");
        }
        else {
            $this->addOption(CURLOPT_POSTFIELDS, $data);
        }

        return true;
    }

    final protected function addBasicAuthentication($username, $password)
    {
        if(is_null($username) || is_null($password)){
            return false;
        }

        $this->addOption(CURLOPT_USERPWD, $username . ":" . $password);

        return true;
    }

    final protected function addBearerAuthentication($token)
    {
        if (is_null($token)) {
            return false;
        }

        $this->addHeader("Authorization: Bearer " . $token);

        return true;
    }

    final protected function setRequestType($requestType)
    {
        $this->requestType = $requestType;
    }

    final protected function setRequestURL($requestURL)
    {
        $this->requestURL = $requestURL;
    }

    final public function getRequestURL()
    {
        return $this->requestURL;
    }
}