<?php

namespace DeutschePost\Internetmarke\Connectors\OneClickForApp;

use SoapFault;
use DeutschePost\Internetmarke\Connectors\Abstracts\SOAP\Connector as AbstractConnector;
use DeutschePost\Internetmarke\Connectors\Interfaces\Connector as ConnectorInterface;

/**
 * @method authenticateUser(array $args)
 * @method checkoutShoppingCartPDF(array $args)
 * @method createShopOrderId(array $args)
 * @method retrieveContractProducts(array $args)
 * @method retrieveOrder(array $args)
 * @method retrievePageFormats()
 * @method retrievePreviewVoucherPDF(array $args)
 * @method retrievePreviewVoucherPNG(array $args)
 * @method retrievePrivateGallery(array $args)
 * @method retrievePublicGallery()
 */
class Connector extends AbstractConnector implements ConnectorInterface
{
    /**
     * @var string
     */
    const SERVICE_WSDL_URL = 'https://internetmarke.deutschepost.de/OneClickForAppV3?wsdl';

    /**
     * @var string
     */
    const SERVICE_NAMESPACE = 'http://oneclickforapp.dpag.de/V3';

    /**
     * @var RequestFactory
     */
    protected $requestFactory;

    /**
     * @var ResponseFactory
     */
    protected $responseFactory;

    /**
     * @var integer
     */
    protected $partnerId;

    /**
     * @var integer
     */
    protected $keyPhase;

    /**
     * @var string
     */
    protected $marketplaceKey;

    /**
     * Connector constructor.
     *
     * @param int    $partnerId      Merchant Partner ID
     * @param int    $keyPhase       Merchant Key Phase
     * @param string $marketplaceKey Merchant Marketplace Key
     * @throws SoapFault
     */
    public function __construct($partnerId, $keyPhase, $marketplaceKey)
    {
        $this->requestFactory = new RequestFactory();
        $this->responseFactory = new ResponseFactory();

        $this->setPartnerId($partnerId);
        $this->setKeyPhase($keyPhase);
        $this->setMarketplaceKey($marketplaceKey);
        
        $this->init(
            self::SERVICE_WSDL_URL,
            self::SERVICE_NAMESPACE,
            $partnerId,
            $keyPhase,
            $marketplaceKey
        );
    }

    /**
     * @return int
     */
    public function getPartnerId()
    {
        return $this->partnerId;
    }

    /**
     * @param int $partnerId
     */
    public function setPartnerId($partnerId)
    {
        $this->partnerId = $partnerId;
    }

    /**
     * @return int
     */
    public function getKeyPhase()
    {
        return $this->keyPhase;
    }

    /**
     * @param int $keyPhase
     */
    public function setKeyPhase($keyPhase)
    {
        $this->keyPhase = $keyPhase;
    }

    /**
     * @return string
     */
    public function getMarketplaceKey()
    {
        return $this->marketplaceKey;
    }

    /**
     * @param string $marketplaceKey
     */
    public function setMarketplaceKey($marketplaceKey)
    {
        $this->marketplaceKey = $marketplaceKey;
    }
}