<?php

namespace DeutschePost\Internetmarke\Connectors\OneClickForApp\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\SOAP\Response as AbstractResponse;

defined ('ABSPATH') || exit;

class AuthenticateUser extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $userToken;

    /**
     * @var int
     */
    protected $walletBalance;

    /**
     * @var bool
     */
    protected $showTermsAndConditions;

    /**
     * @var string
     */
    protected $infoMessage;

    /**
     * AuthenticateUser constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse($responseObject);
        $this->parseResponse($responseObject);
    }

    /**
     * @param stdClass $responseObject
     *
     * @return mixed|void
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        if (!property_exists($responseObject, 'userToken')) {
            throw new Exception('The userToken property is missing from the response object');
        }

        if (!property_exists($responseObject, 'walletBalance')) {
            throw new Exception('The walletBalance property is missing from the response object');
        }

        if (!property_exists($responseObject, 'showTermsAndConditions')) {
            throw new Exception('The showTermsAndConditions property is missing from the response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @return mixed|void
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        $this->setUserToken($responseObject->userToken);
        $this->setWalletBalance($responseObject->walletBalance);
        $this->setShowTermsAndConditions($responseObject->showTermsAndConditions);

        if (property_exists($responseObject, 'infoMessage')) {
            $this->setInfoMessage($responseObject->infoMessage);
        }
    }

    /**
     * @return string
     */
    public function getUserToken()
    {
        return $this->userToken;
    }

    /**
     * @param $value
     */
    public function setUserToken($value)
    {
        $this->userToken = $value;
    }

    /**
     * @return int
     */
    public function getWalletBalance()
    {
        return $this->walletBalance;
    }

    /**
     * @param $value
     */
    public function setWalletBalance($value)
    {
        $this->walletBalance = $value;
    }

    /**
     * @return bool
     */
    public function getShowTermsAndConditions()
    {
        return $this->showTermsAndConditions;
    }

    /**
     * @param $value
     */
    public function setShowTermsAndConditions($value)
    {
        $this->showTermsAndConditions = $value;
    }

    /**
     * @return string
     */
    public function getInfoMessage()
    {
        return $this->infoMessage;
    }

    /**
     * @param string $infoMessage
     */
    public function setInfoMessage($infoMessage)
    {
        $this->infoMessage = $infoMessage;
    }
}
