<?php

namespace DeutschePost\Internetmarke\Connectors\OneClickForApp\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\SOAP\Response as AbstractResponse;

defined ('ABSPATH') || exit;

class CheckoutShoppingCartPDF extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $link;

    /**
     * @var string
     */
    protected $manifestLink;

    /**
     * @var integer
     */
    protected $walletBallance;

    /**
     * @var stdClass
     */
    protected $shoppingCart;

    /**
     * CheckoutShoppingCartPDF constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject) {
        $this->validateResponse($responseObject);
        $this->parseResponse($responseObject);
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        if (!property_exists($responseObject, 'link')) {
            throw new Exception('The link property is missing from the response object');
        }

        if (!property_exists($responseObject, 'walletBallance')) {
            throw new Exception('The walletBallance property is missing from the response object');
        }

        if (!property_exists($responseObject, 'shoppingCart')) {
            throw new Exception('The shoppingCart property is missing from the response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        $this->setLink($responseObject->link);
        $this->setWalletBallance($responseObject->walletBallance);
        $this->setShoppingCart($responseObject->shoppingCart);

        if (property_exists($responseObject, 'manifestLink')) {
            $this->setManifestLink($responseObject->manifestLink);
        }
    }

    /**
     * @return string
     */
    public function getLink()
    {
        return $this->link;
    }

    /**
     * @param $link
     */
    public function setLink($link)
    {
        $this->link = $link;
    }

    /**
     * @return string
     */
    public function getManifestLink()
    {
        return $this->manifestLink;
    }

    /**
     * @param $manifestLink
     */
    public function setManifestLink($manifestLink)
    {
        $this->manifestLink = $manifestLink;
    }

    /**
     * @return int
     */
    public function getWalletBallance()
    {
        return $this->walletBallance;
    }

    /**
     * @param $walletBallance
     */
    public function setWalletBallance($walletBallance)
    {
        $this->walletBallance = $walletBallance;
    }

    /**
     * @return stdClass
     */
    public function getShoppingCart()
    {
        return $this->shoppingCart;
    }

    /**
     * @param $shoppingCart
     */
    public function setShoppingCart($shoppingCart)
    {
        $this->shoppingCart = $shoppingCart;
    }

}
