<?php

namespace DeutschePost\Internetmarke\Connectors\OneClickForApp\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\SOAP\Response as AbstractResponse;

defined ('ABSPATH') || exit;

class RetrieveOrder extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $link;

    /**
     * @var string
     */
    protected $manifestLink;

    /**
     * @var stdClass
     */
    protected $shoppingCart;

    /**
     * RetrieveOrder constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse($responseObject);
        $this->parseResponse($responseObject);
    }

    /**
     * @param stdClass $responseObject
     *
     * @return mixed|void
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        if (!property_exists($responseObject, 'link')) {
            throw new Exception('The link property is missing from the response object');
        }

        if (!property_exists($responseObject, 'shoppingCart')) {
            throw new Exception('The shoppingCart property is missing from the response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @return mixed|void
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        $this->setLink($responseObject->link);
        $this->setShoppingCart($responseObject->shoppingCart);

        if (property_exists($responseObject, 'manifestLink')) {
            $this->setManifestLink($responseObject->manifestLink);
        }
    }

    /**
     * @return string
     */
    public function getLink()
    {
        return $this->link;
    }

    /**
     * @param $value
     */
    public function setLink($value)
    {
        $this->link = $value;
    }

    /**
     * @return string
     */
    public function getManifestLink()
    {
        return $this->manifestLink;
    }

    /**
     * @param $value
     */
    public function setManifestLink($value)
    {
        $this->manifestLink = $value;
    }

    /**
     * @return stdClass
     */
    public function getShoppingCart()
    {
        return $this->shoppingCart;
    }

    /**
     * @param $value
     */
    public function setShoppingCart($value)
    {
        $this->shoppingCart = $value;
    }
}
