<?php

namespace DeutschePost\Internetmarke\Connectors\OneClickForRefund\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\SOAP\Response as AbstractResponse;

class AuthenticateUser extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $userToken;

    /**
     * AuthenticateUser constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse($responseObject);
        $this->parseResponse($responseObject);
    }

    /**
     * @param stdClass $responseObject
     *
     * @return mixed|void
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        if (!property_exists($responseObject, 'userToken')) {
            throw new Exception('The userToken property is missing from the response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @return mixed|void
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        $this->setUserToken($responseObject->userToken);
    }

    /**
     * @return string
     */
    public function getUserToken()
    {
        return $this->userToken;
    }

    /**
     * @param $value
     */
    public function setUserToken($value)
    {
        $this->userToken = $value;
    }
}
