<?php

namespace DeutschePost\Internetmarke\Connectors\PacketPlus\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\Response as AbstractResponse;

defined('ABSPATH') || exit();

class CreateSingleAwb extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var int
     */
    protected $awbCopyCount;

    /**
     * @var string
     */
    protected $contactName;

    /**
     * @var string
     */
    protected $customerEkp;

    /**
     * @var string
     */
    protected $itemFormat;

    /**
     * @var string
     */
    protected $jobReference;

    /**
     * @var string
     */
    protected $product;

    /**
     * @var string
     */
    protected $serviceLevel;

    /**
     * @var string
     */
    protected $telephoneNumber;

    /**
     * @var float
     */
    protected $totalWeight;

    /**
     * CreateAwb constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject) {
        $this->validateResponse($responseObject);
        $this->parseResponse($responseObject);
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        if (property_exists($responseObject, 'awbCopyCount')) {
            $this->setAwbCopyCount($responseObject->awbCopyCount);
        }

        if (property_exists($responseObject, 'contactName')) {
            $this->setContactName($responseObject->contactName);
        }

        if (property_exists($responseObject, 'customerEkp')) {
            $this->setCustomerEkp($responseObject->customerEkp);
        }

        if (property_exists($responseObject, 'itemFormat')) {
            $this->setItemFormat($responseObject->itemFormat);
        }

        if (property_exists($responseObject, 'jobReference')) {
            $this->setJobReference($responseObject->jobReference);
        }

        if (property_exists($responseObject, 'product')) {
            $this->setProduct($responseObject->product);
        }

        if (property_exists($responseObject, 'serviceLevel')) {
            $this->setServiceLevel($responseObject->serviceLevel);
        }

        if (property_exists($responseObject, 'telephoneNumber')) {
            $this->setTelephoneNumber($responseObject->telephoneNumber);
        }

        if (property_exists($responseObject, 'totalWeight')) {
            $this->setTotalWeight($responseObject->totalWeight);
        }
    }

    /**
     * @return int
     */
    public function getAwbCopyCount()
    {
        return $this->awbCopyCount;
    }

    /**
     * @param int $awbCopyCount
     */
    public function setAwbCopyCount($awbCopyCount)
    {
        $this->awbCopyCount = $awbCopyCount;
    }

    /**
     * @return string
     */
    public function getContactName()
    {
        return $this->contactName;
    }

    /**
     * @param string $contactName
     */
    public function setContactName($contactName)
    {
        $this->contactName = $contactName;
    }

    /**
     * @return string
     */
    public function getCustomerEkp()
    {
        return $this->customerEkp;
    }

    /**
     * @param string $customerEkp
     */
    public function setCustomerEkp($customerEkp)
    {
        $this->customerEkp = $customerEkp;
    }

    /**
     * @return string
     */
    public function getItemFormat()
    {
        return $this->itemFormat;
    }

    /**
     * @param string $itemFormat
     */
    public function setItemFormat($itemFormat)
    {
        $this->itemFormat = $itemFormat;
    }

    /**
     * @return string
     */
    public function getJobReference()
    {
        return $this->jobReference;
    }

    /**
     * @param string $jobReference
     */
    public function setJobReference($jobReference)
    {
        $this->jobReference = $jobReference;
    }

    /**
     * @return string
     */
    public function getProduct()
    {
        return $this->product;
    }

    /**
     * @param string $product
     */
    public function setProduct($product)
    {
        $this->product = $product;
    }

    /**
     * @return string
     */
    public function getServiceLevel()
    {
        return $this->serviceLevel;
    }

    /**
     * @param string $serviceLevel
     */
    public function setServiceLevel($serviceLevel)
    {
        $this->serviceLevel = $serviceLevel;
    }

    /**
     * @return string
     */
    public function getTelephoneNumber()
    {
        return $this->telephoneNumber;
    }

    /**
     * @param string $telephoneNumber
     */
    public function setTelephoneNumber($telephoneNumber)
    {
        $this->telephoneNumber = $telephoneNumber;
    }

    /**
     * @return float
     */
    public function getTotalWeight()
    {
        return $this->totalWeight;
    }

    /**
     * @param float $totalWeight
     */
    public function setTotalWeight($totalWeight)
    {
        $this->totalWeight = $totalWeight;
    }
}