<?php


namespace DeutschePost\Internetmarke\Connectors\PacketPlus\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\Response as AbstractResponse;

defined('ABSPATH') || exit();

class GetAccessToken extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $userToken;

    /**
     * @var int
     */
    protected $walletBalance;

    /**
     * @var bool
     */
    protected $showTac;

    /**
     * GetAccessToken constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $authenticationData = $this->parseAuthenticationData($responseObject);

        $this->validateResponse($authenticationData);
        $this->parseResponse($authenticationData);
    }

    /**
     * @param array $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject || !is_array($responseObject)) {
            throw new Exception('Invalid response object');
        }

        if (!array_key_exists('user_token', $responseObject)) {
            throw new Exception('The user_token property is missing from the response object');
        }

        if (!array_key_exists('wallet_balance', $responseObject)) {
            throw new Exception('The wallet_balance property is missing from the response object');
        }

        if (!array_key_exists('show_tac', $responseObject)) {
            throw new Exception('The show_tac property is missing from the response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param array $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        $this->setUserToken($responseObject['user_token']);
        $this->setWalletBalance($responseObject['wallet_balance']);
        $this->setShowTac($responseObject['show_tac']);
    }

    /**
     * @param stdClass $result
     *
     * @return array
     */
    protected function parseAuthenticationData($result)
    {
        $returnValue = array();

        $regExpUt = "/<userToken>(.+?)<\/userToken>/";

        if (preg_match($regExpUt, $result, $matchUt)) {
            $userToken = $matchUt[1];
            $returnValue["user_token"] = $userToken;
        }

        $reg_exp_wb = "/<walletBalance>(.+?)<\/walletBalance>/";

        if (preg_match($reg_exp_wb, $result, $matchWb)) {
            $walletBalance = $matchWb[1];
            $returnValue['wallet_balance'] = $walletBalance;
        }

        $regExpTac = "/<showTermsAndConditions>(.+?)<\/showTermsAndConditions>/";

        if (preg_match($regExpTac, $result, $matchTac)) {
            $tac = $matchTac[1];
            $returnValue['show_tac'] = $tac;
        }

        return $returnValue;
    }

    /**
     * @return string
     */
    public function getUserToken()
    {
        return $this->userToken;
    }

    /**
     * @param string $userToken
     */
    public function setUserToken($userToken)
    {
        $this->userToken = $userToken;
    }

    /**
     * @return int
     */
    public function getWalletBalance()
    {
        return $this->walletBalance;
    }

    /**
     * @param int $walletBalance
     */
    public function setWalletBalance($walletBalance)
    {
        $this->walletBalance = $walletBalance;
    }

    /**
     * @return bool
     */
    public function isShowTac()
    {
        return $this->showTac;
    }

    /**
     * @param bool $showTac
     */
    public function setShowTac($showTac)
    {
        $this->showTac = $showTac;
    }
}