<?php

namespace DeutschePost\Internetmarke\Connectors\PacketPlus\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\Response as AbstractResponse;

defined('ABSPATH') || exit();

class GetAccessTokenInfo extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $audience;

    /**
     * @var string
     */
    protected $userId;

    /**
     * @var string
     */
    protected $scope;

    /**
     * @var int
     */
    protected $expiresIn;

    /**
     * GetAccessTokenInfo constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse(json_decode($responseObject));
        $this->parseResponse(json_decode($responseObject));
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        if (property_exists($responseObject, 'audience')) {
            $this->setAudience($responseObject->audience);
        }

        if (property_exists($responseObject, 'user_id')) {
            $this->setUserId($responseObject->user_id);
        }

        if (property_exists($responseObject, 'scope')) {
            $this->setScope($responseObject->scope);
        }

        if (property_exists($responseObject, 'expires_in')) {
            $this->setExpiresIn($responseObject->expires_in);
        }
    }

    /**
     * @return string
     */
    public function getAudience()
    {
        return $this->audience;
    }

    /**
     * @param string $audience
     */
    public function setAudience($audience)
    {
        $this->audience = $audience;
    }

    /**
     * @return string
     */
    public function getUserId()
    {
        return $this->userId;
    }

    /**
     * @param string $userId
     */
    public function setUserId($userId)
    {
        $this->userId = $userId;
    }

    /**
     * @return string
     */
    public function getScope()
    {
        return $this->scope;
    }

    /**
     * @param string $scope
     */
    public function setScope($scope)
    {
        $this->scope = $scope;
    }

    /**
     * @return int
     */
    public function getExpiresIn()
    {
        return $this->expiresIn;
    }

    /**
     * @param int $expiresIn
     */
    public function setExpiresIn($expiresIn)
    {
        $this->expiresIn = $expiresIn;
    }
}