<?php

namespace DeutschePost\Internetmarke\Connectors\PacketPlus\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\Response as AbstractResponse;

defined('ABSPATH') || exit();

class GetItemLabel extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $label;

    /**
     * GetItemLabel constructor.
     *
     * @param stdClass|string $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse($responseObject);
        $this->parseResponse($responseObject);
    }

    /**
     * @param stdClass|string $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        if (property_exists($responseObject, 'messages')) {
            $errorMessages = '';

            foreach ($responseObject->messages as $message) {
                $errorMessages .= $message . "|\n";
            }

            throw new Exception($errorMessages);
        }

        if (property_exists($responseObject, 'reasons')) {
            $errorMessages = '';

            foreach ($responseObject->reasons as $reason) {
                $errorMessages .= $reason->msg . "|\n";
            }

            throw new Exception($errorMessages);
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass|string $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        $this->setLabel($responseObject);
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return $this->label;
    }

    /**
     * @param string $label
     */
    public function setLabel($label)
    {
        $this->label = $label;
    }
}