<?php

namespace DeutschePost\Internetmarke\Connectors\PacketPlus\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\Response as AbstractResponse;

defined('ABSPATH') || exit();

class GetOrder extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $customerEkp;

    /**
     * @var int
     */
    protected $orderId;

    /**
     * @var array
     */
    protected $items;

    /**
     * @var string
     */
    protected $orderStatus;

    /**
     * @var stdClass
     */
    protected $paperwork;

    /**
     * @var array
     */
    protected $shipments;

    /**
     * GetOrder constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse(json_decode($responseObject));
        $this->parseResponse(json_decode($responseObject));
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        $this->setCustomerEkp($responseObject->customerEkp);
        $this->setOrderId($responseObject->orderId);
        $this->setShipments($responseObject->shipments);

        if (property_exists($responseObject, 'items')) {
            $this->setItems($responseObject->items);
        }

        if (property_exists($responseObject, 'orderStatus')) {
            $this->setOrderStatus($responseObject->orderStatus);
        }

        if (property_exists($responseObject, 'paperwork')) {
            $this->setPaperwork($responseObject->paperwork);
        }
    }

    /**
     * @return string
     */
    public function getCustomerEkp()
    {
        return $this->customerEkp;
    }

    /**
     * @param string $customerEkp
     */
    public function setCustomerEkp($customerEkp)
    {
        $this->customerEkp = $customerEkp;
    }

    /**
     * @return int
     */
    public function getOrderId()
    {
        return $this->orderId;
    }

    /**
     * @param int $orderId
     */
    public function setOrderId($orderId)
    {
        $this->orderId = $orderId;
    }

    /**
     * @return array
     */
    public function getItems()
    {
        return $this->items;
    }

    /**
     * @param array $items
     */
    public function setItems($items)
    {
        $this->items = $items;
    }

    /**
     * @return string
     */
    public function getOrderStatus()
    {
        return $this->orderStatus;
    }

    /**
     * @param string $orderStatus
     */
    public function setOrderStatus($orderStatus)
    {
        $this->orderStatus = $orderStatus;
    }

    /**
     * @return stdClass
     */
    public function getPaperwork()
    {
        return $this->paperwork;
    }

    /**
     * @param stdClass $paperwork
     */
    public function setPaperwork($paperwork)
    {
        $this->paperwork = $paperwork;
    }

    /**
     * @return array
     */
    public function getShipments()
    {
        return $this->shipments;
    }

    /**
     * @param array $shipments
     */
    public function setShipments($shipments)
    {
        $this->shipments = $shipments;
    }
}