<?php

namespace DeutschePost\Internetmarke\Connectors\PacketPlus\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\Response as AbstractResponse;

defined('ABSPATH') || exit();

class GetShipmentAwb extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $awb;

    /**
     * @var array
     */
    protected $items;

    /**
     * @var stdClass
     */
    protected $order;

    /**
     * GetOrderShipments constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse(json_decode($responseObject));
        $this->parseResponse(json_decode($responseObject));
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        if (property_exists($responseObject, 'awb')) {
            $this->setAwb($responseObject->awb);
        }

        if (property_exists($responseObject, 'items')) {
            $this->setItems($responseObject->items);
        }

        if (property_exists($responseObject, 'order')) {
            $this->setOrder($responseObject->order);
        }
    }

    /**
     * @return string
     */
    public function getAwb()
    {
        return $this->awb;
    }

    /**
     * @param string $awb
     */
    public function setAwb($awb)
    {
        $this->awb = $awb;
    }

    /**
     * @return array
     */
    public function getItems()
    {
        return $this->items;
    }

    /**
     * @param array $items
     */
    public function setItems($items)
    {
        $this->items = $items;
    }

    /**
     * @return stdClass
     */
    public function getOrder()
    {
        return $this->order;
    }

    /**
     * @param stdClass $order
     */
    public function setOrder($order)
    {
        $this->order = $order;
    }
}