<?php

namespace DeutschePost\Internetmarke\Connectors\ProdWS\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\SOAP\Response as AbstractResponse;

class GetProductList extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $date;

    /**
     * @var array
     */
    protected $salesProductList;

    /**
     * @var array
     */
    protected $basicProductList;

    /**
     * @var array
     */
    protected $additionalProductList;

    /**
     * @var array
     */
    protected $specialServiceList;

    /**
     * @var array
     */
    protected $shortSalesProductList;

    /**
     * @var string
     */
    protected $message;

    /**
     * GetProductList constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse($responseObject);
        $this->parseResponse($responseObject);
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        if (!property_exists($responseObject, 'success')) {
            throw new Exception('Invalid response object');
        }

        if (!property_exists($responseObject, 'Response')) {
            throw new Exception('Invalid response object');
        }

        if (!property_exists($responseObject->Response, 'date')) {
            throw new Exception('Property date is missing from the response object');
        }

        if (!property_exists($responseObject->Response, 'salesProductList')) {
            throw new Exception('Property salesProductList is missing from the response object');
        }

        if (!property_exists($responseObject->Response->salesProductList, 'SalesProduct')) {
            throw new Exception('Property SalesProduct is missing from the response object');
        }

        if (!property_exists($responseObject->Response, 'basicProductList')) {
            throw new Exception('Property basicProductList is missing from the response object');
        }

        if (!property_exists($responseObject->Response->basicProductList, 'BasicProduct')) {
            throw new Exception('Property BasicProduct is missing from the response object');
        }

        if (!property_exists($responseObject->Response, 'additionalProductList')) {
            throw new Exception('Property additionalProductList is missing from the response object');
        }

        if (!property_exists($responseObject->Response->additionalProductList, 'AdditionalProduct')) {
            throw new Exception('Property AdditionalProduct is missing from the response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        $this->setDate($responseObject->Response->date);
        $this->setSalesProductList($responseObject->Response->salesProductList->SalesProduct);
        $this->setBasicProductList($responseObject->Response->basicProductList->BasicProduct);
        $this->setAdditionalProductList($responseObject->Response->additionalProductList->AdditionalProduct);

        if (property_exists($responseObject->Response, 'specialServiceList')) {
            $this->setSpecialServiceList($responseObject->Response->specialServiceList);
        }

        if (property_exists($responseObject->Response, 'shortSalesProductList')) {
            $this->setShortSalesProductList($responseObject->Response->shortSalesProductList);
        }

        if (property_exists($responseObject->Response, 'message')) {
            $this->setMessage($responseObject->Response->message);
        }
    }

    /**
     * @return string
     */
    public function getDate()
    {
        return $this->date;
    }

    /**
     * @param string $date
     */
    public function setDate($date)
    {
        $this->date = $date;
    }

    /**
     * @return array
     */
    public function getSalesProductList()
    {
        return $this->salesProductList;
    }

    /**
     * @param array $salesProductList
     */
    public function setSalesProductList($salesProductList)
    {
        $this->salesProductList = $salesProductList;
    }

    /**
     * @return array
     */
    public function getBasicProductList()
    {
        return $this->basicProductList;
    }

    /**
     * @param array $basicProductList
     */
    public function setBasicProductList($basicProductList)
    {
        $this->basicProductList = $basicProductList;
    }

    /**
     * @return array
     */
    public function getAdditionalProductList()
    {
        return $this->additionalProductList;
    }

    /**
     * @param array $additionalProductList
     */
    public function setAdditionalProductList($additionalProductList)
    {
        $this->additionalProductList = $additionalProductList;
    }

    /**
     * @return array
     */
    public function getSpecialServiceList()
    {
        return $this->specialServiceList;
    }

    /**
     * @param array $specialServiceList
     */
    public function setSpecialServiceList($specialServiceList)
    {
        $this->specialServiceList = $specialServiceList;
    }

    /**
     * @return array
     */
    public function getShortSalesProductList()
    {
        return $this->shortSalesProductList;
    }

    /**
     * @param array $shortSalesProductList
     */
    public function setShortSalesProductList($shortSalesProductList)
    {
        $this->shortSalesProductList = $shortSalesProductList;
    }

    /**
     * @return string
     */
    public function getMessage()
    {
        return $this->message;
    }

    /**
     * @param string $message
     */
    public function setMessage($message)
    {
        $this->message = $message;
    }
}
