<?php

namespace DeutschePost\Internetmarke\Connectors\ProdWS\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\SOAP\Response as AbstractResponse;

class GetProductVersionsList extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var array
     */
    protected $salesProductList;

    /**
     * @var array
     */
    protected $basicProductList;

    /**
     * @var array
     */
    protected $additionalProductList;

    /**
     * @var array
     */
    protected $specialServiceList;

    /**
     * @var array
     */
    protected $shortSalesProductList;

    /**
     * @var string
     */
    protected $message;

    /**
     * GetProductVersionsList constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse($responseObject);
        $this->parseResponse($responseObject);
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        if (property_exists($responseObject, 'salesProductList')) {
            $this->setSalesProductList($responseObject->salesProductList);
        }

        if (property_exists($responseObject, 'basicProductList')) {
            $this->setBasicProductList($responseObject->basicProductList);
        }

        if (property_exists($responseObject, 'additionalProductList')) {
            $this->setAdditionalProductList($responseObject->additionalProductList);
        }

        if (property_exists($responseObject, 'specialServiceList')) {
            $this->setSpecialServiceList($responseObject->specialServiceList);
        }

        if (property_exists($responseObject, 'shortSalesProductList')) {
            $this->setShortSalesProductList($responseObject->shortSalesProductList);
        }

        if (property_exists($responseObject, 'message')) {
            $this->setMessage($responseObject->message);
        }
    }

    /**
     * @return array
     */
    public function getSalesProductList()
    {
        return $this->salesProductList;
    }

    /**
     * @param array $salesProductList
     */
    public function setSalesProductList($salesProductList)
    {
        $this->salesProductList = $salesProductList;
    }

    /**
     * @return array
     */
    public function getBasicProductList()
    {
        return $this->basicProductList;
    }

    /**
     * @param array $basicProductList
     */
    public function setBasicProductList($basicProductList)
    {
        $this->basicProductList = $basicProductList;
    }

    /**
     * @return array
     */
    public function getAdditionalProductList()
    {
        return $this->additionalProductList;
    }

    /**
     * @param array $additionalProductList
     */
    public function setAdditionalProductList($additionalProductList)
    {
        $this->additionalProductList = $additionalProductList;
    }

    /**
     * @return array
     */
    public function getSpecialServiceList()
    {
        return $this->specialServiceList;
    }

    /**
     * @param array $specialServiceList
     */
    public function setSpecialServiceList($specialServiceList)
    {
        $this->specialServiceList = $specialServiceList;
    }

    /**
     * @return array
     */
    public function getShortSalesProductList()
    {
        return $this->shortSalesProductList;
    }

    /**
     * @param array $shortSalesProductList
     */
    public function setShortSalesProductList($shortSalesProductList)
    {
        $this->shortSalesProductList = $shortSalesProductList;
    }

    /**
     * @return string
     */
    public function getMessage()
    {
        return $this->message;
    }

    /**
     * @param string $message
     */
    public function setMessage($message)
    {
        $this->message = $message;
    }
}
