<?php

namespace DeutschePost\Internetmarke\Connectors\ProdWS\Response;

use Exception;
use stdClass;
use DeutschePost\Internetmarke\Connectors\Abstracts\SOAP\Response as AbstractResponse;

class RegisterNotification extends AbstractResponse
{
    /**
     * @var bool
     */
    protected $isValidObject = false;

    /**
     * @var string
     */
    protected $mandantID;

    /**
     * @var string
     */
    protected $subMandantID;

    /**
     * @var bool
     */
    protected $registration;

    /**
     * @var string
     */
    protected $registrationDateTime;

    /**
     * @var string
     */
    protected $message;

    /**
     * RegisterNotification constructor.
     *
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    public function __construct($responseObject)
    {
        $this->validateResponse($responseObject);
        $this->parseResponse($responseObject);
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function validateResponse($responseObject)
    {
        if (!$responseObject) {
            throw new Exception('Invalid response object');
        }

        $this->isValidObject = true;
    }

    /**
     * @param stdClass $responseObject
     *
     * @throws Exception
     */
    protected function parseResponse($responseObject)
    {
        if (!$this->isValidObject) {
            throw new Exception('Invalid response object');
        }

        if (property_exists($responseObject, 'mandantID')) {
            $this->setMandantID($responseObject->mandantID);
        }

        if (property_exists($responseObject, 'subMandantID')) {
            $this->setSubMandantID($responseObject->subMandantID);
        }

        if (property_exists($responseObject, 'registration')) {
            $this->setRegistration($responseObject->registration);
        }

        if (property_exists($responseObject, 'registrationDateTime')) {
            $this->setRegistrationDateTime($responseObject->registrationDateTime);
        }

        if (property_exists($responseObject, 'message')) {
            $this->setMessage($responseObject->message);
        }
    }

    /**
     * @return string
     */
    public function getMandantID()
    {
        return $this->mandantID;
    }

    /**
     * @param string $mandantID
     */
    public function setMandantID($mandantID)
    {
        $this->mandantID = $mandantID;
    }

    /**
     * @return string
     */
    public function getSubMandantID()
    {
        return $this->subMandantID;
    }

    /**
     * @param string $subMandantID
     */
    public function setSubMandantID($subMandantID)
    {
        $this->subMandantID = $subMandantID;
    }

    /**
     * @return bool
     */
    public function isRegistration()
    {
        return $this->registration;
    }

    /**
     * @param bool $registration
     */
    public function setRegistration($registration)
    {
        $this->registration = $registration;
    }

    /**
     * @return string
     */
    public function getRegistrationDateTime()
    {
        return $this->registrationDateTime;
    }

    /**
     * @param string $registrationDateTime
     */
    public function setRegistrationDateTime($registrationDateTime)
    {
        $this->registrationDateTime = $registrationDateTime;
    }

    /**
     * @return string
     */
    public function getMessage()
    {
        return $this->message;
    }

    /**
     * @param string $message
     */
    public function setMessage($message)
    {
        $this->message = $message;
    }
}
