<?php

namespace DeutschePost\Internetmarke;

defined('ABSPATH') || exit;

class Crypto
{
    protected $arg;
    protected $iv_length;
    protected $key_digest;

    public function __construct($arg = array())
    {
        $this->arg = array();

        // Set cipher algorithm
        $ciphers = openssl_get_cipher_methods();
        if (isset($arg['cipher'])) {
            if (!in_array($arg['cipher'], $ciphers)) {
                throw new \Exception('Invalid cipher algorithm: ' . $cipher);
            }
        } else {
            if (in_array('aes-256-ctr', $ciphers)) {
                $arg['cipher'] = 'aes-256-ctr';
            } else {
                $arg['cipher'] = $ciphers[0];
            }
        }
        $this->arg['cipher'] = $arg['cipher'];

        // Set initialisation vector length
        $this->iv_length = openssl_cipher_iv_length($this->arg['cipher']);

        // Set encryption key
        if (!isset($arg['key'])) {
            $arg['key'] = openssl_random_pseudo_bytes(32);
        }
        $this->arg['key'] = $arg['key'];

        // Set key hash algorithm
        $hashes = openssl_get_md_methods();
        if (isset($arg['hash'])) {
            if (!in_array($arg['hash'], $hashes)) {
                throw new \Exception('Invalid hash algorithm: ' . $hash);
            }
        } else {
            if (in_array('sha256', $hashes)) {
                $arg['hash'] = 'sha256';
            } else {
                $arg['hash'] = $hashes[0];
            }
        }
        $this->arg['hash'] = $arg['hash'];

        // Set key digest
        $this->key_digest = openssl_digest($this->arg['key'], $this->arg['hash'], true);
    }

    public function getArgs()
    {
        return $this->arg;
    }

    public function getCipher()
    {
        return $this->arg['cipher'];
    }

    public function getKey()
    {
        return $this->arg['key'];
    }

    public function getHash()
    {
        return $this->arg['hash'];
    }

    public function encrypt($str)
    {
        $iv = openssl_random_pseudo_bytes($this->iv_length);

        return $iv . openssl_encrypt(
            $str,
            $this->arg['cipher'],
            $this->key_digest,
            OPENSSL_RAW_DATA,
            $iv
        );
    }

    public function decrypt($str)
    {
        if (strlen($str) < $this->iv_length) {
            throw new \Exception('Data is shorter than the initialisation vector length (' . $this->iv_length . 'B)');
        }

        return openssl_decrypt(
            substr($str, $this->iv_length),
            $this->arg['cipher'],
            $this->key_digest,
            OPENSSL_RAW_DATA,
            substr($str, 0, $this->iv_length)
        );
    }
}
