<?php


namespace DeutschePost\Internetmarke;


use DeutschePost\Internetmarke\Repositories\Resources\Shipment as ShipmentResourceModel;
use DeutschePost\Internetmarke\Repositories\Resources\Shipment as ShipmentResourceRepository;

class Export
{
    /**
     * Export constructor.
     */
    public function __construct()
    {
        add_action('wcdpi_export_shipments_csv', array($this, 'exportShipmentsCsv'), 10, 1);
    }

    /**
     * @param int[] $shipmentIds
     */
    public function exportShipmentsCsv($shipmentIds)
    {
        $shipments = array();
        $columns = array(
            'id'                       => true,
            'order_id'                 => true,
            'prodws_id'                => true,
            'harmonized_label'         => true,
            'ekp'                      => true,
            'shop_order_id'            => true,
            'shop_pp_order_id'         => true,
            'pp_shipment_id'           => true,
            'awb_id'                   => true,
            'voucher_id'               => true,
            'tracking'                 => true,
            'retoure_transaction_id'   => true,
            'price'                    => true,
            'nature_type'              => true,
            'service_level'            => true,
            'status'                   => true,
            'purchase_dt'              => true,
            'status_update_dt'         => true,
            'sender_first_name'        => true,
            'sender_last_name'         => true,
            'sender_company_name'      => true,
            'sender_contact_name'      => true,
            'sender_email'             => true,
            'sender_phone'             => true,
            'sender_is_company'        => true,
            'sender_street'            => true,
            'sender_house_number'      => true,
            'sender_additional'        => true,
            'sender_zip'               => true,
            'sender_city'              => true,
            'sender_country_alpha2'    => true,
            'sender_country_alpha3'    => true,
            'recipient_first_name'     => true,
            'recipient_last_name'      => true,
            'recipient_company_name'   => true,
            'recipient_is_company'     => true,
            'recipient_email'          => true,
            'recipient_phone'          => true,
            'recipient_street'         => true,
            'recipient_house_number'   => true,
            'recipient_additional'     => true,
            'recipient_zip'            => true,
            'recipient_city'           => true,
            'recipient_state'          => true,
            'recipient_country_alpha2' => true,
            'recipient_country_alpha3' => true,
            'meta_insert_dt'           => true,
            'meta_insert_by'           => true,
            'meta_update_dt'           => true,
            'meta_update_by'           => true
        );

        foreach ($shipmentIds as $shipmentId) {
            /** @var ShipmentResourceModel $shipment */
            $shipment = ShipmentResourceRepository::instance()->find($shipmentId);
            $data = array();

            if (!$shipment) {
                continue;
            }

            foreach (array_keys($columns) as $exportColumn) {
                switch ($exportColumn) {
                    default:
                        $getter              = 'get' . wcdpi_camelize($exportColumn);
                        $data[$exportColumn] = null;

                        if (method_exists($shipment, $getter)) {
                            $data[$exportColumn] = $shipment->{$getter}();
                        }

                        break;
                }
            }

            $shipments[] = $data;
        }

        $filename    = date('YmdHis') . '_internetmarke_shipments_export.csv';

        // Disable caching
        $now = gmdate("D, d M Y H:i:s");
        header("Cache-Control: max-age=0, no-cache, must-revalidate, proxy-revalidate");
        header("Last-Modified: {$now} GMT");

        // Force download
        header("Content-Type: application/force-download");
        header("Content-Type: application/octet-stream");
        header("Content-Type: application/download");

        // Disposition / encoding on response body
        header("Content-Disposition: attachment;filename={$filename}");
        header("Content-Transfer-Encoding: binary");

        ob_clean();
        ob_start();
        $df = fopen("php://output", 'w');
        fputcsv($df, array_keys($shipments[0]));

        foreach ($shipments as $row) {
            fputcsv($df, $row);
        }

        fclose($df);
        ob_end_flush();

        exit();
    }
}