<?php

namespace DeutschePost\Internetmarke\Integrations\WooCommerce;

use WP_Post;

defined('ABSPATH') || exit;

class ProductData
{
    /**
     * ProductData constructor.
     */
    public function __construct()
    {
        add_action('woocommerce_product_options_shipping',          array($this, 'simpleProductDataPanel'), 8);
        add_action('save_post_product',                             array($this, 'simpleProductSave'));
        add_action('woocommerce_product_after_variable_attributes', array($this, 'variableProductDataPanel'), 10, 3);
        add_action('woocommerce_save_product_variation',            array($this, 'variableProductSave'), 10, 2);
    }

    /**
     * Displays additional shipping options when editing a product.
     */
    public function simpleProductDataPanel()
    {
        global $thepostid, $post;

        $productId = empty($thepostid) ? $post->ID : $thepostid;
        $product   = wc_get_product($productId);

        // The options for variable products are shown inside the variations,
        // not the parent.
        if (!$product || $product->is_type('variable')) {
            return;
        }

        $customsDeclarationText   = get_post_meta($productId, 'wcdpi_customs_declaration_text',    true);
        $customsDeclarationPrice  = get_post_meta($productId, 'wcdpi_customs_declaration_price',   true);
        $customsDeclarationHsCode = get_post_meta($productId, 'wcdpi_customs_declaration_hs_code', true);
        $customsDeclarationOrigin = get_post_meta($productId, 'wcdpi_customs_declaration_origin',  true);

        $countries = array_merge(
            array('0' => __( '- Select country -', 'woo-dp-internetmarke' )),
            WC()->countries->get_countries()
        );

        echo '</div><div class="options_group">';
        echo '<p class="form-field"><label style="width: 100%;"><strong>Deutsche Post Internetmarke</strong></label></p>';

        woocommerce_wp_text_input(
            array(
                'id'          => 'wcdpi_customs_declaration_text',
                'label'       => __('Customs declaration text', 'woo-dp-internetmarke'),
                'description' => $this->getCustomsDeclarationTextTooltip(),
                'desc_tip'    => 'true',
                'placeholder' => '',
                'value'       => ($customsDeclarationText) ? $customsDeclarationText : ''
            )
        );

        woocommerce_wp_text_input(
            array(
                'id'          => 'wcdpi_customs_declaration_price',
                'label'       => __('Customs declaration price', 'woo-dp-internetmarke'),
                'description' => $this->getCustomsDeclarationPriceTooltip(),
                'desc_tip'    => 'true',
                'placeholder' => '',
                'value'       => ($customsDeclarationPrice) ? $customsDeclarationPrice : ''
            )
        );

        woocommerce_wp_text_input(
            array(
                'id'          => 'wcdpi_customs_declaration_hs_code',
                'label'       => __('HS Code', 'woo-dp-internetmarke'),
                'description' => $this->getHsCodeTooltip(),
                'desc_tip'    => 'true',
                'placeholder' => '',
                'value'       => ($customsDeclarationHsCode) ? $customsDeclarationHsCode : ''
            ) 
        );

        woocommerce_wp_select(
            array(
                'id'       => 'wcdpi_customs_declaration_origin',
                'label'    => __('Country of Manufacture', 'woo-dp-internetmarke'),
                'desc_tip' => 'true',
                'value'    => $customsDeclarationOrigin,
                'options'  => $countries
            ) 
        );
    }

    /**
     * Stores the data from the additional fields inside the database
     *
     * @param int $postId
     */
    public function simpleProductSave($postId)
    {
        // Customs declaration text
        if (isset($_POST['wcdpi_customs_declaration_text'])) {
            update_post_meta($postId, 'wcdpi_customs_declaration_text', wc_clean($_POST['wcdpi_customs_declaration_text']));
        }

        // Customs declaration price
        if (isset($_POST['wcdpi_customs_declaration_price'])) {
            $price = str_replace(',', '.', $_POST['wcdpi_customs_declaration_price']);

            update_post_meta($postId, 'wcdpi_customs_declaration_price', wc_clean(floatval($price)));
        }

        // HS Code
        if (isset($_POST['wcdpi_customs_declaration_hs_code'])) {
            update_post_meta($postId, 'wcdpi_customs_declaration_hs_code', wc_clean($_POST['wcdpi_customs_declaration_hs_code']));
        }

        // Country of manufacture
        if (isset($_POST['wcdpi_customs_declaration_origin'])) {
            update_post_meta($postId, 'wcdpi_customs_declaration_origin', wc_clean($_POST['wcdpi_customs_declaration_origin']));
        }
    }

    /**
     * Adds the new product data fields to variable WooCommerce Products.
     *
     * @param int     $loop
     * @param array   $variationData
     * @param WP_Post $variation
     */
    public function variableProductDataPanel($loop, $variationData, $variation)
    {
        $productId                = $variation->ID;
        $customsDeclarationText   = get_post_meta($productId, 'wcdpi_customs_declaration_text',    true);
        $customsDeclarationPrice  = get_post_meta($productId, 'wcdpi_customs_declaration_price',   true);
        $customsDeclarationHsCode = get_post_meta($productId, 'wcdpi_customs_declaration_hs_code', true);
        $customsDeclarationOrigin = get_post_meta($productId, 'wcdpi_customs_declaration_origin',  true);

        $countries = array_merge(
            array('0' => __( '- Select country -', 'woo-dp-internetmarke' )),
            WC()->countries->get_countries()
        );

        echo '<p class="form-row form-row-full"><strong>Deutsche Post Internetmarke</strong></p>';
        echo '<input type="hidden" name="internetmarke_edit_flag" value="true"/>';

        woocommerce_wp_text_input(
            array(
                'id'            => "wcdpi_customs_declaration_text{$loop}",
                'name'          => "wcdpi_customs_declaration_text[{$loop}]",
                'label'         => __('Customs declaration text', 'woo-dp-internetmarke'),
                'value'         => ($customsDeclarationText) ? $customsDeclarationText : '',
                'wrapper_class' => 'form-row form-row-full',
                'description'   => $this->getCustomsDeclarationTextTooltip(),
                'desc_tip'      => true,
                'type'          => 'text'
            )
        );

        woocommerce_wp_text_input(
            array(
                'id'            => "wcdpi_customs_declaration_price{$loop}",
                'name'          => "wcdpi_customs_declaration_price[{$loop}]",
                'label'         => __('Customs declaration price', 'woo-dp-internetmarke'),
                'value'         => ($customsDeclarationPrice) ? $customsDeclarationPrice : '',
                'wrapper_class' => 'form-row form-row-full',
                'description'   => $this->getCustomsDeclarationPriceTooltip(),
                'desc_tip'      => true,
                'type'          => 'text'
            )
        );

        woocommerce_wp_text_input(
            array(
                'id'            => "wcdpi_customs_declaration_hs_code{$loop}",
                'name'          => "wcdpi_customs_declaration_hs_code[{$loop}]",
                'label'         => __('HS Code', 'woo-dp-internetmarke'),
                'value'         => ($customsDeclarationHsCode) ? $customsDeclarationHsCode : '',
                'wrapper_class' => 'form-row form-row-full',
                'description'   => $this->getHsCodeTooltip(),
                'desc_tip'      => true,
                'type'          => 'text'
            )
        );

        woocommerce_wp_select(
            array(
                'id'            => "wcdpi_customs_declaration_origin{$loop}",
                'name'          => "wcdpi_customs_declaration_origin[{$loop}]",
                'label'         => __('Country of Manufacture', 'woo-dp-internetmarke'),
                'value'         => ($customsDeclarationOrigin) ? $customsDeclarationOrigin : '0',
                'wrapper_class' => 'form-row form-row-full',
                'desc_tip'      => false,
                'options'       => $countries
            )
        );
    }

    /**
     * Saves the data from the product variation fields.
     *
     * @param int $variationId
     * @param int $i
     */
    public function variableProductSave($variationId, $i)
    {
        // Customs declaration text
        if (isset($_POST['wcdpi_customs_declaration_text'][$i])) {
            $customsDeclarationText = wc_clean($_POST['wcdpi_customs_declaration_text'][$i]);
            update_post_meta($variationId, 'wcdpi_customs_declaration_text', $customsDeclarationText);
        }

        // Customs declaration price
        if (isset($_POST['wcdpi_customs_declaration_price'][$i])) {
            $price = str_replace(',', '.', $_POST['wcdpi_customs_declaration_price'][$i]);
            update_post_meta($variationId, 'wcdpi_customs_declaration_price', wc_clean(floatval($price)));
        }

        // HS Code
        if (isset($_POST['wcdpi_customs_declaration_hs_code'][$i])) {
            $customsDeclarationHsCode = wc_clean($_POST['wcdpi_customs_declaration_hs_code'][$i]);
            update_post_meta($variationId, 'wcdpi_customs_declaration_hs_code', $customsDeclarationHsCode);
        }

        // Country of manufacture
        if (isset($_POST['wcdpi_customs_declaration_origin'][$i])) {
            $customsDeclarationOrigin = wc_clean($_POST['wcdpi_customs_declaration_origin'][$i]);
            update_post_meta($variationId, 'wcdpi_customs_declaration_origin', $customsDeclarationOrigin);
        }
    }

    /**
     * Customs declaration text tooltip text.
     *
     * @return string
     */
    public function getCustomsDeclarationTextTooltip()
    {
        return __('This text will be printed directly onto the CN22 customs declaration.', 'woo-dp-internetmarke');
    }

    /**
     * Customs declaration price tooltip text.
     *
     * @return string
     */
    public function getCustomsDeclarationPriceTooltip()
    {
        return __('This price will only be used to for the CN22 customs declaration.', 'woo-dp-internetmarke');
    }

    /**
     * HS Code tooltip text.
     * 
     * @return string
     */
    public function getHsCodeTooltip()
    {
        return __('HS Code is a number assigned to every possible commodity that can be imported or exported from any country.', 'woo-dp-internetmarke');
    }
}