<?php

namespace DeutschePost\Internetmarke\Models\Resources;

use stdClass;
use DeutschePost\Internetmarke\Abstracts\Model as AbstractResourceModel;
use DeutschePost\Internetmarke\Enums\ShipmentStatus;
use DeutschePost\Internetmarke\Interfaces\Model as ModelInterface;
use DeutschePost\Internetmarke\Repositories\Resources\Shipment as ShipmentResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ShipmentAttachment as ShipmentAttachmentResourceRepository;

defined('ABSPATH') || exit;

class Shipment extends AbstractResourceModel implements ModelInterface
{
    /**
     * @var int
     */
    protected $id;

    /**
     * @var int
     */
    protected $orderId;

    /**
     * @var int
     */
    protected $prodwsId;

    /**
     * @var bool
     */
    protected $harmonizedLabel;

    /**
     * @var string
     */
    protected $ekp;

    /**
     * @var string
     */
    protected $shopOrderId;

    /**
     * @var string
     */
    protected $shopPpOrderId;

    /**
     * @var string
     */
    protected $ppShipmentId;

    /**
     * @var string
     */
    protected $awbId;

    /**
     * @var string
     */
    protected $voucherId;

    /**
     * @var string
     */
    protected $tracking;

    /**
     * @var string
     */
    protected $retoureTransactionId;

    /**
     * @var int
     */
    protected $price;

    /**
     * @var string
     */
    protected $natureType;

    /**
     * @var string
     */
    protected $serviceLevel;

    /**
     * @var int
     */
    protected $status;

    /**
     * @var string
     */
    protected $purchaseDt;

    /**
     * @var string
     */
    protected $statusUpdateDt;

    /**
     * @var string
     */
    protected $senderFirstName;

    /**
     * @var string
     */
    protected $senderLastName;

    /**
     * @var string
     */
    protected $senderCompanyName;

    /**
     * @var string
     */
    protected $senderContactName;

    /**
     * @var string
     */
    protected $senderEmail;

    /**
     * @var string
     */
    protected $senderPhone;

    /**
     * @var string
     */
    protected $senderIsCompany;

    /**
     * @var string
     */
    protected $senderStreet;

    /**
     * @var string
     */
    protected $senderHouseNumber;

    /**
     * @var string
     */
    protected $senderAdditional;

    /**
     * @var string
     */
    protected $senderZip;

    /**
     * @var string
     */
    protected $senderCity;

    /**
     * @var string
     */
    protected $senderCountryAlpha2;

    /**
     * @var string
     */
    protected $senderCountryAlpha3;

    /**
     * @var string
     */
    protected $recipientFirstName;

    /**
     * @var string
     */
    protected $recipientLastName;

    /**
     * @var string
     */
    protected $recipientCompanyName;

    /**
     * @var string
     */
    protected $recipientIsCompany;

    /**
     * @var string
     */
    protected $recipientEmail;

    /**
     * @var string
     */
    protected $recipientPhone;

    /**
     * @var string
     */
    protected $recipientStreet;

    /**
     * @var string
     */
    protected $recipientHouseNumber;

    /**
     * @var string
     */
    protected $recipientAdditional;

    /**
     * @var string
     */
    protected $recipientZip;

    /**
     * @var string
     */
    protected $recipientCity;

    /**
     * @var string
     */
    protected $recipientState;

    /**
     * @var string
     */
    protected $recipientCountryAlpha2;

    /**
     * @var string
     */
    protected $recipientCountryAlpha3;

    /**
     * @var string
     */
    protected $metaInsertDt;

    /**
     * @var string
     */
    protected $metaInsertBy;

    /**
     * @var string
     */
    protected $metaUpdateDt;

    /**
     * @var string
     */
    protected $metaUpdateBy;

    /**
     * Shipment constructor.
     *
     * @param null|stdClass $shipment
     */
    public function __construct($shipment = null)
    {
        if (!$shipment instanceof stdClass) {
            return;
        }

        $this->setId($shipment->id);
        $this->setOrderId($shipment->order_id);
        $this->setProdwsId($shipment->prodws_id);
        $this->setHarmonizedLabel((bool)$shipment->harmonized_label);
        $this->setEkp($shipment->ekp);
        $this->setShopOrderId($shipment->shop_order_id);
        $this->setShopPpOrderId($shipment->shop_pp_order_id);
        $this->setPpShipmentId($shipment->pp_shipment_id);
        $this->setAwbId($shipment->awb_id);
        $this->setVoucherId($shipment->voucher_id);
        $this->setTracking($shipment->tracking);
        $this->setRetoureTransactionId($shipment->retoure_transaction_id);
        $this->setPrice($shipment->price);
        $this->setNatureType($shipment->nature_type);
        $this->setServiceLevel($shipment->service_level);
        $this->setStatus($shipment->status);
        $this->setPurchaseDt($shipment->purchase_dt);
        $this->setStatusUpdateDt($shipment->status_update_dt);
        $this->setSenderFirstName($shipment->sender_first_name);
        $this->setSenderLastName($shipment->sender_last_name);
        $this->setSenderCompanyName($shipment->sender_company_name);
        $this->setSenderContactName($shipment->sender_contact_name);
        $this->setSenderEmail($shipment->sender_email);
        $this->setSenderPhone($shipment->sender_phone);
        $this->setSenderIsCompany($shipment->sender_is_company);
        $this->setSenderStreet($shipment->sender_street);
        $this->setSenderHouseNumber($shipment->sender_house_number);
        $this->setSenderAdditional($shipment->sender_additional);
        $this->setSenderZip($shipment->sender_zip);
        $this->setSenderCity($shipment->sender_city);
        $this->setSenderCountryAlpha2($shipment->sender_country_alpha2);
        $this->setSenderCountryAlpha3($shipment->sender_country_alpha3);
        $this->setRecipientFirstName($shipment->recipient_first_name);
        $this->setRecipientLastName($shipment->recipient_last_name);
        $this->setRecipientCompanyName($shipment->recipient_company_name);
        $this->setRecipientIsCompany($shipment->recipient_is_company);
        $this->setRecipientEmail($shipment->recipient_email);
        $this->setRecipientPhone($shipment->recipient_phone);
        $this->setRecipientStreet($shipment->recipient_street);
        $this->setRecipientHouseNumber($shipment->recipient_house_number);
        $this->setRecipientAdditional($shipment->recipient_additional);
        $this->setRecipientZip($shipment->recipient_zip);
        $this->setRecipientCity($shipment->recipient_city);
        $this->setRecipientState($shipment->recipient_state);
        $this->setRecipientCountryAlpha2($shipment->recipient_country_alpha2);
        $this->setRecipientCountryAlpha3($shipment->recipient_country_alpha3);
        $this->setMetaInsertDt($shipment->meta_insert_dt);
        $this->setMetaInsertBy($shipment->meta_insert_by);
        $this->setMetaUpdateDt($shipment->meta_update_dt);
        $this->setMetaUpdateBy($shipment->meta_update_by);
    }

    /**
     * Returns the total shipment value.
     *
     * @return float
     */
    public function getValue()
    {
        return ShipmentResourceRepository::instance()->value($this->getId());
    }

    /**
     * Returns the total shipment weight (in grams).
     *
     * @return float
     */
    public function getWeight()
    {
        return ShipmentResourceRepository::instance()->weight($this->getId());
    }

    /**
     * Returns the formatted shipment weight.
     */
    public function getFormattedWeight()
    {
        $weight = wcdpi_convert_weight('g', get_option('woocommerce_weight_unit'), $this->getWeight());

        return $weight;
    }

    /**
     * Retrieves the URL to download the shipment label.
     *
     * @return string|bool
     */
    public function getLabel()
    {
        /** @var ShipmentAttachment $shipmentAttachment */
        $shipmentAttachment = ShipmentAttachmentResourceRepository::instance()->findBy(
            array(
                'shipment_id' => $this->getId(),
                'type' => \DeutschePost\Internetmarke\Enums\ShipmentAttachment::LABEL
            )
        );

        if (!$shipmentAttachment) {
            return false;
        }

        return $shipmentAttachment->getUrl();
    }

    /**
     * Returns the human readable status
     *
     * @return string
     */
    public function getStatusReadable()
    {
        return ShipmentStatus::i18n($this->getStatus());
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return int
     */
    public function getOrderId()
    {
        return $this->orderId;
    }

    /**
     * @param int $orderId
     */
    public function setOrderId($orderId)
    {
        $this->orderId = $orderId;
    }

    /**
     * @return int
     */
    public function getProdwsId()
    {
        return $this->prodwsId;
    }

    /**
     * @param int $prodwsId
     */
    public function setProdwsId($prodwsId)
    {
        $this->prodwsId = $prodwsId;
    }

    /**
     * @return bool
     */
    public function isHarmonizedLabel()
    {
        return $this->harmonizedLabel;
    }

    /**
     * @param bool $harmonizedLabel
     */
    public function setHarmonizedLabel($harmonizedLabel)
    {
        $this->harmonizedLabel = $harmonizedLabel;
    }

    /**
     * @return string
     */
    public function getEkp()
    {
        return $this->ekp;
    }

    /**
     * @param string $ekp
     */
    public function setEkp($ekp)
    {
        $this->ekp = $ekp;
    }

    /**
     * @return string
     */
    public function getShopOrderId()
    {
        return $this->shopOrderId;
    }

    /**
     * @param string $shopOrderId
     */
    public function setShopOrderId($shopOrderId)
    {
        $this->shopOrderId = $shopOrderId;
    }

    /**
     * @return string
     */
    public function getShopPpOrderId()
    {
        return $this->shopPpOrderId;
    }

    /**
     * @param string $shopPpOrderId
     */
    public function setShopPpOrderId($shopPpOrderId)
    {
        $this->shopPpOrderId = $shopPpOrderId;
    }

    /**
     * @return string
     */
    public function getPpShipmentId()
    {
        return $this->ppShipmentId;
    }

    /**
     * @param string $ppShipmentId
     */
    public function setPpShipmentId($ppShipmentId)
    {
        $this->ppShipmentId = $ppShipmentId;
    }

    /**
     * @return string
     */
    public function getAwbId()
    {
        return $this->awbId;
    }

    /**
     * @param string $awbId
     */
    public function setAwbId($awbId)
    {
        $this->awbId = $awbId;
    }

    /**
     * @return string
     */
    public function getVoucherId()
    {
        return $this->voucherId;
    }

    /**
     * @param string $voucherId
     */
    public function setVoucherId($voucherId)
    {
        $this->voucherId = $voucherId;
    }

    /**
     * @return string
     */
    public function getTracking()
    {
        return $this->tracking;
    }

    /**
     * @param string $tracking
     */
    public function setTracking($tracking)
    {
        $this->tracking = $tracking;
    }

    /**
     * @return string
     */
    public function getRetoureTransactionId()
    {
        return $this->retoureTransactionId;
    }

    /**
     * @param string $retoureTransactionId
     */
    public function setRetoureTransactionId($retoureTransactionId)
    {
        $this->retoureTransactionId = $retoureTransactionId;
    }

    /**
     * @return int
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * @param int $price
     */
    public function setPrice($price)
    {
        $this->price = $price;
    }

    /**
     * @return string
     */
    public function getNatureType()
    {
        return $this->natureType;
    }

    /**
     * @param string $natureType
     */
    public function setNatureType($natureType)
    {
        $this->natureType = $natureType;
    }

    /**
     * @return string
     */
    public function getServiceLevel()
    {
        return $this->serviceLevel;
    }

    /**
     * @param string $serviceLevel
     */
    public function setServiceLevel($serviceLevel)
    {
        $this->serviceLevel = $serviceLevel;
    }

    /**
     * @return int
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @param int $status
     */
    public function setStatus($status)
    {
        $this->status = $status;
    }

    /**
     * @return string
     */
    public function getPurchaseDt()
    {
        return $this->purchaseDt;
    }

    /**
     * @param string $purchaseDt
     */
    public function setPurchaseDt($purchaseDt)
    {
        $this->purchaseDt = $purchaseDt;
    }

    /**
     * @return string
     */
    public function getStatusUpdateDt()
    {
        return $this->statusUpdateDt;
    }

    /**
     * @param string $statusUpdateDt
     */
    public function setStatusUpdateDt($statusUpdateDt)
    {
        $this->statusUpdateDt = $statusUpdateDt;
    }

    /**
     * @return string
     */
    public function getSenderFirstName()
    {
        return $this->senderFirstName;
    }

    /**
     * @param string $senderFirstName
     */
    public function setSenderFirstName($senderFirstName)
    {
        $this->senderFirstName = $senderFirstName;
    }

    /**
     * @return string
     */
    public function getSenderLastName()
    {
        return $this->senderLastName;
    }

    /**
     * @param string $senderLastName
     */
    public function setSenderLastName($senderLastName)
    {
        $this->senderLastName = $senderLastName;
    }

    /**
     * @return string
     */
    public function getSenderCompanyName()
    {
        return $this->senderCompanyName;
    }

    /**
     * @param string $senderCompanyName
     */
    public function setSenderCompanyName($senderCompanyName)
    {
        $this->senderCompanyName = $senderCompanyName;
    }

    /**
     * @return string
     */
    public function getSenderContactName()
    {
        return $this->senderContactName;
    }

    /**
     * @param string $senderContactName
     */
    public function setSenderContactName($senderContactName)
    {
        $this->senderContactName = $senderContactName;
    }

    /**
     * @return string
     */
    public function getSenderEmail()
    {
        return $this->senderEmail;
    }

    /**
     * @param string $senderEmail
     */
    public function setSenderEmail($senderEmail)
    {
        $this->senderEmail = $senderEmail;
    }

    /**
     * @return string
     */
    public function getSenderPhone()
    {
        return $this->senderPhone;
    }

    /**
     * @param string $senderPhone
     */
    public function setSenderPhone($senderPhone)
    {
        $this->senderPhone = $senderPhone;
    }

    /**
     * @return string
     */
    public function getSenderIsCompany()
    {
        return $this->senderIsCompany;
    }

    /**
     * @param string $senderIsCompany
     */
    public function setSenderIsCompany($senderIsCompany)
    {
        $this->senderIsCompany = $senderIsCompany;
    }

    /**
     * @return string
     */
    public function getSenderStreet()
    {
        return $this->senderStreet;
    }

    /**
     * @param string $senderStreet
     */
    public function setSenderStreet($senderStreet)
    {
        $this->senderStreet = $senderStreet;
    }

    /**
     * @return string
     */
    public function getSenderHouseNumber()
    {
        return $this->senderHouseNumber;
    }

    /**
     * @param string $senderHouseNumber
     */
    public function setSenderHouseNumber($senderHouseNumber)
    {
        $this->senderHouseNumber = $senderHouseNumber;
    }

    /**
     * @return string
     */
    public function getSenderAdditional()
    {
        return $this->senderAdditional;
    }

    /**
     * @param string $senderAdditional
     */
    public function setSenderAdditional($senderAdditional)
    {
        $this->senderAdditional = $senderAdditional;
    }

    /**
     * @return string
     */
    public function getSenderZip()
    {
        return $this->senderZip;
    }

    /**
     * @param string $senderZip
     */
    public function setSenderZip($senderZip)
    {
        $this->senderZip = $senderZip;
    }

    /**
     * @return string
     */
    public function getSenderCity()
    {
        return $this->senderCity;
    }

    /**
     * @param string $senderCity
     */
    public function setSenderCity($senderCity)
    {
        $this->senderCity = $senderCity;
    }

    /**
     * @return string
     */
    public function getSenderCountryAlpha2()
    {
        return $this->senderCountryAlpha2;
    }

    /**
     * @param string $senderCountryAlpha2
     */
    public function setSenderCountryAlpha2($senderCountryAlpha2)
    {
        $this->senderCountryAlpha2 = $senderCountryAlpha2;
    }

    /**
     * @return string
     */
    public function getSenderCountryAlpha3()
    {
        return $this->senderCountryAlpha3;
    }

    /**
     * @param string $senderCountryAlpha3
     */
    public function setSenderCountryAlpha3($senderCountryAlpha3)
    {
        $this->senderCountryAlpha3 = $senderCountryAlpha3;
    }

    /**
     * @return string
     */
    public function getRecipientFirstName()
    {
        return $this->recipientFirstName;
    }

    /**
     * @param string $recipientFirstName
     */
    public function setRecipientFirstName($recipientFirstName)
    {
        $this->recipientFirstName = $recipientFirstName;
    }

    /**
     * @return string
     */
    public function getRecipientLastName()
    {
        return $this->recipientLastName;
    }

    /**
     * @param string $recipientLastName
     */
    public function setRecipientLastName($recipientLastName)
    {
        $this->recipientLastName = $recipientLastName;
    }

    /**
     * @return string
     */
    public function getRecipientCompanyName()
    {
        return $this->recipientCompanyName;
    }

    /**
     * @param string $recipientCompanyName
     */
    public function setRecipientCompanyName($recipientCompanyName)
    {
        $this->recipientCompanyName = $recipientCompanyName;
    }

    /**
     * @return string
     */
    public function getRecipientIsCompany()
    {
        return $this->recipientIsCompany;
    }

    /**
     * @param string $recipientIsCompany
     */
    public function setRecipientIsCompany($recipientIsCompany)
    {
        $this->recipientIsCompany = $recipientIsCompany;
    }

    /**
     * @return string
     */
    public function getRecipientEmail()
    {
        return $this->recipientEmail;
    }

    /**
     * @param string $recipientEmail
     */
    public function setRecipientEmail($recipientEmail)
    {
        $this->recipientEmail = $recipientEmail;
    }

    /**
     * @return string
     */
    public function getRecipientPhone()
    {
        return $this->recipientPhone;
    }

    /**
     * @param string $recipientPhone
     */
    public function setRecipientPhone($recipientPhone)
    {
        $this->recipientPhone = $recipientPhone;
    }

    /**
     * @return string
     */
    public function getRecipientStreet()
    {
        return $this->recipientStreet;
    }

    /**
     * @param string $recipientStreet
     */
    public function setRecipientStreet($recipientStreet)
    {
        $this->recipientStreet = $recipientStreet;
    }

    /**
     * @return string
     */
    public function getRecipientHouseNumber()
    {
        return $this->recipientHouseNumber;
    }

    /**
     * @param string $recipientHouseNumber
     */
    public function setRecipientHouseNumber($recipientHouseNumber)
    {
        $this->recipientHouseNumber = $recipientHouseNumber;
    }

    /**
     * @return string
     */
    public function getRecipientAdditional()
    {
        return $this->recipientAdditional;
    }

    /**
     * @param string $recipientAdditional
     */
    public function setRecipientAdditional($recipientAdditional)
    {
        $this->recipientAdditional = $recipientAdditional;
    }

    /**
     * @return string
     */
    public function getRecipientZip()
    {
        return $this->recipientZip;
    }

    /**
     * @param string $recipientZip
     */
    public function setRecipientZip($recipientZip)
    {
        $this->recipientZip = $recipientZip;
    }

    /**
     * @return string
     */
    public function getRecipientCity()
    {
        return $this->recipientCity;
    }

    /**
     * @param string $recipientCity
     */
    public function setRecipientCity($recipientCity)
    {
        $this->recipientCity = $recipientCity;
    }

    /**
     * @return string
     */
    public function getRecipientState()
    {
        return $this->recipientState;
    }

    /**
     * @param string $recipientState
     */
    public function setRecipientState($recipientState)
    {
        $this->recipientState = $recipientState;
    }

    /**
     * @return string
     */
    public function getRecipientCountryAlpha2()
    {
        return $this->recipientCountryAlpha2;
    }

    /**
     * @param string $recipientCountryAlpha2
     */
    public function setRecipientCountryAlpha2($recipientCountryAlpha2)
    {
        $this->recipientCountryAlpha2 = $recipientCountryAlpha2;
    }

    /**
     * @return string
     */
    public function getRecipientCountryAlpha3()
    {
        return $this->recipientCountryAlpha3;
    }

    /**
     * @param string $recipientCountryAlpha3
     */
    public function setRecipientCountryAlpha3($recipientCountryAlpha3)
    {
        $this->recipientCountryAlpha3 = $recipientCountryAlpha3;
    }

    /**
     * @return string
     */
    public function getMetaInsertDt()
    {
        return $this->metaInsertDt;
    }

    /**
     * @param string $metaInsertDt
     */
    public function setMetaInsertDt($metaInsertDt)
    {
        $this->metaInsertDt = $metaInsertDt;
    }

    /**
     * @return string
     */
    public function getMetaInsertBy()
    {
        return $this->metaInsertBy;
    }

    /**
     * @param string $metaInsertBy
     */
    public function setMetaInsertBy($metaInsertBy)
    {
        $this->metaInsertBy = $metaInsertBy;
    }

    /**
     * @return string
     */
    public function getMetaUpdateDt()
    {
        return $this->metaUpdateDt;
    }

    /**
     * @param string $metaUpdateDt
     */
    public function setMetaUpdateDt($metaUpdateDt)
    {
        $this->metaUpdateDt = $metaUpdateDt;
    }

    /**
     * @return string
     */
    public function getMetaUpdateBy()
    {
        return $this->metaUpdateBy;
    }

    /**
     * @param string $metaUpdateBy
     */
    public function setMetaUpdateBy($metaUpdateBy)
    {
        $this->metaUpdateBy = $metaUpdateBy;
    }
}