<?php

namespace DeutschePost\Internetmarke\Repositories\Credentials;

use DeutschePost\Internetmarke\Abstracts\Singleton;
use DeutschePost\Internetmarke\Crypto;
use DeutschePost\Internetmarke\Interfaces\CredentialsModel;
use DeutschePost\Internetmarke\Interfaces\CredentialsRepository as CredentialsRepositoryInterface;
use DeutschePost\Internetmarke\Models\Credentials\Partner as CredentialsPartnerModel;
use Exception;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class Partner extends Singleton implements CredentialsRepositoryInterface
{
    /**
     * @return bool|CredentialsPartnerModel
     */
    public function get()
    {
        $partner = get_option('_wcdpi_partner_credentials');

        if (!$partner) {
            return false;
        }

        try {
            $info = (new Crypto(array('key' => $partner->data->key)))->decrypt(base64_decode($partner->data->encrypted_data));
        } catch (Exception $e) {
            Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
            return false;
        }

        return new CredentialsPartnerModel(json_decode($info));
    }

    /**
     * @param array $credentials
     *
     * @return bool|CredentialsModel
     */
    public function set($credentials)
    {
        if (!update_option('_wcdpi_partner_credentials', $credentials, true)) {
            return false;
        }

        return $this->get();
    }

    /**
     * @return bool|void
     */
    public function check()
    {
        if (get_option('_wcdpi_partner_credentials')) {
            return true;
        }

        return false;
    }

    /**
     * @return bool
     */
    public function delete()
    {
        delete_option('_wcdpi_partner_credentials');

        return true;
    }
}