<?php

namespace DeutschePost\Internetmarke\Repositories\Credentials;

use DeutschePost\Internetmarke\Abstracts\Singleton;
use Exception;
use DeutschePost\Internetmarke\Crypto;
use DeutschePost\Internetmarke\Interfaces\CredentialsRepository as CredentialsRepositoryInterface;
use DeutschePost\Internetmarke\Models\Credentials\User as CredentialsUserModel;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class User extends Singleton implements CredentialsRepositoryInterface
{
    /**
     * @return bool|CredentialsUserModel
     */
    public function get()
    {
        $passwordCryptoArgs = get_option('_wcdpi_crypto_args');
        $credentials        = get_option('_wcdpi_user_credentials');

        if (empty($passwordCryptoArgs) || empty($credentials)) {
            return false;
        }

        try {
            $crypto      = new Crypto(unserialize(base64_decode($passwordCryptoArgs)));
            $credentials = json_decode(gzinflate($crypto->decrypt(base64_decode($credentials))), true);

            return new CredentialsUserModel($credentials['email'], $credentials['password']);
        } catch (Exception $e) {
            Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
            return false;
        }
    }

    /**
     * @param array $credentials
     *
     * @throws Exception
     *
     * @return bool|CredentialsUserModel
     */
    public function set($credentials)
    {
        $passwordCryptoArgs = get_option('_wcdpi_crypto_args');

        if (empty($passwordCryptoArgs)) {
            $crypto = new Crypto();
            update_option('_wcdpi_crypto_args', base64_encode(serialize($crypto->getArgs())), false);
        } else {
            try {
                $crypto = new Crypto(unserialize(base64_decode($passwordCryptoArgs)));
            } catch (Exception $e) {
                Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
                return false;
            }
        }

        $update = update_option(
            '_wcdpi_user_credentials',
            base64_encode($crypto->encrypt(gzdeflate(json_encode($credentials), 9))),
            true
        );

        if (!$update) {
            return false;
        }

        return new CredentialsUserModel($credentials['email'], $credentials['password']);
    }

    /**
     * @return bool
     */
    public function check()
    {
        if (get_option('_wcdpi_user_credentials')) {
            return true;
        }

        return false;
    }

    /**
     * @return bool|void
     */
    public function delete()
    {
        return delete_option('_wcdpi_user_credentials');
    }

}