<?php

namespace DeutschePost\Internetmarke\Repositories\Resources;

use stdClass;
use DeutschePost\Internetmarke\Abstracts\Repository as AbstractResourceRepository;
use DeutschePost\Internetmarke\Interfaces\ResourceRepository as ResourceRepositoryInterface;
use DeutschePost\Internetmarke\Models\Resources\Shipment as ResourceShipmentModel;

defined('ABSPATH') || exit;

class Shipment extends AbstractResourceRepository implements ResourceRepositoryInterface
{
    /**
     * @var string
     */
    const TABLE = 'wcdpi_shipment';

    /**
     * Shipment constructor.
     */
    public function __construct()
    {
        global $wpdb;

        $this->table      = $wpdb->prefix . self::TABLE;
        $this->primaryKey = 'id';
    }

    /**
     * @param array $query
     *
     * @return int
     */
    public function countBy($query)
    {
        if (!$query || !is_array($query) || count($query) <= 0) {
            return false;
        }

        global $wpdb;

        $sqlQuery = "SELECT COUNT(*) FROM {$this->table} WHERE 1=1 ";

        foreach ($query as $columnName => $value) {
            if (is_array($value)) {
                $valuesIn = implode(', ', array_map('absint', $value));
                $sqlQuery .= "AND {$columnName} IN ({$valuesIn}) ";
            }

            if (is_string($value)) {
                $sqlQuery .= "AND {$columnName} = '{$value}' ";
            }

            if (is_numeric($value)) {
                $value = absint($value);
                $sqlQuery .= "AND {$columnName} = {$value} ";
            }
        }

        $sqlQuery .= ';';

        return intval($wpdb->get_var($sqlQuery));
    }

    /**
     * @param int $id
     *
     * @return float
     */
    public function value($id)
    {
        global $wpdb;

        $sqlQuery = "
            SELECT
                SUM(product_price * product_quantity)
            FROM
                {$wpdb->prefix}wcdpi_shipment_item
            WHERE
                1=1
                AND shipment_id = %d
                AND product_quantity > 0
        ";

        $value = $wpdb->get_var($wpdb->prepare($sqlQuery, $id));

        if (!$value) {
            return 0;
        }

        return floatval($value);
    }

    /**
     * @param int $id
     *
     * @return float
     */
    public function weight($id)
    {
        global $wpdb;

        $sqlQuery = "
            SELECT
                SUM(product_weight * product_quantity)
            FROM
                {$wpdb->prefix}wcdpi_shipment_item
            WHERE
                1=1
                AND shipment_id = %d
                AND product_quantity > 0
        ";

        $value = $wpdb->get_var($wpdb->prepare($sqlQuery, $id));

        if (!$value) {
            return 0;
        }

        return floatval($value);
    }

    /**
     * @param stdClass $dataObject
     *
     * @return mixed|ResourceShipmentModel
     */
    public function createResourceModel($dataObject)
    {
        return new ResourceShipmentModel($dataObject);
    }
}