<?php

namespace DeutschePost\Internetmarke\Services;

use WC_Order_Item;
use WC_Product_Simple;

defined('ABSPATH') || exit;

class ShipmentItemService
{
    /**
     * Prepares the order line items for shipment service use.
     *
     * @param int $orderId WooCommerce Order ID
     *
     * @return array
     */
    public function prepare($orderId)
    {
        $lineItems = array();
        $order     = wc_get_order($orderId);

        if (!$order) {
            return $lineItems;
        }

        /** @var WC_Order_Item $orderItemProduct */
        foreach ($order->get_items() as $orderItemProduct) {
            /** @var WC_Product_Simple $product **/
            $product = $orderItemProduct->get_product();

            // Skip non-existing products
            if (!$product) {
                continue;
            }

            $productId = $product->get_id();
            $qtyShipped = 0;

            $customsDeclarationText   = get_post_meta($productId, 'wcdpi_customs_declaration_text', true);
            $customsDeclarationPrice  = get_post_meta($productId, 'wcdpi_customs_declaration_price', true);
            $customsDeclarationOrigin = get_post_meta($productId, 'wcdpi_customs_declaration_origin', true);
            $customsDeclarationHsCode = get_post_meta($productId, 'wcdpi_customs_declaration_hs_code', true);

            if (!$customsDeclarationText) {
                $customsDeclarationText = $orderItemProduct->get_name();
            }

            if (!$customsDeclarationPrice) {
                $customsDeclarationPrice = $order->get_item_subtotal($orderItemProduct, false, true);
            }

            $qtyOrdered   = (int)$orderItemProduct->get_quantity();
            $qtySelected  = $qtyOrdered - $qtyShipped;
            $qtyAvailable = $qtySelected;

            // Convert the weight to grams before saving to the database.
            $weight = wcdpi_convert_weight(get_option('woocommerce_weight_unit'), 'g', (float)$product->get_weight());

            array_push(
                $lineItems,
                array(
                    'productId'            => $productId,
                    'lineItemId'           => $orderItemProduct->get_id(),
                    'orderId'              => $orderItemProduct->get_order_id(),
                    'editUrl'              => get_edit_post_link($product->get_id(), '&'),
                    'name'                 => $orderItemProduct->get_name(),
                    'sku'                  => $product->get_sku(),
                    'weight'               => $weight,
                    'price'                => (float)$customsDeclarationPrice,
                    'dimensions'           => wc_format_dimensions($product->get_dimensions(false)),
                    'qtyAvailable'         => $qtyAvailable,
                    'qtySelected'          => $qtySelected,
                    'qtyOrdered'           => $qtyOrdered,
                    'qtyShipped'           => $qtyShipped,
                    'countryOfManufacture' => $customsDeclarationOrigin,
                    'hsCode'               => $customsDeclarationHsCode,
                    'customsDeclaration'   => $customsDeclarationText
                )
            );
        }

        return $lineItems;
    }
}