<?php

namespace DeutschePost\Internetmarke\Settings;

use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;

defined('ABSPATH') || exit;

class General extends AbstractSettings
{
    /**
     * @var array
     */
    protected $settings;

    /**
     * General constructor.
     */
    public function __construct()
    {
        $this->settings = get_option('_wcdpi_settings_general', array());

        add_action('admin_init', array($this, 'init'));
    }

    /**
     * Registers the "General" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        register_setting(
            '_wcdpi_settings_group_general',
            '_wcdpi_settings_general',
            array()
        );

        add_settings_section(
            '_wcdpi_default_sender_section',
            __('Default sender', 'woo-dp-internetmarke'),
            array($this, 'defaultSender'),
            '_wcdpi_default_sender'
        );

        add_settings_field(
            '_wcdpi_default_sender_first_name',
            __('First name', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderFirstName'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'first-name text')
        );
        add_settings_field(
            '_wcdpi_default_sender_last_name',
            __('Last name', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderLastName'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'last-name text')
        );
        add_settings_field(
            '_wcdpi_default_sender_company_name',
            __('Company', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderCompanyName'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'company-name text')
        );
        add_settings_field(
            '_wcdpi_default_sender_contact_name',
            __('Contact name', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderContactName'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'contact-name text')
        );
        add_settings_field(
            '_wcdpi_default_sender_email',
            __('Email', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderEmail'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'email text')
        );
        add_settings_field(
            '_wcdpi_default_sender_phone',
            __('Phone', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderPhone'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'phone text')
        );
        add_settings_field(
            '_wcdpi_default_sender_street',
            __('Street', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderStreet'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'street text')
        );
        add_settings_field(
            '_wcdpi_default_sender_house_number',
            __('House number', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderHouseNumber'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'house-number text')
        );
        add_settings_field(
            '_wcdpi_default_sender_additional',
            __('Additional address field (Building, staircase, etc)', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderAdditional'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'additional text')
        );
        add_settings_field(
            '_wcdpi_default_sender_zip',
            __('ZIP', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderZip'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'zip text')
        );
        add_settings_field(
            '_wcdpi_default_sender_city',
            __('City', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderCity'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'city text')
        );
        add_settings_field(
            '_wcdpi_default_sender_country',
            __('Country', 'woo-dp-internetmarke'),
            array($this, 'defaultSenderCountry'),
            '_wcdpi_default_sender',
            '_wcdpi_default_sender_section',
            array('class' => 'country')
        );
    }

    public function defaultSender()
    {
        _e('Please define the default sender address information.', 'woo-dp-internetmarke');
    }

    public function defaultSenderFirstName()
    {
        $field = '_wcdpi_default_sender_first_name';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderLastName()
    {
        $field = '_wcdpi_default_sender_last_name';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderCompanyName()
    {
        $field = '_wcdpi_default_sender_company_name';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderEmail()
    {
        $field = '_wcdpi_default_sender_email';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderPhone()
    {
        $field = '_wcdpi_default_sender_phone';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderContactName()
    {
        $field = '_wcdpi_default_sender_contact_name';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);
        $html .= $this->createDescription(__('Only relevant for CN22 customs declarations.', 'woo-dp-internetmarke'));

        echo $html;
    }

    public function defaultSenderStreet()
    {
        $field = '_wcdpi_default_sender_street';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderHouseNumber()
    {
        $field = '_wcdpi_default_sender_house_number';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderAdditional()
    {
        $field = '_wcdpi_default_sender_additional';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderZip()
    {
        $field = '_wcdpi_default_sender_zip';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderCity()
    {
        $field = '_wcdpi_default_sender_city';
        $value = $this->getSettingValue($field);
        $html  = $this->createInputField('_wcdpi_settings_general', $field, $value);

        echo $html;
    }

    public function defaultSenderCountry()
    {
        $field = '_wcdpi_default_sender_country';

        $html = sprintf(
            '<select id="%s" class="regular-text" name="_wcdpi_settings_general[%s]" disabled="disabled">',
            $field,
            $field
        );

        $html .= sprintf('<option value="%s" selected="selected">%s</option>', 'DE', __('Germany'));
        $html .= '</select>';

        echo $html;
    }
}