<?php

namespace DeutschePost\Internetmarke\Settings;

use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;
use DeutschePost\Internetmarke\Models\Resources\PageFormat as PageFormatResourceModel;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;
use DeutschePost\Internetmarke\Repositories\Resources\PageFormat as PageFormatResourceRepository;

defined('ABSPATH') || exit;

class Internetmarke1C4A extends AbstractSettings
{
    /**
     * @var array
     */
    protected $settings;

    /**
     * Internetmarke1C4A constructor.
     */
    public function __construct()
    {
        $this->settings = get_option('_wcdpi_settings_internetmarke_1c4a', array());

        add_action('admin_init', array($this, 'init'));
    }

    /**
     * Registers the "Internetmarke1C4A" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        register_setting(
            '_wcdpi_settings_group_internetmarke_1c4a',
            '_wcdpi_settings_internetmarke_1c4a',
            array()
        );

        add_settings_section(
            '_wcdpi_1c4a_activation_section',
            __('Service status', 'woo-dp-internetmarke'),
            null,
            '_wcdpi_label_options'
        );

        add_settings_field(
            '_wcdpi_1c4a_activation',
            __('Enabled', 'woo-dp-internetmarke'),
            array($this, 'oneClickForAppActivation'),
            '_wcdpi_label_options',
            '_wcdpi_1c4a_activation_section'
        );

        add_settings_section(
            '_wcdpi_label_options_section',
            __('Label options', 'woo-dp-internetmarke'),
            array($this, 'labelOptions'),
            '_wcdpi_label_options'
        );

        add_settings_field(
            '_wcdpi_label_options_manifest',
            __('Manifest', 'woo-dp-internetmarke'),
            array($this, 'manifest'),
            '_wcdpi_label_options',
            '_wcdpi_label_options_section'
        );
        add_settings_field(
            '_wcdpi_label_options_shipping_list',
            __('Shipping List', 'woo-dp-internetmarke'),
            array($this, 'shippingList'),
            '_wcdpi_label_options',
            '_wcdpi_label_options_section'
        );
        add_settings_section(
            '_wcdpi_page_formats_section',
            __('Page formats', 'woo-dp-internetmarke'),
            array($this, 'pageFormats'),
            '_wcdpi_page_formats'
        );
        add_settings_field(
            '_wcdpi_page_formats_default_label_format',
            __('Default label format', 'woo-dp-internetmarke'),
            array($this, 'selectedFormat'),
            '_wcdpi_page_formats',
            '_wcdpi_page_formats_section'
        );
        add_settings_section(
            '_wcdpi_favorite_labels_section',
            __('Quick select labels', 'woo-dp-internetmarke'),
            array($this, 'favoriteLabels'),
            '_wcdpi_favorite_labels'
        );
        add_settings_field(
            '_wcdpi_favorite_labels_selection',
            __('Selection', 'woo-dp-internetmarke'),
            array($this, 'favoriteLabelsSelection'),
            '_wcdpi_favorite_labels',
            '_wcdpi_favorite_labels_section'
        );
    }

    public function labelOptions()
    {
        _e('Additional label options', 'woo-dp-internetmarke');
    }

    public function oneClickForAppActivation()
    {
        $loggedIn = UserCredentialsRepository::instance()->check();
        $field = '_wcdpi_1c4a_activation';

        $html = sprintf(
            '<select id="%s" name="_wcdpi_settings_internetmarke_1c4a[%s]" disabled="disabled">',
            $field,
            $field
        );

        $html .= sprintf(
            '<option value="1" %s>%s</option>',
            $loggedIn ? ' selected' : '',
            __('Yes', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="0" %s>%s</option>',
            $loggedIn ? '' : ' selected',
            __('No', 'woo-dp-internetmarke')
        );

        $html .= '</select>';

        $html .= $this->createDescription(
            __('When the Portokasse service is activated, the Plugin service is automatically activated.', 'woo-dp-internetmarke')
        );

        echo $html;
    }

    public function manifest()
    {
        $field = '_wcdpi_label_options_manifest';
        $value = $this->getSettingValue($field, true);

        $html = '<fieldset>';
        $html .= sprintf('<label for="%s">', $field);

        $html .= sprintf(
            '<input id="%s" type="checkbox" name="_wcdpi_settings_internetmarke_1c4a[%s]" value="1" %s/>',
            $field,
            $field,
            checked(true, $value, false)
        );

        $html .= sprintf('<span>%s</span>', __('Provide a manifest with every purchased label.', 'woo-dp-internetmarke'));
        $html .= '</label>';

        $html .= $this->createDescription(
            __('Identifier whether a posting receipt is to be created or not.', 'woo-dp-internetmarke')
        );

        $html .= '</fieldset>';

        echo $html;
    }

    public function shippingList()
    {
        $field = '_wcdpi_label_options_shipping_list';
        $value = $this->getSettingValue($field);

        $html = sprintf('<select id="%s" name="_wcdpi_settings_internetmarke_1c4a[%s]">', $field, $field);

        $html .= sprintf(
            '<option value="0" %s>%s</option>',
            ($value == 0) ? 'selected="selected"' : '',
            __('No shipping list', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="1" %s>%s</option>',
            ($value == 1) ? 'selected="selected"' : '',
            __('Shipping list without addresses', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="2" %s>%s</option>',
            ($value == 2) ? 'selected="selected"' : '',
            __('Shipping list with addresses', 'woo-dp-internetmarke')
        );

        $html .= '</select>';
        $html .= $this->createDescription(__('Identifier of which type of shipping list is to be created.', 'woo-dp-internetmarke'));

        echo $html;
    }

    public function pageFormats()
    {
        _e('This option defines the page format of the label.', 'woo-dp-internetmarke');
    }

    public function selectedFormat()
    {
        /** @var PageFormatResourceModel[] $pageFormats */
        $pageFormats = PageFormatResourceRepository::instance()->findAll();
        $field       = '_wcdpi_page_formats_default_label_format';
        $value       = $this->getSettingValue($field);
        $html        = sprintf('<select id="%s" name="_wcdpi_settings_internetmarke_1c4a[%s]">', $field, $field);

        if (!empty($pageFormats)) {
            foreach ($pageFormats as $pageFormat) {
                $selected = '';

                if ($value === $pageFormat->getFormatId()) {
                    $selected = 'selected="selected"';
                }

                $html .= sprintf(
                    '<option value="%s" %s>%s</option>',
                    $pageFormat->getFormatId(),
                    $selected,
                    $pageFormat->getName()
                );
            }
        }

        $html .= '</select>';

        $html .= $this->createDescription(
            sprintf(
                __('All shown formats can print addresses. To update the page formats, go to the <a href="%s">tools</a>.', 'woo-dp-internetmarke'),
                admin_url('admin.php?page=internetmarke_settings&tab=tools&section=synchronization')
            )
        );

        echo $html;
    }

    public function favoriteLabels()
    {
        _e('Define your favorite labels for simple access from the WooCommerce orders.', 'woo-dp-internetmarke');
    }

    public function favoriteLabelsSelection()
    {
        $field = '_wcdpi_favorite_labels_selection';

        if (!$value = $this->getSettingValue($field)) {
            $value = array();
        }

        include_once WCDPI_TEMPLATES_DIR . 'settings/section-favorite-labels.php';
    }
}