<?php

namespace DeutschePost\Internetmarke\Settings;

use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;

defined('ABSPATH') || exit;

class Internetmarke1C4R extends AbstractSettings
{
    /**
     * Internetmarke1C4R constructor.
     */
    public function __construct()
    {
        add_action('admin_init', array($this, 'init'));
    }

    /**
     * Registers the "Internetmarke1C4R" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        register_setting(
            '_wcdpi_settings_group_internetmarke_1c4r',
            '_wcdpi_settings_1c4r',
            array()
        );

        add_settings_section(
            '_wcdpi_1c4r_activation_section',
            __('Service status', 'woo-dp-internetmarke'),
            null,
            '_wcdpi_settings_1c4r'
        );

        add_settings_field(
            '_wcdpi_1c4r_activation',
            __('Enabled', 'woo-dp-internetmarke'),
            array($this, 'oneClickForRefundActivation'),
            '_wcdpi_settings_1c4r',
            '_wcdpi_1c4r_activation_section'
        );
    }

    public function oneClickForRefundActivation()
    {
        $field    = '_wcdpi_1c4r_activation';
        $loggedIn = UserCredentialsRepository::instance()->check();

        $html = sprintf(
            '<select id="%s" name="_wcdpi_settings_internetmarke_1c4r[%s]" disabled="disabled">',
            $field,
            $field
        );

        $html .= sprintf(
            '<option value="1" %s>%s</option>',
            $loggedIn ? ' selected="selected"' : '',
            __('Yes', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="0" %s>%s</option>',
            $loggedIn ? '' : ' selected="selected"',
            __('No', 'woo-dp-internetmarke')
        );

        $html .= '</select>';

        $html .= $this->createDescription(
            __('When the Portokasse service is activated, the refund service is automatically activated.', 'woo-dp-internetmarke')
        );

        echo $html;
    }
}