<?php

namespace DeutschePost\Internetmarke\Settings;

use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;

defined('ABSPATH') || exit;

class Logger extends AbstractSettings
{
    /**
     * @var array
     */
    protected $settings;

    /**
     * Logger constructor.
     */
    public function __construct()
    {
        add_action('admin_init', array($this, 'init'));

        $this->settings = get_option('_wcdpi_settings_logger', array());
    }

    /**
     * Registers the "Logger" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        register_setting(
            '_wcdpi_settings_group_logger',
            '_wcdpi_settings_logger',
            array()
        );

        add_settings_section(
            '_wcdpi_logger_section',
            __('Logging', 'woo-dp-internetmarke'),
            array($this, 'logger'),
            '_wcdpi_logger'
        );
        add_settings_field(
            '_wcdpi_logger_selection',
            __('Enabled', 'woo-dp-internetmarke'),
            array($this, 'loggerSelection'),
            '_wcdpi_logger',
            '_wcdpi_logger_section'
        );
        add_settings_field(
            '_wcdpi_logger_viewer',
            __('Log viewer', 'woo-dp-internetmarke'),
            array($this, 'loggerViewer'),
            '_wcdpi_logger',
            '_wcdpi_logger_section'
        );
    }

    public function logger()
    {
        _e('Logging related settings', 'woo-dp-internetmarke');
    }

    public function loggerSelection()
    {
        $field = '_wcdpi_logger_selection';
        $value = $this->getSettingValue($field, false);

        $html = '<fieldset>';
        $html .= sprintf('<label for="%s">', $field);

        $html .= sprintf(
            '<input id="%s" type="checkbox" name="_wcdpi_settings_logger[%s]" value="1" %s/>',
            $field,
            $field,
            checked(true, $value, false)
        );

        $html .= sprintf('<span>%s</span>', __('Enable/Disable', 'woo-dp-internetmarke'));
        $html .= '</label>';

        $html .= $this->createDescription(
            __('By ticking this box, all errors will be logged to a file. This can be useful when an issue is present and support is necessary.', 'woo-dp-internetmarke')
        );

        $html .= '</fieldset>';

        echo $html;
    }

    public function loggerViewer()
    {
        if (file_exists(WCDPI_LOG_DIR . 'internetmarke.log')) {
            ob_start();
            include WCDPI_LOG_DIR . 'internetmarke.log';
            $contents = ob_get_clean();
        }

        else {
            $contents = __('Log file is empty.', 'woo-dp-internetmarke');
        }

        $html = sprintf('<div class="wcdpi-log-viewer"><pre>%s</pre></div>', $contents);

        echo $html;
    }

}