<?php

namespace DeutschePost\Internetmarke\Settings;

use Exception;
use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class PortChecker extends AbstractSettings
{
    /**
     * @var array
     */
    protected $settings;

    /**
     * PortChecker constructor.
     */
    public function __construct()
    {
        add_action('admin_init', array($this, 'init'));

        $this->settings = get_option('_wcdpi_settings_port_checker', array());
    }

    /**
     * Registers the "PortChecker" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        register_setting(
            '_wcdpi_settings_group_port_checker',
            '_wcdpi_settings_port_checker',
            array(
                'sanitize_callback' => array($this, 'sanitizeSettings')
            )
        );
    }

    /**
     * Callback for settings sanitization.
     *
     * @return array
     */
    public function sanitizeSettings()
    {
        // Check ports.
        if (array_key_exists('check_ports', (array)$_POST)) {
            try {
                return array(
                    '_wcdpi_portokasse_port_status'           => $this->getPortStatus(),
                    '_wcdpi_portokasse_port_status_timestamp' => date('Y-m-d, H:i:s')
                );
            } catch (Exception $e) {
                add_settings_error(
                    '_wcdpi_settings_group_port_checker',
                    'port_status_error',
                    $e->getMessage(),
                    'error'
                );
                Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
                return array();
            }
        }

        return $this->settings;
    }

    /**
     * Checks if all necessary ports are open.
     *
     * @return array
     */
    private function getPortStatus()
    {
        $results      = array();
        $destinations = array(
            array(
                'host' => 'tcp://www.deutschepost.de',
                'ports' => array(443)
            ),
            array(
                'host' => 'tcp://internetmarke.deutschepost.de',
                'ports' => array(443)
            ),
            array(
                'host' => 'tcp://prodws.deutschepost.de',
                'ports' => array(443, 8443)
            ),
            array(
                'host' => 'udp://www.deutschepost.de',
                'ports' => array(443, 8443)
            ),
            array(
                'host' => 'udp://internetmarke.deutschepost.de',
                'ports' => array(443, 8443)
            ),
            array(
                'host' => 'udp://prodws.deutschepost.de',
                'ports' => array(443, 8443)
            ),
        );

        foreach ($destinations as $destination) {
            $host = $destination['host'];

            foreach ($destination['ports'] as $port) {
                $connection = @fsockopen($host, $port);

                if (is_resource($connection)) {
                    $results[$host][$port] = true;
                    fclose($connection);
                } else {
                    $results[$host][$port] = false;
                }
            }
        }

        return $results;
    }
}