<?php

namespace DeutschePost\Internetmarke\Settings;

use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;

defined('ABSPATH') || exit;

class ScheduledTasks extends AbstractSettings
{
    /**
     * @var array
     */
    protected $settings;

    /**
     * ScheduledTasks constructor.
     */
    public function __construct()
    {
        add_action('admin_init', array($this, 'init'));

        $this->settings = get_option('_wcdpi_settings_scheduled_tasks', array());
    }

    /**
     * Registers the "ScheduledTasks" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        register_setting(
            '_wcdpi_settings_group_scheduled_tasks',
            '_wcdpi_settings_scheduled_tasks',
            array(
                'sanitize_callback' => array($this, 'sanitizeSettings')
            )
        );
    }

    /**
     * Callback for settings sanitization.
     *
     * @return array
     */
    public function sanitizeSettings()
    {
        // Schedule the product list update cron.
        if (isset($_POST['wcdpi_schedule_cron_internetmarke_update_product_list'])) {

            // Schedule the next update
            if (!wp_next_scheduled('internetmarke_update_product_list')) {
                wp_schedule_event(time(), 'daily', 'internetmarke_update_product_list');
            }
        }

        // Schedule the wallet update cron.
        if (isset($_POST['wcdpi_schedule_cron_internetmarke_update_portokasse_wallet'])) {

            // Update the wallet right now.
            do_action('wcdpi_integration_oneclickforapp_update_wallet');

            // Schedule the next update
            if (!wp_next_scheduled('internetmarke_update_portokasse_wallet')) {
                wp_schedule_event(time(), 'daily', 'internetmarke_update_portokasse_wallet');
            }
        }

        return $this->settings;
    }
}