<?php

namespace DeutschePost\Internetmarke\Settings;

use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;

defined('ABSPATH') || exit;

class Tools extends AbstractSettings
{
    /**
     * @var array
     */
    protected $settings;

    /**
     * Tools constructor.
     */
    public function __construct()
    {
        add_action('admin_init', array($this, 'init'));

        $this->settings = get_option('_wcdpi_settings_tools', array());
    }

    /**
     * Registers the "Tools" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        register_setting(
            '_wcdpi_settings_group_tools',
            '_wcdpi_settings_tools',
            array(
                'sanitize_callback' => array($this, 'sanitizeSettings')
            )
        );
    }

    /**
     * Callback for settings sanitization.
     *
     * @return array
     */
    public function sanitizeSettings()
    {
        // Update Page Formats
        if (array_key_exists('update_page_formats', (array)$_POST)) {
            do_action('wcdpi_integration_oneclickforapp_update_page_formats');

            return array(
                '_wcdpi_update_product_list_timestamp' => $this->settings['_wcdpi_update_product_list_timestamp'],
                '_wcdpi_update_page_formats_timestamp' => date('d.m.Y, H:i')
            );
        }

        // Update Product List
        if (array_key_exists('update_product_list', (array)$_POST)) {
            do_action('wcdpi_integration_prodws_update_product_list');
            do_action('wcdpi_integration_oneclickforapp_update_contract_products');

            return array(
                '_wcdpi_update_product_list_timestamp' => date('d.m.Y, H:i'),
                '_wcdpi_update_page_formats_timestamp' => $this->settings['_wcdpi_update_page_formats_timestamp']
            );
        }

        return array(
            '_wcdpi_update_product_list_timestamp' => $this->settings['_wcdpi_update_product_list_timestamp'],
            '_wcdpi_update_page_formats_timestamp' => $this->settings['_wcdpi_update_page_formats_timestamp']
        );
    }
}