<?php

namespace DeutschePost\Internetmarke;

use Exception;
use WC_Customer;

defined('ABSPATH') || exit;

class Store
{
    /**
     * @var array
     */
    protected $defaultSender = array();

    /**
     * Store constructor.
     */
    public function __construct()
    {
        $this->init();
    }

    /**
     * Initializes the default sender.
     */
    public function init()
    {
        $options = (array)get_option('_wcdpi_settings_general');

        $firstName   = '';
        $lastName    = '';
        $company     = '';
        $contactName = '';
        $email       = '';
        $phone       = '';
        $additional  = '';
        $street      = '';
        $houseNumber = '';
        $zip         = '';
        $city        = '';

        if (array_key_exists('_wcdpi_default_sender_first_name', $options)) {
            $firstName = $options['_wcdpi_default_sender_first_name'];
        }

        if (array_key_exists('_wcdpi_default_sender_last_name', $options)) {
            $lastName = $options['_wcdpi_default_sender_last_name'];
        }

        if (array_key_exists('_wcdpi_default_sender_company_name', $options)) {
            $company = $options['_wcdpi_default_sender_company_name'];
        }

        if (array_key_exists('_wcdpi_default_sender_contact_name', $options)) {
            $contactName = $options['_wcdpi_default_sender_contact_name'];
        }

        if (array_key_exists('_wcdpi_default_sender_email', $options)) {
            $email = $options['_wcdpi_default_sender_email'];
        }

        if (array_key_exists('_wcdpi_default_sender_phone', $options)) {
            $phone = $options['_wcdpi_default_sender_phone'];
        }

        if (array_key_exists('_wcdpi_default_sender_additional', $options)) {
            $additional =$options['_wcdpi_default_sender_additional'];
        }

        if (array_key_exists('_wcdpi_default_sender_street', $options)) {
            $street = $options['_wcdpi_default_sender_street'];
        }

        if (array_key_exists('_wcdpi_default_sender_house_number', $options)) {
            $houseNumber = $options['_wcdpi_default_sender_house_number'];
        }

        if (array_key_exists('_wcdpi_default_sender_zip', $options)) {
            $zip = $options['_wcdpi_default_sender_zip'];
        }

        if (array_key_exists('_wcdpi_default_sender_city', $options)) {
            $city = $options['_wcdpi_default_sender_city'];
        }

        $this->defaultSender = array(
            'firstName'   => $firstName,
            'lastName'    => $lastName,
            'company'     => $company,
            'contactName' => $contactName,
            'email'       => $email,
            'phone'       => $phone,
            'street'      => $street,
            'houseNumber' => $houseNumber,
            'additional'  => $additional,
            'city'        => $city,
            'zip'         => $zip,
            'state'       => null,
            'country'     => 'DE'
        );
    }

    /**
     * Returns the order address for the given order ID.
     * If there is no shipping address return the billing address.
     *
     * @param int $orderId
     *
     * @return Address
     */
    public function getOrderAddress($orderId = null)
    {
        if (!is_numeric($orderId) || !$orderId) {
            $address = new Address(
                array(
                    'firstName'   => '',
                    'lastName'    => '',
                    'company'     => '',
                    'email'       => '',
                    'phone'       => '',
                    'street'      => '',
                    'houseNumber' => '',
                    'additional'  => '',
                    'zip'         => '',
                    'city'        => '',
                    'state'       => '',
                    'country'     => ''
                )
            );

            return apply_filters('internetmarke_get_order_address', $address, $orderId);
        }

        $address = array();

        // Email & Phone
        $address['email'] = get_post_meta($orderId, '_billing_email', true);
        $address['phone'] = get_post_meta($orderId, '_billing_phone', true);

        // Get the shipping address.
        $wcAddress = array(
            'first_name' => get_post_meta($orderId, '_shipping_first_name', true),
            'last_name'  => get_post_meta($orderId, '_shipping_last_name', true),
            'company'    => get_post_meta($orderId, '_shipping_company', true),
            'address_1'  => get_post_meta($orderId, '_shipping_address_1', true),
            'address_2'  => get_post_meta($orderId, '_shipping_address_2', true),
            'city'       => get_post_meta($orderId, '_shipping_city', true),
            'state'      => $this->getOrderState($orderId, 'shipping'),
            'zip'        => get_post_meta($orderId, '_shipping_postcode', true),
            'country'    => get_post_meta($orderId, '_shipping_country', true)
        );

        // If no shipping address is provided, the billing address will be used instead.
        if (!array_filter($wcAddress) == true) {
            $wcAddress = array(
                'first_name' => get_post_meta($orderId, '_billing_first_name', true),
                'last_name'  => get_post_meta($orderId, '_billing_last_name', true),
                'company'    => get_post_meta($orderId, '_billing_company', true),
                'address_1'  => get_post_meta($orderId, '_billing_address_1', true),
                'address_2'  => get_post_meta($orderId, '_billing_address_2', true),
                'city'       => get_post_meta($orderId, '_billing_city', true),
                'state'      => $this->getOrderState($orderId, 'billing'),
                'zip'        => get_post_meta($orderId, '_billing_postcode', true),
                'country'    => get_post_meta($orderId, '_billing_country', true)
            );
        }

        $address['firstName']   = $wcAddress['first_name'];
        $address['lastName']    = $wcAddress['last_name'];
        $address['company']     = $wcAddress['company'];
        $address['street']      = $wcAddress['address_1'];
        $address['houseNumber'] = '';
        $address['additional']  = $wcAddress['address_2'];
        $address['zip']         = $wcAddress['zip'];
        $address['city']        = $wcAddress['city'];
        $address['state']       = $wcAddress['state'];
        $address['country']     = $wcAddress['country'];

        return apply_filters('internetmarke_get_order_address', new Address($address), $orderId);
    }

    /**
     * @param int    $userId
     * @param string $addressType
     *
     * @return false|Address
     */
    public function getUserAddress($userId, $addressType = 'billing')
    {
        try {
            $customer = new WC_Customer($userId);
        } catch (Exception $e) {
            return false;
        }

        $billingAddress = $customer->get_billing();

        $address = array(
            'email' => $billingAddress['email'] ? $billingAddress['email'] : '',
            'phone' => $billingAddress['phone'] ? $billingAddress['phone'] : '',
            'houseNumber' => ''
        );

        if ($addressType === 'billing') {
            $wooAddress = $billingAddress;
        } else {
            $wooAddress = $customer->get_shipping();
        }

        $address['firstName']   = $wooAddress['first_name'] ? $wooAddress['first_name'] : '';
        $address['lastName']    = $wooAddress['last_name']  ? $wooAddress['last_name']  : '';
        $address['company']     = $wooAddress['company']    ? $wooAddress['company']    : '';
        $address['street']      = $wooAddress['address_1']  ? $wooAddress['address_1']  : '';
        $address['additional']  = $wooAddress['address_2']  ? $wooAddress['address_2']  : '';
        $address['zip']         = $wooAddress['postcode']   ? $wooAddress['postcode']   : '';
        $address['city']        = $wooAddress['city']       ? $wooAddress['city']       : '';
        $address['state']       = $wooAddress['state']      ? $wooAddress['state']      : '';
        $address['country']     = $wooAddress['country']    ? $wooAddress['country']    : '';

        return new Address($address);
    }

    /**
     * Returns the default store address
     *
     * @return Address
     */
    public function getStoreAddress()
    {
        return new Address(
            array(
                'firstName'   => $this->defaultSender['firstName'],
                'lastName'    => $this->defaultSender['lastName'],
                'company'     => $this->defaultSender['company'],
                'contactName' => $this->defaultSender['contactName'],
                'email'       => $this->defaultSender['email'],
                'phone'       => $this->defaultSender['phone'],
                'street'      => $this->defaultSender['street'],
                'houseNumber' => $this->defaultSender['houseNumber'],
                'additional'  => $this->defaultSender['additional'],
                'city'        => $this->defaultSender['city'],
                'zip'         => $this->defaultSender['zip'],
                'state'       => null,
                'country'     => $this->defaultSender['country']
            )
        );
    }

    /**
     * Returns the state name for a given order ID / address type combination.
     *
     * @param int    $orderId
     * @param string $addressType This value be either "shipping" or "billing"
     *
     * @return null|string
     */
    public function getOrderState($orderId, $addressType)
    {
        if (!$orderId) {
            return null;
        }

        if ($addressType === 'billing') {
            $state   = get_post_meta($orderId, '_billing_state', true);
            $country = get_post_meta($orderId, '_billing_country', true);
        } elseif ($addressType === 'shipping') {
            $state   = get_post_meta($orderId, '_shipping_state', true);
            $country = get_post_meta($orderId, '_shipping_country', true);
        } else {
            return null;
        }

        if (!$state) {
            return null;
        }

        $countryStates = WC()->countries->get_states($country);

        if (!isset($countryStates[$state])) {
            return $state;
        }

        return $countryStates[$state];
    }
}
