<?php

use DeutschePost\Internetmarke\Enums\ShipmentStatus as ShipmentStatusEnum;
use DeutschePost\Internetmarke\Models\Resources\Shipment as ShipmentResourceModel;

defined('ABSPATH') || exit;

/**
 * @var ShipmentResourceModel[] $orderShipments
 */

?>

<div id="wcdpi-order-shipment-meta-box-content" class="d-f">
    <input id="wcdpi-order-shipment-meta-box-content-order-id" type="hidden" value="<?php echo $post->ID; ?>">

    <div class="wrapper-outer">
        <div class="shipments">
            <div class="header">
                <div><?php esc_html_e('ID', 'woo-dp-internetmarke'); ?></div>
                <div><?php esc_html_e('Tracking', 'woo-dp-internetmarke'); ?></div>
                <div><?php esc_html_e('Value', 'woo-dp-internetmarke'); ?></div>
                <div><?php esc_html_e('Weight', 'woo-dp-internetmarke'); ?></div>
                <div><?php esc_html_e('Shipping label', 'woo-dp-internetmarke'); ?></div>
                <div><?php esc_html_e('Status', 'woo-dp-internetmarke'); ?></div>
                <div><?php esc_html_e('Last modified', 'woo-dp-internetmarke'); ?></div>
            </div>
            <?php foreach ($orderShipments as $i => $shipment): ?>
                <div class="table-row">
                    <div class="shipment-id">
                        <span><?php echo wcdpi_format_shipment_number($shipment->getId()); ?></span>
                    </div>
                    <div class="tracking-no">
                        <span><?php
                            if ($shipment->getTracking()) {
                                echo wcdpi_tracking_link($shipment->getTracking(), $shipment->getMetaUpdateDt());
                            }
                            ?></span>
                    </div>
                    <div class="value">
                        <span><?php echo wcdpi_format_price($shipment->getValue()); ?></span>
                    </div>
                    <div class="weight">
                        <span><?php echo wcdpi_format_weight($shipment->getFormattedWeight()); ?></span>
                    </div>
                    <div class="shipping-label">
                        <?php if ($shipment->getLabel()): ?>
                            <a href="<?php echo esc_url($shipment->getLabel()); ?>" target="_blank">
                                <span><?php esc_html_e('Download', 'woo-dp-internetmarke'); ?></span>
                            </a>
                        <?php endif; ?>
                    </div>
                    <div class="status">
                        <?php switch ($shipment->getStatus()) {
                            case ShipmentStatusEnum::READY_TO_SHIP:
                                $badge = 'secondary';
                                break;
                            case ShipmentStatusEnum::SHIPPED:
                                $badge = 'primary';
                                break;
                            case ShipmentStatusEnum::DELIVERED:
                                $badge = 'info';
                                break;
                            case ShipmentStatusEnum::REFUNDED:
                            case ShipmentStatusEnum::RETURNED:
                                $badge = 'warning';
                                break;
                            case ShipmentStatusEnum::FAILED:
                            case ShipmentStatusEnum::DELETED:
                                $badge = 'danger';
                                break;
                            default:
                                $badge = '';
                                break;
                        } ?>

                        <span class="wcdpi-badge <?php echo esc_attr($badge); ?>">
                            <?php echo esc_html($shipment->getStatusReadable()); ?>
                        </span>
                    </div>
                    <div class="last-modified">
                        <?php

                        // Created
                        if ($shipment->getMetaInsertDt() && !$shipment->getMetaUpdateDt()) {
                            // Created at
                            if ($shipment->getMetaInsertDt()) {
                                try {
                                    $date = new DateTime($shipment->getMetaInsertDt());
                                } catch (Exception $e) {
                                    $date = __('Not available', 'woo-dp-internetmarke');
                                }

                                echo sprintf(
                                    '<span>%s <b>%s, %s</b></span>',
                                    __('at', 'woo-dp-internetmarke'),
                                    $date->format(get_option('date_format')),
                                    $date->format(get_option('time_format'))
                                );
                            }

                            // Created by
                            if ($shipment->getMetaInsertBy()) {
                                $user = get_user_by('id', $shipment->getMetaInsertBy());

                                if ($user instanceof WP_User) {
                                    if (current_user_can('manage_options')) {
                                        echo sprintf(
                                            '<br>%s <a href="%s"><b>%s</b></a>',
                                            __('by', 'woo-dp-internetmarke'),
                                            get_edit_user_link($user->ID),
                                            $user->display_name
                                        );
                                    }

                                    else {
                                        echo sprintf(
                                            '<br><span>%s <b>%s</b></span>',
                                            __('by', 'woo-dp-internetmarke'),
                                            $user->display_name
                                        );
                                    }
                                }

                                else {
                                    echo sprintf(
                                        '<br><span>%s <b>%s</b></span>',
                                        __('by', 'woo-dp-internetmarke'),
                                        __('unknown user', 'woo-dp-internetmarke')
                                    );
                                }
                            }
                        }

                        // Updated
                        elseif ($shipment->getMetaInsertDt() && $shipment->getMetaUpdateDt()) {
                            // Updated at
                            if ($shipment->getMetaUpdateDt()) {
                                try {
                                    $date = new DateTime($shipment->getMetaUpdateDt());
                                } catch (Exception $e) {
                                    $date = __('Not available', 'woo-dp-internetmarke');
                                }

                                echo sprintf(
                                    '<span>%s <b>%s, %s</b></span>',
                                    __('at', 'woo-dp-internetmarke'),
                                    $date->format(get_option('date_format')),
                                    $date->format(get_option('time_format'))
                                );
                            }

                            // Updated by
                            if ($shipment->getMetaUpdateBy()) {
                                /** @var WP_User $user */
                                $user = get_user_by('id', $shipment->getMetaUpdateBy());

                                if ($user instanceof WP_User) {
                                    if (current_user_can('manage_options')) {
                                        echo sprintf(
                                            '<br>%s <a href="%s">%s</a>',
                                            __('by', 'woo-dp-internetmarke'),
                                            get_edit_user_link($user->ID),
                                            $user->display_name
                                        );
                                    }

                                    else {
                                        echo sprintf(
                                            '<br><span>%s %s</span>',
                                            __('by', 'woo-dp-internetmarke'),
                                            $user->display_name
                                        );
                                    }
                                }
                            }

                            else {
                                echo sprintf(
                                    '<br><span>%s <b>%s</b></span>',
                                    __('by', 'woo-dp-internetmarke'),
                                    __('unknown user', 'woo-dp-internetmarke')
                                );
                            }
                        }

                        ?>

                    </div>
                </div>
            <?php endforeach; ?>
        </div>
    </div>

    <div class="taskbar">
        <a class="action-button yellow m-0" @click="openLabelWizard()">
            <?php esc_html_e('New shipment', 'woo-dp-internetmarke'); ?>
        </a>

        <div v-if="active.tab === tabs.prepared">
            <button class="action-button yellow fl-r m-0" name="wcdpi_shipment_action_order_labels">
                <span><?php esc_html_e('Order labels', 'woo-dp-internetmarke'); ?></span>
            </button>
            <button class="action-button red fl-r m-0 mr-1" name="wcdpi_shipment_action_delete_shipments">
                <span><?php esc_html_e('Delete', 'woo-dp-internetmarke'); ?></span>
            </button>
        </div>

        <div v-if="active.tab === tabs.readyToShip">
            <button class="action-button yellow fl-r m-0" name="wcdpi_shipment_action_mark_as_shipped">
                <span><?php esc_html_e('Mark as shipped', 'woo-dp-internetmarke'); ?></span>
            </button>
            <button class="action-button red fl-r m-0 mr-1" name="wcdpi_shipment_action_refund_label">
                <span><?php esc_html_e('Refund label & cancel shipment', 'woo-dp-internetmarke'); ?></span>
            </button>
        </div>

        <div v-if="active.tab === tabs.shipped">
            <button class="action-button yellow fl-r m-0" name="wcdpi_shipment_action_mark_as_delivered">
                <span><?php esc_html_e('Mark as delivered', 'woo-dp-internetmarke'); ?></span>
            </button>
        </div>

        <div v-if="active.tab === tabs.delivered">
            <button class="action-button yellow fl-r m-0" name="wcdpi_shipment_action_mark_as_returned">
                <span><?php esc_html_e('Mark as returned', 'woo-dp-internetmarke'); ?></span>
            </button>
        </div>
    </div>
</div>
