<?php

use DeutschePost\Internetmarke\Enums\ShipmentStatus;

defined('ABSPATH') || exit;

/**
 * @var float  $wallet
 * @var string $email
 * @var array  $shipments
 * @var string $partnerId
 * @var string $keyPhase
 * @var string $marketplaceKey
 * @var string $userToken
 */

?>

<div id="wcdpi-dashboard" class="wrap">

    <div class="stina-row boxed">

        <div class="col-12 header">
            <img class="logo d-ib" src="<?php echo esc_url(WCDPI_IMG_URL . 'dp-logo-64x64.jpg'); ?>" alt="">
            <h1 class="text d-ib"><strong>Deutsche Post Internetmarke</strong></h1>
        </div>

    </div>

    <div class="stina-row boxed">
        <div class="col-4">
            <h2><strong>Portokasse</strong></h2>
            <hr/>

            <button id="wcdpi-wallet"
                    class="action-button yellow teaser-button"
                    style="text-align: left; margin-right: 0; max-width: unset"
                    title="<?php esc_attr_e('Click to update', 'woo-dp-internetmarke'); ?>">
                <strong id="wcdpi-wallet-amount" class="main"><?php echo wcdpi_format_price($wallet / 100); ?></strong>
                <small class="sub"><?php _e('Current balance', 'woo-dp-internetmarke'); ?></small>
            </button>

            <div class="mb-1">
                <strong><?php _e('User', 'woo-dp-internetmarke'); ?></strong>
                <br>
                <span><?php echo esc_html($email); ?></span>
            </div>

            <div>
                <form id="wcdpi-wallet-top-up"
                      action="https://portokasse.deutschepost.de/portokasse/marketplace/enter-app-payment"
                      method="post"
                      target="_blank">
                    <label for="wcdpi-top-up-amount-input" style="display: none;">
                        <?php _e('Top-up amount', 'woo-dp-internetmarke'); ?>
                    </label>
                    <input type="text"
                           id="wcdpi-top-up-amount-input"
                           class="wcdpi-input wcdpi-w-12 mb-1"
                           name="AMOUNT"
                           placeholder="<?php esc_attr_e('Minimum amount: 10,00 €', 'woo-dp-internetmarke'); ?>">
                    <input type="hidden" name="SUCCESS_URL" value="<?php echo esc_url(admin_url('admin.php?page=internetmarke&top-up=success')); ?>">
                    <input type="hidden" name="CANCEL_URL" value="<?php echo esc_url(admin_url('admin.php?page=internetmarke&top-up=failed')); ?>">
                    <input type="hidden" name="PARTNER_ID" value="<?php echo esc_attr($partnerId); ?>">
                    <input type="hidden" name="KEY_PHASE" value="<?php echo esc_attr($keyPhase); ?>">
                    <input type="hidden" name="SCHLUESSEL_DPWN_PARTNER" value="<?php echo esc_attr($marketplaceKey); ?>">
                    <input type="hidden" name="USER_TOKEN" value="<?php echo esc_attr($userToken); ?>">
                    <input type="hidden" name="WALLET" value="<?php echo esc_attr($wallet); ?>">
                    <input type="hidden" name="BALANCE" value=""/>
                    <input type="hidden" name="PARTNER_SIGNATURE" value=""/>
                    <input type="hidden" name="REQUEST_TIMESTAMP" value=""/>
                    <button class="action-button yellow" type="submit">
                        <?php _e('Top-up', 'woo-dp-internetmarke'); ?>
                    </button>
                </form>
            </div>
        </div>

        <div class="col-4">
            <h2><strong><?php _e('Shipments', 'woo-dp-internetmarke'); ?></strong></h2>
            <hr/>
            <ul class="shipments-list">
                <?php foreach ($shipments as $shipmentStatus => $count): ?>
                    <li>
                        <span class="count wcdpi-badge primary">
                            <?php echo esc_html($count); ?>
                        </span>
                        <span class="label">
                            <a href="<?php echo esc_url(admin_url('admin.php?page=internetmarke_shipments&status=' . ShipmentStatus::getSlug($shipmentStatus))); ?>"
                               class="wcdpi-action-link">
                                <?php echo esc_html(ShipmentStatus::i18n($shipmentStatus)); ?>
                            </a>
                        </span>
                    </li>
                <?php endforeach; ?>
            </ul>
        </div>

        <div class="col-4">
            <h2><strong><?php _e('Tools', 'woo-dp-internetmarke'); ?></strong></h2>
            <hr/>

            <form method="post" action="<?php echo esc_url(admin_url('admin-post.php')) ;?>">
                <input type="hidden" name="action" value="wcdpi_update_product_list">
                <?php wp_nonce_field('wcdpi_update_product_list'); ?>

                <button class="action-button yellow teaser-button"
                        name="update_product_list"
                        value="1"
                        type="submit"
                        style="text-align: left; margin-right: 0; max-width: unset">
                    <strong class="main"><?php _e('Product list', 'woo-dp-internetmarke'); ?></strong>
                    <small class="sub"><?php _e('Update', 'woo-dp-internetmarke'); ?></small>
                </button>
            </form>

            <form method="post" action="<?php echo esc_url(admin_url('admin-post.php')) ;?>">
                <input type="hidden" name="action" value="wcdpi_update_page_formats">
                <?php wp_nonce_field('wcdpi_update_page_formats'); ?>

                <button class="action-button yellow teaser-button"
                        name="update_page_formats"
                        value="1"
                        type="submit"
                        style="text-align: left; margin-right: 0; max-width: unset">
                    <strong class="main"><?php _e('Page formats', 'woo-dp-internetmarke'); ?></strong>
                    <small class="sub"><?php _e('Update', 'woo-dp-internetmarke'); ?></small>
                </button>
            </form>

        </div>
    </div>

    <div class="stina-row boxed">
        <div class="col-12 footer">
            <h3><?php _e('Do you have questions or require further assistance?', 'woo-dp-internetmarke'); ?></h3>
            <p><?php echo sprintf(
                    __('Additional information and answered questions can be found in the <a  href="%s" target="_blank" rel="noopener">WordPress Support Forum for the Internetmarke</a>. You can also contact our support team via email at <a class="textlink" href="mailto:support@stina-global.com">support@stina-global.com</a>.', 'woo-dp-internetmarke'),
                    'https://wordpress.org/support/plugin/woo-dp-internetmarke/'
                ); ?></p>
        </div>
    </div>

</div>
