<?php

use DeutschePost\Internetmarke\Models\Resources\ProductSales as ProductSalesResourceModel;

defined('ABSPATH') || exit;

/**
 * Available variables
 *
 * @var array                       $orders
 * @var string                      $ordersValue
 * @var ProductSalesResourceModel[] $labels
 * @var string                      $region
 * @var string $destination
 */
?>

<div class="wrap">

    <div id="wcdpi-shipments-bulk-create">
        <input id="wcdpi-orders" type="hidden" value="<?php echo esc_attr($ordersValue); ?>">

        <div class="d-f" :class="{ 'd-n': mode }">
            <div class="ta-c wcdpi-w-6">
                <p><?php _e('Pick a single label for all the selected orders.', 'woo-dp-internetmarke'); ?></p>
                <button class="action-button yellow" @click="mode = 'single'">
                    <?php _e('Single Label', 'woo-dp-internetmarke'); ?>
                </button>
            </div>
            <div class="ta-c wcdpi-w-6">
                <p><?php _e('Pick a different label for each order.', 'woo-dp-internetmarke'); ?></p>
                <button class="action-button yellow" @click="mode = 'multi'">
                    <?php _e('Multi Label', 'woo-dp-internetmarke'); ?>
                </button>
            </div>
        </div>

        <div class="wcdpi-shipments-bulk-create single" :class="{ 'd-b': mode === 'single' }">

            <div class="d-f" style="flex-wrap: wrap;">

                <div class="top-bar wcdpi-w-12 mb-1">
                    <h2 class="d-ib m-0">
                        <?php echo esc_html(wcdpi_get_shipping_region_name($region)); ?>
                    </h2>
                </div>

                <div class="sidebar pos-r">
                    <article class="order" v-for="order in orders">
                        <header class="number">
                            <a :href="order.editUrl" target="_blank" rel="noopener noreferrer">
                                <span>#{{ order.number }}</span>
                                <span>-</span>
                                <span>{{ order.email }}</span>
                            </a>
                            <button class="expand fl-r" @click="order.active = order.active !== true;">
                                <span v-if="order.active === false">&plus;</span>
                                <span v-if="order.active === true">&minus;</span>
                            </button>
                        </header>
                        <footer class="data d-f" :class="{ expanded: order.active }">
                            <address class="wcdpi-w-12" v-html="order.address"></address>
                        </footer>
                    </article>

                    <div class="wcdpi-loading" v-if="loading">
                        <img src="/wp-includes/images/spinner.gif"
                             alt="<?php _e('Loading...', 'woo-dp-internetmarke'); ?>">
                    </div>
                </div>

                <div class="label pos-r">
                    <div class="d-f">
                        <div class="options p-1">

                            <div class="mb-1">
                                <div class="mb-1">
                                    <strong>
                                        <label for="wcdpi-shipping-label">
                                            <?php _e('Shipping product', 'woo-dp-internetmarke'); ?>
                                        </label>
                                    </strong>
                                </div>
                                <select id="wcdpi-shipping-label"
                                        class="wcdpi-select wcdpi-w-12"
                                        @change="onChangeLabel($event)"
                                        v-model="shippingLabel">

                                    <?php foreach (wcdpi_get_shipping_products_array(true, $destination) as $shippingProductGroup): ?>

                                        <optgroup label="<?php echo esc_attr($shippingProductGroup['label']); ?>">

                                            <?php
                                            /** @var ProductSalesResourceModel $shippingProduct */
                                            foreach ($shippingProductGroup['products'] as $shippingProduct):
                                                ?>

                                                <option value="<?php echo esc_attr($shippingProduct->getProdwsId()); ?>">
                                                    <?php echo esc_html($shippingProduct->getName()); ?>
                                                </option>

                                            <?php endforeach; ?>

                                        </optgroup>

                                    <?php endforeach; ?>

                                </select>
                            </div>

                            <div>
                                <div class="mb-1">
                                    <strong>
                                        <label>
                                            <?php _e('Harmonized Label, CN22', 'woo-dp-internetmarke'); ?>
                                        </label>
                                    </strong>
                                </div>
                                <input id="wcdpi-harmonized-label"
                                       type="checkbox"
                                       value="1"
                                       v-model="harmonizedLabel"
                                       @change="onChangeLabel($event)">
                                <label for="wcdpi-harmonized-label">
                                    <?php _e('Create a harmonized label (with or without customs declaration).'); ?>
                                </label>
                            </div>
                        </div>
                        <div class="preview p-1">

                            <div class="mb-1">
                                <strong>
                                    <label><?php _e('Preview', 'woo-dp-internetmarke'); ?></label>
                                </strong>
                            </div>
                            <div class="mb-1">
                                <img class="selected-label mb-1"
                                     style="max-width: 450px;"
                                     alt="<?php esc_attr_e('Select a product to preview its label before purchase.', 'woo-dp-internetmarke'); ?>"
                                     :src="label.previewUrl">
                                <div class="total mt-1">
                                    <div class="price" v-if="label.total === null">
                                        <?php echo wcdpi_format_price(0, '€'); ?>
                                    </div>
                                    <div class="price" v-if="label.total !== null" v-html="label.total"></div>
                                    <div>
                                        <small><?php esc_html_e('Total', 'woo-dp-internetmarke'); ?></small>
                                    </div>
                                </div>
                                <p><i><?php _e(sprintf('Please note, that the displayed total is for <b>%d shipping label(s)</b>.', count($orders))); ?></i></p>
                            </div>
                        </div>
                    </div>

                    <div class="d-f">
                        <div class="mb-1 wcdpi-w-2">
                            <strong>
                                <?php _e('Allowed dimensions', 'woo-dp-internetmarke'); ?>
                            </strong>
                            <p v-if="label.dimensions" v-html="label.dimensions"></p>
                        </div>
                        <div class="mb-1 wcdpi-w-10">
                            <strong>
                                <?php _e('Shipping product information', 'woo-dp-internetmarke'); ?>
                            </strong>
                            <p v-if="label.info" v-html="label.info"></p>
                        </div>
                    </div>

                    <div class="d-f">
                        <div>
                            <button class="action-button grey"
                                    @click="onSubmit('prepare')"
                                    :disabled="shippingLabel === null">
                                <span><?php _e('Prepare', 'woo-dp-internetmarke'); ?></span>
                            </button>
                            <button class="action-button yellow"
                                    @click="confirmationDialog = true"
                                    :disabled="shippingLabel === null">
                                <span><?php _e('Create', 'woo-dp-internetmarke'); ?></span>
                            </button>
                        </div>
                    </div>

                    <div class="wcdpi-loading" v-if="loading">
                        <img src="/wp-includes/images/spinner.gif"
                             alt="<?php esc_html_e('Loading...', 'woo-dp-internetmarke'); ?>">
                    </div>

                    <div class="dialog d-f" v-if="confirmationDialog === true">
                        <div class="content">
                            <p><?php _e('This action will deduct the displayed total from your Portokasse wallet and create the requested shipping labels. Are you sure?', 'woo-dp-internetmarke'); ?></p>
                            <button class="action-button red" @click="confirmationDialog = false">
                                <span><?php _e('No', 'woo-dp-internetmarke'); ?></span>
                            </button>
                            <button class="action-button yellow" @click="onSubmit('create')">
                                <span><?php _e('Yes', 'woo-dp-internetmarke'); ?></span>
                            </button>
                        </div>
                    </div>

                    <div id="wcdpi-progress-screen" class="dialog d-f" v-if="progressDialog === true">
                        <div class="progress-bar">
                            <h2 style="margin-top: 0;" v-if="orders.length !== progressValue">
                                <?php _e('Please wait...', 'woo-dp-internetmarke'); ?>
                            </h2>
                            <h2 style="margin-top: 0" v-if="orders.length === progressValue">
                                <?php _e('Completed', 'woo-dp-internetmarke'); ?>
                            </h2>
                            <progress :max="orders.length" :value="progressValue"></progress>
                        </div>

                        <div class="wcdpi-progress-table">
                            <div>
                                <div><strong><?php _e('Order', 'woo-dp-internetmarke'); ?></strong></div>
                                <div><strong><?php _e('Status', 'woo-dp-internetmarke'); ?></strong></div>
                                <div><strong><?php _e('Details', 'woo-dp-internetmarke'); ?></strong></div>
                                <div><strong><?php _e('Shipping label', 'woo-dp-internetmarke'); ?></strong></div>
                            </div>

                            <div v-for="(order, i) in orders" :key="'wcdpi-bulk-order-' + i">
                                <div><strong>#{{ order.number }}</strong></div>
                                <div>
                                    <span v-if="order.status === 'progress'" class="wcdpi-badge info">
                                        <?php _e('In progress', 'woo-dp-internetmarke'); ?>
                                    </span>
                                    <span v-if="order.status === 'success'" class="wcdpi-badge primary">
                                        <?php _e('Success', 'woo-dp-internetmarke'); ?>
                                    </span>
                                    <span v-if="order.status === 'error'" class="wcdpi-badge danger">
                                        <?php _e('Failed', 'woo-dp-internetmarke'); ?>
                                    </span>
                                    <span v-if="order.status === undefined" class="wcdpi-badge secondary">
                                        <?php _e('In queue', 'woo-dp-internetmarke'); ?>
                                    </span>
                                </div>
                                <div class="shipment-details">
                                    <pre class="m-0" v-if="order.details">{{ order.details }}</pre>
                                </div>
                                <div>
                                    <a :href="order.shippingLabelUrl" target="_blank" v-if="order.shippingLabelUrl">
                                        <?php _e('Download', 'woo-dp-internetmarke'); ?>
                                    </a>
                                </div>
                            </div>
                        </div>

                    </div>
                </div>
            </div>

        </div>

    </div>


</div>